@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -32,6 +32,8 @@
 #include "gdebug.h"
 #include "gxbitmap.h"
 #include "gsmchunk.h"
+#include "gxfont.h"
+#include "gxfont1.h"
 
 #include "stream.h"
 #include "gxiodev.h"            /* must come after stream.h */
@@ -54,6 +56,8 @@
 #include FT_IMAGE_H
 #include FT_BITMAP_H
 #include FT_TRUETYPE_DRIVER_H
+#include FT_MULTIPLE_MASTERS_H
+#include FT_TYPE1_TABLES_H
 
 /* Note: structure definitions here start with FF_, which stands for 'FAPI FreeType". */
 
@@ -802,6 +806,12 @@ load_glyph(gs_fapi_server * a_server, gs_fapi_font * a_fapi_font,
     }
 
     if (!a_fapi_font->metrics_only) {
+        /* The following works around the fact that at the scales we deal with
+         * these values may not fit in a 16.16 fixed point value, and thus cause
+         * freetype to error due to overflow - but we don't use these values
+         * and neither does freetype, we can set them to zero and avoid the error
+         */
+        ft_face->glyph->advance.x = ft_face->glyph->advance.y = 0;
         if ((!ft_error || !ft_error_fb) && a_glyph) {
             ft_error = FT_Get_Glyph(ft_face->glyph, a_glyph);
         }
@@ -964,13 +974,19 @@ make_rotation(FT_Matrix * a_transform, const FT_Vector * a_vector)
  */
 static void
 transform_decompose(FT_Matrix * a_transform, FT_UInt * xresp, FT_UInt * yresp,
-                    FT_Fixed * a_x_scale, FT_Fixed * a_y_scale)
+                    FT_Fixed * a_x_scale, FT_Fixed * a_y_scale, int units_per_EM)
 {
     double scalex, scaley, fact = 1.0;
     double factx = 1.0, facty = 1.0;
     FT_Matrix ftscale_mat;
     FT_UInt xres;
     FT_UInt yres;
+    /* We have to account for units_per_EM as we fiddle with the scaling
+     * in order to avoid underflow (mostly in the TTF hinting code), but
+     * we also want to clamp to a lower value (512, admittedly arrived at
+     * via experimentation) in order to preserve the fidelity of the outlines.
+     */
+    double upe = units_per_EM > 512 ? (float)units_per_EM : 512.0;
 
     scalex = hypot((double)a_transform->xx, (double)a_transform->xy);
     scaley = hypot((double)a_transform->yx, (double)a_transform->yy);
@@ -1059,7 +1075,7 @@ transform_decompose(FT_Matrix * a_transform, FT_UInt * xresp, FT_UInt * yresp,
 
     /* see above */
     fact = 1.0;
-    while (scaley * yres > 512.0 * 72.0 && (xres > 0 && yres > 0)
+    while (scaley * yres > (double)upe * 72.0 && (xres > 0 && yres > 0)
            && (scalex > 0.0 && scaley > 0.0)) {
         if (scaley < yres) {
             xres >>= 1;
@@ -1320,7 +1336,7 @@ gs_fapi_ft_get_scaled_font(gs_fapi_server * a_server, gs_fapi_font * a_font,
          * transform.
          */
         transform_decompose(&face->ft_transform, &face->horz_res,
-                            &face->vert_res, &face->width, &face->height);
+                            &face->vert_res, &face->width, &face->height, face->ft_face->units_per_EM);
 
         ft_error = FT_Set_Char_Size(face->ft_face, face->width, face->height,
                                     face->horz_res, face->vert_res);
@@ -1368,6 +1384,21 @@ gs_fapi_ft_get_scaled_font(gs_fapi_server * a_server, gs_fapi_font * a_font,
                  */
                 (void)FT_Select_Charmap(face->ft_face, ft_encoding_unicode);
             }
+            /* For PDF, we have to know which cmap table actually was selected */
+            if (face->ft_face->charmap != NULL) {
+                a_font->ttf_cmap_selected.platform_id = face->ft_face->charmap->platform_id;
+                a_font->ttf_cmap_selected.encoding_id = face->ft_face->charmap->encoding_id;
+            }
+            else {
+                /* Just in case */
+                a_font->ttf_cmap_selected.platform_id = -1;
+                a_font->ttf_cmap_selected.encoding_id = -1;
+            }
+        }
+        else {
+            /* Just in case */
+            a_font->ttf_cmap_selected.platform_id = -1;
+            a_font->ttf_cmap_selected.encoding_id = -1;
         }
     }
 
@@ -1752,6 +1783,42 @@ gs_fapi_ft_check_cmap_for_GID(gs_fapi_server * server, uint * index)
     return 0;
 }
 
+static gs_fapi_retcode
+gs_fapi_ft_set_mm_weight_vector(gs_fapi_server *server, gs_fapi_font *ff, float *wvector, int length)
+{
+#if defined(SHARE_FT) && SHARE_FT == 1 && \
+    FREETYPE_MAJOR <= 2 && FREETYPE_MINOR <= 9 && FREETYPE_PATCH <= 1
+
+    return gs_error_invalidaccess;
+#else
+    ff_face *face = (ff_face *) ff->server_font_data;
+    FT_Fixed nwv[16] = {0};
+    FT_Fixed cwv[16] = {0};
+    FT_UInt len = 16;
+    int i;
+    bool setit = false;
+    FT_Error ft_error;
+    (void)server;
+
+    ft_error = FT_Get_MM_WeightVector(face->ft_face, &len, cwv);
+    if (ft_error != 0) return_error(gs_error_invalidaccess);
+
+    for (i = 0; i < length; i++) {
+        nwv[i] = (FT_Fixed)(wvector[i] * 65536.0);
+        if (nwv[i] != cwv[i]) {
+            setit = true;
+        }
+    }
+    
+    if (setit == true) {
+        ft_error = FT_Set_MM_WeightVector(face->ft_face, length, nwv);
+        if (ft_error != 0) return_error(gs_error_invalidaccess);
+    }
+
+    return 0;
+#endif
+}
+
 static void gs_fapi_freetype_destroy(gs_fapi_server ** serv);
 
 static const gs_fapi_server_descriptor freetypedescriptor = {
@@ -1787,7 +1854,8 @@ static const gs_fapi_server freetypeserver = {
     gs_fapi_ft_release_char_data,
     gs_fapi_ft_release_typeface,
     gs_fapi_ft_check_cmap_for_GID,
-    NULL                        /* get_font_info */
+    NULL,                        /* get_font_info */
+    gs_fapi_ft_set_mm_weight_vector,
 };
 
 int gs_fapi_ft_init(gs_memory_t * mem, gs_fapi_server ** server);
