@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -18,6 +18,7 @@
 
 /* GS includes : */
 #include "stdio_.h"
+#include "string_.h"
 #include "stream.h"             /* for files.h */
 #include "strmio.h"
 
@@ -926,22 +927,10 @@ ufst_make_font_data(fapi_ufst_server * r, const char *font_file_path,
 #endif
     }
     else {
-        int sind = strlen(font_file_path) - 1;
+        int sind = strlen(font_file_path);
 
-        if ((font_file_path[sind] != 'o' || font_file_path[sind] != 'O') &&
-            (font_file_path[sind - 1] != 'c'
-             || font_file_path[sind - 1] != 'C')
-            && (font_file_path[sind - 2] != 'f'
-                || font_file_path[sind - 2] != 'F')
-            && font_file_path[sind - 3] != '.') {
-#if UFST_VERSION_MAJOR < 6
+        if (strncasecmp(font_file_path + sind - 4, ".fco", 4) != 0) {
             return (gs_error_invalidaccess);
-#else
-            /* If we have the Freetype server available, always use it for non-FCO fonts */
-            if (gs_fapi_available(r->mem, (char *)"FreeType")) {
-                return (gs_error_invalidaccess);
-            }
-#endif
         }
         area_length += strlen(font_file_path) + 1;
     }
@@ -2105,6 +2094,17 @@ gs_fapi_ufst_check_cmap_for_GID(gs_fapi_server * server, uint * index)
     return 0;
 }
 
+static gs_fapi_retcode
+gs_fapi_ufst_set_mm_weight_vector(gs_fapi_server *server, gs_fapi_font *ff, float *wvector, int length)
+{
+    (void)server;
+    (void)ff;
+    (void)wvector;
+    (void)length;
+    
+    return gs_error_invalidaccess;
+}
+
 /* --------------------- The plugin definition : ------------------------- */
 
 static void gs_fapi_ufst_destroy(gs_fapi_server ** server);
@@ -2142,7 +2142,8 @@ static const gs_fapi_server ufstserver = {
     gs_fapi_ufst_release_char_data,
     gs_fapi_ufst_release_typeface,
     gs_fapi_ufst_check_cmap_for_GID,
-    gs_fapi_ufst_get_font_info
+    gs_fapi_ufst_get_font_info,
+    gs_fapi_ufst_set_mm_weight_vector
 };
 
 int gs_fapi_ufst_init(gs_memory_t * mem, gs_fapi_server ** server);
@@ -2164,8 +2165,10 @@ gs_fapi_ufst_init(gs_memory_t * mem, gs_fapi_server ** server)
                                                       sizeof
                                                       (fapi_ufst_server),
                                                       "fapi_ufst_server");
-    if (serv == 0)
+    if (serv == NULL) {
+        gs_memory_chunk_release(cmem);
         return_error(gs_error_Fatal);
+    }
     memset(serv, 0, sizeof(*serv));
 
     serv->mem = cmem;
