@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 /* Default polygon and image drawing device procedures */
@@ -1010,9 +1010,6 @@ gx_default_begin_image(gx_device * dev,
     const gs_image_t *ptim;
     int code;
 
-    /* Processing an image object operation */
-    dev_proc(dev, set_graphics_type_tag)(dev, GS_IMAGE_TAG);
-
     set_dev_proc(dev, begin_image, gx_no_begin_image);
     if (pim->format == format)
         ptim = pim;
@@ -1035,10 +1032,6 @@ gx_default_begin_typed_image(gx_device * dev,
               const gx_drawing_color * pdcolor, const gx_clip_path * pcpath,
                       gs_memory_t * memory, gx_image_enum_common_t ** pinfo)
 {
-    /* Processing an image object operation */
-    if (pgs != NULL)   /* Null can happen when generating image3 mask */
-        dev_proc(dev, set_graphics_type_tag)(dev, GS_IMAGE_TAG);
-
     /* If this is an ImageType 1 image using the gs_gstate's CTM,
          * defer to begin_image.
          */
