@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 /* Example DeviceN process color model devices. */
@@ -268,7 +268,7 @@ devn_get_color_comp_index(gx_device * dev, gs_devn_params * pdevn_params,
 {
     int num_order = pdevn_params->num_separation_order_names;
     int color_component_number = 0;
-    int max_spot_colors = GX_DEVICE_MAX_SEPARATIONS - MAX_DEVICE_PROCESS_COLORS;
+    int max_spot_colors = GX_DEVICE_MAX_SEPARATIONS - pdevn_params->num_std_colorant_names;
 
     /*
      * Check if the component is in either the process color model list
@@ -318,15 +318,19 @@ devn_get_color_comp_index(gx_device * dev, gs_devn_params * pdevn_params,
      * Check if we have room for another spot colorant.
      */
     if (auto_spot_colors == ENABLE_AUTO_SPOT_COLORS)
-        max_spot_colors = dev->color_info.max_components -
-            pdevn_params->num_std_colorant_names;
+        /* limit max_spot_colors to what the device can handle given max_components */
+        max_spot_colors = min(max_spot_colors,
+                              dev->color_info.max_components - pdevn_params->num_std_colorant_names);
     if (pdevn_params->separations.num_separations < max_spot_colors) {
         byte * sep_name;
         gs_separations * separations = &pdevn_params->separations;
         int sep_num = separations->num_separations++;
         /* We have a new spot colorant - put in stable memory to avoid "restore" */
         sep_name = gs_alloc_bytes(dev->memory->stable_memory, name_size, "devn_get_color_comp_index");
-
+        if (sep_name == NULL) {
+            separations->num_separations--;	/* we didn't add it */
+            return -1;
+        }
         memcpy(sep_name, pname, name_size);
         separations->names[sep_num].size = name_size;
         separations->names[sep_num].data = sep_name;
@@ -360,14 +364,17 @@ devn_get_params(gx_device * pdev, gs_param_list * plist,
     gs_param_string_array scna;
     gs_param_string_array sona;
     gs_param_int_array equiv_cmyk;
-    int equiv_elements[5 * (GX_DEVICE_MAX_SEPARATIONS - MAX_DEVICE_PROCESS_COLORS)] = { 0 }; /* 5 * max_spot_colors */
+    /* there are 5 ints  per colorant in equiv_elements: a valid flag and an int for C, M, Y and K */
+    int equiv_elements[5 * GX_DEVICE_MAX_SEPARATIONS] = { 0 }; /* 5 * max_colors */
+    /* limit in case num_separations in pdevn_params exceeds what is expected. */
+    int num_separations = min(pdevn_params->separations.num_separations, sizeof(equiv_elements)/(5*sizeof(int)));
 
 
     set_param_array(scna, NULL, 0);
     set_param_array(sona, NULL, 0);
 
     if (pequiv_colors != NULL) {
-        for (spot_num = 0; spot_num < pdevn_params->separations.num_separations; spot_num++) {
+        for (spot_num = 0; spot_num < num_separations; spot_num++) {
             equiv_elements[i++] = pequiv_colors->color[spot_num].color_info_valid ? 1 : 0;
             equiv_elements[i++] = pequiv_colors->color[spot_num].c;
             equiv_elements[i++] = pequiv_colors->color[spot_num].m;
@@ -444,7 +451,7 @@ devn_put_params(gx_device * pdev, gs_param_list * plist,
     {
         break;
     } END_ARRAY_PARAM(sona, sone);
-    if (sona.data != 0 && sona.size > GX_DEVICE_COLOR_MAX_COMPONENTS) {
+    if (sona.data != 0 && sona.size > pdev->color_info.max_components) {
         param_signal_error(plist, "SeparationOrder", gs_error_rangecheck);
         return_error(gs_error_rangecheck);
     }
@@ -455,7 +462,7 @@ devn_put_params(gx_device * pdev, gs_param_list * plist,
     {
         break;
     } END_ARRAY_PARAM(scna, scne);
-    if (scna.data != 0 && scna.size > GX_DEVICE_MAX_SEPARATIONS) {
+    if (scna.data != 0 && scna.size > pdev->color_info.max_components) {
         param_signal_error(plist, "SeparationColorNames", gs_error_rangecheck);
         return_error(gs_error_rangecheck);
     }
@@ -465,6 +472,10 @@ devn_put_params(gx_device * pdev, gs_param_list * plist,
     {
         break;
     } END_ARRAY_PARAM(equiv_cmyk, equiv_cmyk_e);
+    if (equiv_cmyk.data != 0 && scna.size > pdev->color_info.max_components) {
+        param_signal_error(plist, "SeparationColorNames", gs_error_rangecheck);
+        return_error(gs_error_rangecheck);
+    }
 
     /* Separations are only valid with a subrtractive color model */
     if (pdev->color_info.polarity == GX_CINFO_POLARITY_SUBTRACTIVE) {
@@ -486,6 +497,10 @@ devn_put_params(gx_device * pdev, gs_param_list * plist,
                     /* We have a new separation */
                     sep_name = (byte *)gs_alloc_bytes(pdev->memory,
                         name_size, "devicen_put_params_no_sep_order");
+                    if (sep_name == NULL) {
+                        param_signal_error(plist, "SeparationColorNames", gs_error_VMerror);
+                        return_error(gs_error_VMerror);
+                    }
                     memcpy(sep_name, scna.data[i].data, name_size);
                     pdevn_params->separations.names[num_spot].size = name_size;
                     pdevn_params->separations.names[num_spot].data = sep_name;
@@ -495,9 +510,10 @@ devn_put_params(gx_device * pdev, gs_param_list * plist,
                         pequiv_colors->all_color_info_valid = false;
                     }
                     num_spot++;
+                    num_spot_changed = true;
                 }
             }
-            num_spot_changed = true;
+
             for (i = pdevn_params->separations.num_separations; i < num_spot; i++)
                 pdevn_params->separation_order_map[i + pdevn_params->num_std_colorant_names] =
                 i + pdevn_params->num_std_colorant_names;
@@ -591,8 +607,8 @@ devn_put_params(gx_device * pdev, gs_param_list * plist,
                     param_signal_error(plist, "PageSpotColors", gs_error_rangecheck);
                     return_error(gs_error_rangecheck);
                 }
-                if (page_spot_colors > GX_DEVICE_COLOR_MAX_COMPONENTS - MAX_DEVICE_PROCESS_COLORS)
-                    page_spot_colors = GX_DEVICE_COLOR_MAX_COMPONENTS - MAX_DEVICE_PROCESS_COLORS;
+                if (page_spot_colors > pdev->color_info.max_components - pdevn_params->num_std_colorant_names)
+                    page_spot_colors = pdev->color_info.max_components - pdevn_params->num_std_colorant_names;
                     /* Need to leave room for the process colors in GX_DEVICE_COLOR_MAX_COMPONENTS  */
         }
         /*
@@ -700,6 +716,9 @@ devn_copy_params(gx_device * psrcdev, gx_device * pdesdev)
         int name_size = src_devn_params->separations.names[k].size;
         sep_name = (byte *)gs_alloc_bytes(pdesdev->memory->stable_memory,
                                           name_size, "devn_copy_params");
+        if (sep_name == NULL) {
+            return_error(gs_error_VMerror);
+        }
         memcpy(sep_name, src_devn_params->separations.names[k].data, name_size);
         des_devn_params->separations.names[k].size = name_size;
         des_devn_params->separations.names[k].data = sep_name;
@@ -716,6 +735,9 @@ devn_copy_params(gx_device * psrcdev, gx_device * pdesdev)
         int name_size = src_devn_params->pdf14_separations.names[k].size;
         sep_name = (byte *)gs_alloc_bytes(pdesdev->memory->stable_memory,
                                           name_size, "devn_copy_params");
+        if (sep_name == NULL) {
+            return_error(gs_error_VMerror);
+        }
         memcpy(sep_name, src_devn_params->pdf14_separations.names[k].data,
                name_size);
         des_devn_params->pdf14_separations.names[k].size = name_size;
@@ -985,6 +1007,18 @@ gs_public_st_composite_final(st_gx_devn_prn_device, gx_devn_prn_device,
         gx_devn_prn_ret_devn_params     /* ret_devn_params */\
 }
 
+fixed_colorant_name DeviceGrayComponents[] = {
+        "Gray",
+        0              /* List terminator */
+};
+
+fixed_colorant_name DeviceRGBComponents[] = {
+        "Red",
+        "Green",
+        "Blue",
+        0              /* List terminator */
+};
+
 fixed_colorant_name DeviceCMYKComponents[] = {
         "Cyan",
         "Magenta",
@@ -1042,8 +1076,8 @@ const gx_devn_prn_device gs_devicen_device =
     gx_devn_prn_device_body(devicen_procs, "devicen", 4, GX_CINFO_POLARITY_SUBTRACTIVE, 32, 255, 255, "DeviceCMYK"),
     /* DeviceN device specific parameters */
     { 8,                        /* Bits per color - must match ncomp, depth, etc. above */
-      NULL,                     /* No names for standard DeviceN color model */
-      0,                        /* No standard colorants for DeviceN */
+      DeviceCMYKComponents,     /* Names of color model colorants */
+      4,                        /* Number colorants for CMYK */
       0,                        /* MaxSeparations has not been specified */
       -1,                       /* PageSpotColors has not been specified */
       {0},                      /* SeparationNames */
@@ -1328,7 +1362,7 @@ static int devn_write_pcx_file(gx_device_printer * pdev, char * filename, int nc
  * combinations of bits per pixel and number of colorants that are not supported.
  */
 static int
-spotcmyk_print_page(gx_device_printer * pdev, FILE * prn_stream)
+spotcmyk_print_page(gx_device_printer * pdev, gp_file * prn_stream)
 {
     int line_size = gdev_mem_bytes_per_scan_line((gx_device *) pdev);
     byte *in = gs_alloc_bytes(pdev->memory, line_size, "spotcmyk_print_page(in)");
@@ -1341,7 +1375,7 @@ spotcmyk_print_page(gx_device_printer * pdev, FILE * prn_stream)
     int bpc = pdevn->devn_params.bitspercomponent;
     int lnum = 0, bottom = pdev->height;
     int width = pdev->width;
-    FILE * spot_file[GX_DEVICE_COLOR_MAX_COMPONENTS] = {0};
+    gp_file * spot_file[GX_DEVICE_COLOR_MAX_COMPONENTS] = {0};
     uint i;
     int code = 0;
     int first_bit;
@@ -1370,7 +1404,11 @@ spotcmyk_print_page(gx_device_printer * pdev, FILE * prn_stream)
     /* Open the output files for the spot colors */
     for(i = 0; i < nspot; i++) {
         gs_sprintf(spotname, "%ss%d", pdevn->fname, i);
-        spot_file[i] = gp_fopen(spotname, "wb");
+        code = gs_add_control_path(pdev->memory, gs_permit_file_writing, spotname);
+        if (code < 0)
+            goto prn_done;
+        spot_file[i] = gp_fopen(pdev->memory, spotname, "wb");
+        (void)gs_remove_control_path(pdev->memory, gs_permit_file_writing, spotname);
         if (spot_file[i] == NULL) {
             code = gs_note_error(gs_error_VMerror);
             goto prn_done;
@@ -1379,24 +1417,26 @@ spotcmyk_print_page(gx_device_printer * pdev, FILE * prn_stream)
 
     /* Now create the output bit image files */
     for (; lnum < bottom; ++lnum) {
-        gdev_prn_get_bits(pdev, lnum, in, &data);
+        code = gdev_prn_get_bits(pdev, lnum, in, &data);
+        if (code < 0)
+            goto prn_done;
         /* Now put the pcm data into the output file */
         if (npcmcolors) {
             first_bit = bpc * (ncomp - npcmcolors);
             pcmlinelength = repack_data(data, buf, depth, first_bit, bpc * npcmcolors, width);
-            fwrite(buf, 1, pcmlinelength, prn_stream);
+            gp_fwrite(buf, 1, pcmlinelength, prn_stream);
         }
         /* Put spot color data into the output files */
         for (i = 0; i < nspot; i++) {
             first_bit = bpc * (nspot - 1 - i);
             linelength[i] = repack_data(data, buf, depth, first_bit, bpc, width);
-            fwrite(buf, 1, linelength[i], spot_file[i]);
+            gp_fwrite(buf, 1, linelength[i], spot_file[i]);
         }
     }
 
     /* Close the bit image files */
     for(i = 0; i < nspot; i++) {
-        fclose(spot_file[i]);
+        gp_fclose(spot_file[i]);
         spot_file[i] = NULL;
     }
 
@@ -1418,7 +1458,7 @@ spotcmyk_print_page(gx_device_printer * pdev, FILE * prn_stream)
   prn_done:
     for(i = 0; i < nspot; i++) {
         if (spot_file[i] != NULL)
-            fclose(spot_file[i]);
+            gp_fclose(spot_file[i]);
     }
     if (in != NULL)
         gs_free_object(pdev->memory, in, "spotcmyk_print_page(in)");
@@ -1493,9 +1533,9 @@ static const pcx_header pcx_header_prototype =
 };
 
 /* Forward declarations */
-static void devn_pcx_write_rle(const byte *, const byte *, int, FILE *);
-static int devn_pcx_write_page(gx_device_printer * pdev, FILE * infile,
-    int linesize, FILE * outfile, pcx_header * phdr, bool planar, int depth);
+static void devn_pcx_write_rle(const byte *, const byte *, int, gp_file *);
+static int devn_pcx_write_page(gx_device_printer * pdev, gp_file * infile,
+    int linesize, gp_file * outfile, pcx_header * phdr, bool planar, int depth);
 
 static const byte pcx_cmyk_palette[16 * 3] =
 {
@@ -1626,7 +1666,7 @@ devn_setup_pcx_header(gx_device_printer * pdev, pcx_header * phdr, int num_plane
 
 /* Write a palette on a file. */
 static int
-pc_write_mono_palette(gx_device * dev, uint max_index, FILE * file)
+pc_write_mono_palette(gx_device * dev, uint max_index, gp_file * file)
 {
     uint i, c;
     gx_color_value rgb[3];
@@ -1636,7 +1676,7 @@ pc_write_mono_palette(gx_device * dev, uint max_index, FILE * file)
         for (c = 0; c < 3; c++) {
             byte b = gx_color_value_to_byte(rgb[c]);
 
-            fputc(b, file);
+            gp_fputc(b, file);
         }
     }
     return 0;
@@ -1656,7 +1696,7 @@ pc_write_mono_palette(gx_device * dev, uint max_index, FILE * file)
  *   num_planes - The number of planes.
  */
 static int
-devn_finish_pcx_file(gx_device_printer * pdev, FILE * file, pcx_header * header, int num_planes, int bits_per_plane)
+devn_finish_pcx_file(gx_device_printer * pdev, gp_file * file, pcx_header * header, int num_planes, int bits_per_plane)
 {
     switch (num_planes) {
         case 1:
@@ -1670,7 +1710,7 @@ devn_finish_pcx_file(gx_device_printer * pdev, FILE * file, pcx_header * header,
                 case 5:                         /* Not defined */
                         break;
                 case 8:
-                        fputc(0x0c, file);
+                        gp_fputc(0x0c, file);
                         return pc_write_mono_palette((gx_device *) pdev, 256, file);
                 case 16:                        /* Not defined */
                         break;
@@ -1737,37 +1777,49 @@ devn_write_pcx_file(gx_device_printer * pdev, char * filename, int ncomp,
     int code;
     bool planar;
     char *outname = (char *)gs_alloc_bytes(pdev->memory, gp_file_name_sizeof, "devn_write_pcx_file(outname)");
-    FILE * in = NULL;
-    FILE * out = NULL;
+    gp_file * in = NULL;
+    gp_file * out = NULL;
     int depth = bpc_to_depth(ncomp, bpc);
 
     if (outname == NULL) {
         code = gs_note_error(gs_error_VMerror);
-	goto done;
+        goto done;
     }
 
-    in = gp_fopen(filename, "rb");
+    code = gs_add_control_path(pdev->memory, gs_permit_file_reading, filename);
+    if (code < 0)
+        goto done;
+
+    in = gp_fopen(pdev->memory, filename, "rb");
     if (!in) {
         code = gs_note_error(gs_error_invalidfileaccess);
-	goto done;
+        goto done;
     }
     gs_sprintf(outname, "%s.pcx", filename);
-    out = gp_fopen(outname, "wb");
+    code = gs_add_control_path(pdev->memory, gs_permit_file_writing, outname);
+    if (code < 0)
+        goto done;
+    out = gp_fopen(pdev->memory, outname, "wb");
     if (!out) {
         code = gs_note_error(gs_error_invalidfileaccess);
-	goto done;
+        goto done;
     }
 
+    if (ncomp == 4 && bpc == 8) {
+        ncomp = 3;		/* we will convert 32-bit to 24-bit RGB */
+    }
     planar = devn_setup_pcx_header(pdev, &header, ncomp, bpc);
     code = devn_pcx_write_page(pdev, in, linesize, out, &header, planar, depth);
     if (code >= 0)
         code = devn_finish_pcx_file(pdev, out, &header, ncomp, bpc);
 
 done:
+    (void)gs_remove_control_path(pdev->memory, gs_permit_file_reading, filename);
+    (void)gs_remove_control_path(pdev->memory, gs_permit_file_writing, outname);
     if (in)
-      fclose(in);
+      gp_fclose(in);
     if (out)
-      fclose(out);
+      gp_fclose(out);
 
     if (outname)
       gs_free_object(pdev->memory, outname, "spotcmyk_print_page(outname)");
@@ -1779,7 +1831,7 @@ done:
 /* This routine is used for all formats. */
 /* The caller has set header->bpp, nplanes, and palette. */
 static int
-devn_pcx_write_page(gx_device_printer * pdev, FILE * infile, int linesize, FILE * outfile,
+devn_pcx_write_page(gx_device_printer * pdev, gp_file * infile, int linesize, gp_file * outfile,
                pcx_header * phdr, bool planar, int depth)
 {
     int raster = linesize;
@@ -1787,12 +1839,22 @@ devn_pcx_write_page(gx_device_printer * pdev, FILE * infile, int linesize, FILE
     int height = pdev->height;
     uint lsize = raster + rsize;
     byte *line = gs_alloc_bytes(pdev->memory, lsize, "pcx file buffer");
+    byte *rgb_buff = NULL;
     byte *plane = line + raster;
+    bool convert_to_rgb = false;
     int y;
     int code = 0;               /* return code */
 
     if (line == 0)              /* can't allocate line buffer */
         return_error(gs_error_VMerror);
+    if (pdev->color_info.num_components == 4 && depth == 32) {
+        rgb_buff = gs_alloc_bytes(pdev->memory, lsize, "pcx_rgb_buff");
+        if (rgb_buff == 0)              /* can't allocate line buffer */
+            return_error(gs_error_VMerror);
+        raster = (raster * 3) / 4;	/* will be rounded up to even later */
+        depth = 24;			/* we will be writing 24-bit rgb */
+        convert_to_rgb = true;
+    }
 
     /* Fill in the other variable entries in the header struct. */
 
@@ -1805,7 +1867,7 @@ devn_pcx_write_page(gx_device_printer * pdev, FILE * infile, int linesize, FILE
 
     /* Write the header. */
 
-    if (fwrite((const char *)phdr, 1, 128, outfile) < 128) {
+    if (gp_fwrite((const char *)phdr, 1, 128, outfile) < 128) {
         code = gs_error_ioerror;
         goto pcx_done;
     }
@@ -1814,9 +1876,24 @@ devn_pcx_write_page(gx_device_printer * pdev, FILE * infile, int linesize, FILE
         byte *row = line;
         byte *end;
 
-        code = fread(line, sizeof(byte), linesize, infile);
+        code = gp_fread(line, sizeof(byte), linesize, infile);
         if (code < 0)
             break;
+        /* If needed, convert to rgb */
+        if (convert_to_rgb) {
+            int i;
+			byte *row_in = line;
+
+            /* Transform the data. */
+            row = rgb_buff;	/* adjust to converted output buffer */
+            for (i=0; i < linesize; i += 4) {
+                *row++ = ((255 - row_in[0]) * (255 - row_in[3])) / 255;
+                *row++ = ((255 - row_in[1]) * (255 - row_in[3])) / 255;
+                *row++ = ((255 - row_in[2]) * (255 - row_in[3])) / 255;
+                row_in += 4;
+            }
+            row = rgb_buff;	/* adjust to converted output buffer */
+        }
         end = row + raster;
         if (!planar) {          /* Just write the bits. */
             if (raster & 1) {   /* Round to even, with predictable padding. */
@@ -1865,7 +1942,7 @@ devn_pcx_write_page(gx_device_printer * pdev, FILE * infile, int linesize, FILE
                         for (pnum = 0; pnum < 3; ++pnum) {
                             devn_pcx_write_rle(row + pnum, row + raster, 3, outfile);
                             if (pdev->width & 1)
-                                fputc(0, outfile);              /* pad to even */
+                                gp_fputc(0, outfile);              /* pad to even */
                         }
                     }
                     break;
@@ -1879,6 +1956,8 @@ devn_pcx_write_page(gx_device_printer * pdev, FILE * infile, int linesize, FILE
     }
 
   pcx_done:
+    if (rgb_buff != NULL)
+        gs_free_object(pdev->memory, rgb_buff, "pcx_rgb_buff");
     gs_free_object(pdev->memory, line, "pcx file buffer");
 
     return code;
@@ -1888,7 +1967,7 @@ devn_pcx_write_page(gx_device_printer * pdev, FILE * infile, int linesize, FILE
 
 /* Write one line in PCX run-length-encoded format. */
 static void
-devn_pcx_write_rle(const byte * from, const byte * end, int step, FILE * file)
+devn_pcx_write_rle(const byte * from, const byte * end, int step, gp_file * file)
 {  /*
     * The PCX format theoretically allows encoding runs of 63
     * identical bytes, but some readers can't handle repetition
@@ -1903,7 +1982,7 @@ devn_pcx_write_rle(const byte * from, const byte * end, int step, FILE * file)
         from += step;
         if (data != *from || from == end) {
             if (data >= 0xc0)
-                putc(0xc1, file);
+                gp_fputc(0xc1, file);
         } else {
             const byte *start = from;
 
@@ -1911,14 +1990,14 @@ devn_pcx_write_rle(const byte * from, const byte * end, int step, FILE * file)
                 from += step;
             /* Now (from - start) / step + 1 is the run length. */
             while (from - start >= max_run) {
-                putc(0xc0 + MAX_RUN_COUNT, file);
-                putc(data, file);
+                gp_fputc(0xc0 + MAX_RUN_COUNT, file);
+                gp_fputc(data, file);
                 start += max_run;
             }
             if (from > start || data >= 0xc0)
-                putc((from - start) / step + 0xc1, file);
+                gp_fputc((from - start) / step + 0xc1, file);
         }
-        putc(data, file);
+        gp_fputc(data, file);
     }
 #undef MAX_RUN_COUNT
 }
