@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -80,6 +80,8 @@ static dev_proc_strip_copy_rop2(epo_strip_copy_rop2);
 static dev_proc_copy_planes(epo_copy_planes);
 static dev_proc_copy_alpha_hl_color(epo_copy_alpha_hl_color);
 static dev_proc_process_page(epo_process_page);
+static dev_proc_transform_pixel_region(epo_transform_pixel_region);
+static dev_proc_fill_stroke_path(epo_fill_stroke_path);
 
 /* The device prototype */
 #define MAX_COORD (max_int_in_fixed - 1000)
@@ -182,7 +184,9 @@ gx_device_epo gs_epo_device =
      epo_strip_copy_rop2,
      default_subclass_strip_tile_rect_devn,
      epo_copy_alpha_hl_color,
-     epo_process_page
+     epo_process_page,
+     epo_transform_pixel_region,
+     epo_fill_stroke_path,
     }
 };
 
@@ -208,7 +212,7 @@ is_device_installed(gx_device *dev, const char *name)
 static bool
 device_wants_optimization(gx_device *dev)
 {
-    return (dev_proc(dev, fillpage) == gx_default_fillpage);
+    return (!gs_is_null_device(dev) && dev_proc(dev, fillpage) == gx_default_fillpage);
 }
 
 /* Use this when debugging to enable/disable epo
@@ -466,7 +470,7 @@ int epo_fill_linear_color_triangle(gx_device *dev, const gs_fill_attributes *fa,
     return dev_proc(dev, fill_linear_color_triangle)(dev, fa, p0, p1, p2, c0, c1, c2);
 }
 
-int epo_put_image(gx_device *dev, const byte **buffers, int num_chan, int x, int y,
+int epo_put_image(gx_device *dev, gx_device *mdev, const byte **buffers, int num_chan, int x, int y,
             int width, int height, int row_stride,
             int alpha_plane_index, int tag_plane_index)
 {
@@ -474,7 +478,7 @@ int epo_put_image(gx_device *dev, const byte **buffers, int num_chan, int x, int
 
     if (code != 0)
         return code;
-    return dev_proc(dev, put_image)(dev, buffers, num_chan, x, y, width, height, row_stride, alpha_plane_index, tag_plane_index);
+    return dev_proc(dev, put_image)(dev, mdev, buffers, num_chan, x, y, width, height, row_stride, alpha_plane_index, tag_plane_index);
 }
 
 int epo_create_compositor(gx_device *dev, gx_device **pcdev, const gs_composite_t *pcte,
@@ -676,3 +680,25 @@ int epo_process_page(gx_device *dev, gx_process_page_options_t *options)
         return code;
     return dev_proc(dev, process_page)(dev, options);
 }
+
+int epo_fill_stroke_path(gx_device *dev, const gs_gstate *pgs, gx_path *ppath,
+    const gx_fill_params *fill_params, const gx_drawing_color *pdcolor_fill,
+    const gx_stroke_params *stroke_params, const gx_drawing_color *pdcolor_stroke,
+    const gx_clip_path *pcpath)
+{
+    int code = epo_handle_erase_page(dev);
+
+    if (code != 0)
+        return code;
+    return dev_proc(dev, fill_stroke_path)(dev, pgs, ppath, fill_params, pdcolor_fill,
+                                           stroke_params, pdcolor_stroke, pcpath);
+}
+
+int epo_transform_pixel_region(gx_device *dev, transform_pixel_region_reason reason, transform_pixel_region_data *data)
+{
+    int code = epo_handle_erase_page(dev);
+
+    if (code != 0)
+        return code;
+    return dev_proc(dev, transform_pixel_region)(dev, reason, data);
+}
