@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -151,7 +151,9 @@ gx_device_mplt gs_pcl_mono_palette_device =
      default_subclass_strip_copy_rop2,
      default_subclass_strip_tile_rect_devn,
      default_subclass_copy_alpha_hl_color,
-     default_subclass_process_page
+     default_subclass_process_page,
+     default_subclass_transform_pixel_region,
+     default_subclass_fill_stroke_path,
     }
 };
 
