@@ -1,16 +1,18 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
-  
+
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
-/* $Id: gdevp14.h 11947 2010-12-10 19:50:53Z mvrhel $ */
+
 /* Definitions and interface for PDF 1.4 rendering device */
 
 #ifndef gdevp14_INCLUDED
@@ -22,6 +24,7 @@
 #include "gxdcolor.h"
 #include "gxpcolor.h"
 #include "gdevdevn.h"
+#include "gsgstate.h"
 
 typedef enum {
     PDF14_DeviceGray = 0,
@@ -31,10 +34,11 @@ typedef enum {
     PDF14_DeviceCustom = 4
 } pdf14_default_colorspace_t;
 
-#ifndef pdf14_buf_DEFINED
-#  define pdf14_buf_DEFINED
-typedef struct pdf14_buf_s pdf14_buf;
-#endif
+typedef enum {
+    PDF14_OP_STATE_NONE = 0,
+    PDF14_OP_STATE_FILL = 1,
+    PDF14_OP_STATE_STROKE = 2,
+} PDF14_OP_FS_STATE;
 
 /*
  * This structure contains procedures for processing routine which differ
@@ -50,19 +54,22 @@ typedef struct {
      * execution time.
      */
     void (* unpack_color)(int num_comp, gx_color_index color,
-			       	pdf14_device * p14dev, byte * out);
+                                pdf14_device * p14dev, byte * out);
     /*
      * This procedure sends the final rasterized transparency data to the
      * output device as an image.
      */
     int (* put_image)(gx_device * dev,
-		    gs_imager_state * pis, gx_device * target);
-} pdf14_procs_s;
+                      gs_gstate * pgs, gx_device * target);
 
+    /* And a 16 bit variant */
+    void (* unpack_color16)(int num_comp, gx_color_index color,
+                            pdf14_device * p14dev, uint16_t * out);
+} pdf14_procs_s;
 
 typedef pdf14_procs_s pdf14_procs_t;
 
-/* A stack structure for the softmask buffers. 
+/* A stack structure for the softmask buffers.
    The mask will be pdf14 buffers that are wrapped
    in a refernce counted structure.  We need this to
    be referenced counted since we need to be able to push
@@ -77,7 +84,7 @@ struct pdf14_rcmask_s {
     pdf14_buf   *mask_buf;
     rc_header rc;
     gs_memory_t *memory;
-     
+
 };
 
 typedef struct pdf14_mask_s pdf14_mask_t;
@@ -87,9 +94,8 @@ struct pdf14_mask_s {
     pdf14_rcmask_t *rc_mask;
     pdf14_mask_t *previous;
     gs_memory_t *memory;
- 
-};
 
+};
 
 /* A structure to hold information
  * about the parent color related
@@ -103,7 +109,6 @@ struct pdf14_mask_s {
 typedef struct pdf14_parent_color_s pdf14_parent_color_t;
 
 struct pdf14_parent_color_s {
-
     int num_components;
     bool isadditive;
     gx_color_polarity_t polarity;
@@ -112,31 +117,34 @@ struct pdf14_parent_color_s {
     byte depth;  /* used in clist writer cmd_put_color */
     uint max_gray;  /* Used to determine if device halftones */
     uint max_color; /* Causes issues if these are not maintained */
-    const gx_color_map_procs *(*get_cmap_procs)(const gs_imager_state *,
-						     const gx_device *);
+    const gx_color_map_procs *(*get_cmap_procs)(const gs_gstate *,
+                                                     const gx_device *);
     const gx_cm_color_map_procs *(*parent_color_mapping_procs)(const gx_device *);
-    int (*parent_color_comp_index)(gx_device *, const char *, int, int); 
+    gx_color_index (*encode)(gx_device *, const gx_color_value value[]);
+    int (*decode)(gx_device *, gx_color_index, gx_color_value *);
+    int (*parent_color_comp_index)(gx_device *, const char *, int, int);
     const pdf14_procs_t * unpack_procs;
     const pdf14_nonseparable_blending_procs_t * parent_blending_procs;
     cmm_profile_t *icc_profile;  /* Opaque to GC.  Allocated in non-gc memory */
     pdf14_parent_color_t *previous;
- 
 };
 
 typedef struct pdf14_ctx_s pdf14_ctx;
 
 struct pdf14_buf_s {
     pdf14_buf *saved;
-
+    byte *backdrop;  /* This is needed for proper non-isolated knockout support */
     bool isolated;
     bool knockout;
-    byte alpha;
-    byte shape;
+    uint16_t alpha;
+    uint16_t shape;
     gs_blend_mode_t blend_mode;
-
+    int num_spots;  /* helpful when going between Gray+spots, RGB+spots, CMYK+spots */
     bool has_alpha_g;
     bool has_shape;
     bool has_tags;
+    bool deep; /* false => 8 bits, true => 16 bits */
+    bool page_group;
 
     gs_int_rect rect;
     /* Note: the traditional GS name for rowstride is "raster" */
@@ -146,20 +154,24 @@ struct pdf14_buf_s {
 
     int rowstride;
     int planestride;
-    int n_chan; /* number of pixel planes including alpha */
+    int n_chan;   /* number of pixel planes including alpha */
     int n_planes; /* total number of planes including alpha, shape, alpha_g */
     byte *data;
     byte *transfer_fn;
-    gs_int_rect bbox;
-    pdf14_mask_t *maskbuf;
+    bool is_ident;
+    int matte_num_comps;
+    uint16_t *matte;
+    gs_int_rect dirty;
+    pdf14_mask_t *mask_stack;
     bool idle;
 
     gs_transparency_mask_subtype_t SMask_SubType;
 
     uint mask_id;
-    pdf14_parent_color_t *parent_color_info_procs;
+    pdf14_parent_color_t *parent_color_info;
 
     gs_transparency_color_t color_space;  /* Different groups can have different spaces for blending */
+    gs_memory_t *memory;
 };
 
 typedef struct pdf14_smaskcolor_s {
@@ -169,41 +181,17 @@ typedef struct pdf14_smaskcolor_s {
 
 struct pdf14_ctx_s {
     pdf14_buf *stack;
-    pdf14_mask_t *maskbuf;
+    pdf14_mask_t *mask_stack;
     gs_memory_t *memory;
     gs_int_rect rect;
     bool additive;
     int n_chan;
     int smask_depth;  /* used to catch smasks embedded in smasks.  bug691803 */
-    bool smask_blend; 
+    bool smask_blend;
+    bool deep; /* If true, 16 bit data, false, 8 bit data. */
 };
 
-#ifndef gs_devn_params_DEFINED
-#  define gs_devn_params_DEFINED
-typedef struct gs_devn_params_s gs_devn_params;
-#endif
-
-#ifndef gs_imager_state_DEFINED
-#  define gs_imager_state_DEFINED
-typedef struct gs_imager_state_s gs_imager_state;
-#endif
-
-#ifndef gx_device_DEFINED
-#  define gx_device_DEFINED
-typedef struct gx_device_s gx_device;
-#endif
-
-#ifndef gs_pdf14trans_params_DEFINED
-#  define gs_pdf14trans_params_DEFINED
 typedef struct gs_pdf14trans_params_s gs_pdf14trans_params_t;
-#endif
-
-#ifndef pdf14_device_DEFINED
-#  define pdf14_device_DEFINED
-typedef struct pdf14_device_s pdf14_device;
-#endif
-
-
 
 /*
  * Define the default post-clist (clist reader) PDF 1.4 compositing device.
@@ -216,20 +204,58 @@ typedef struct pdf14_device_s {
     gs_devn_params devn_params;    /* Must follow gx_device_forward_common */
     const pdf14_procs_t * pdf14_procs;	   /* Must follow devn_params. */
     const pdf14_nonseparable_blending_procs_t * blend_procs; /* Must follow pdf14_procs */
+    int num_std_colorants;
 
     pdf14_ctx *ctx;
     pdf14_smaskcolor_t *smaskcolor;
     float opacity;
     float shape;
     float alpha; /* alpha = opacity * shape */
+    float fillconstantalpha;
+    float strokeconstantalpha;
     gs_blend_mode_t blend_mode;
     bool text_knockout;
-	bool overprint;
-	bool overprint_mode;
-	gx_color_index drawn_comps;		/* Used for overprinting.  Passed from overprint compositor */
+    bool overprint;
+    bool effective_overprint_mode;
+    bool stroke_effective_op_mode;
+    bool stroke_overprint;
+    int text_group;
+    gx_color_index drawn_comps;		/* Used for overprinting.  Passed from overprint compositor */
+    gx_color_index drawn_comps_fill;		/* selected by color_is_fill */
+    gx_color_index drawn_comps_stroke;
+    PDF14_OP_FS_STATE op_state;
     gx_device * pclist_device;
-    const gx_color_map_procs *(*save_get_cmap_procs)(const gs_imager_state *,
-						     const gx_device *);
+    bool free_devicen;              /* Used to avoid freeing a deviceN parameter from target clist device */
+    bool sep_device;
+    bool using_blend_cs;
+
+    /* We now have some variables to help us determine whether
+     * we are in an SMask or not. Firstly, we have in_smask_construction,
+     * initially 0, incremented whenever we begin smask in the compositor,
+     * and decremented whenever we finish one. Thus this being non-zero
+     * implies we are in the 'construction' phase of at least one smask.
+     */
+    int in_smask_construction;
+    /* Next, we have smask_constructed. Again, initially 0. This is set
+     * to 1 whenever in_smask_construction returns to 0. It is reset to
+     * 0 on the next transparency group push (which takes possession of
+     * the smask that was just created. Thus this being 1 implies
+     * we have just constructed an smask, but not used it yet. */
+    int smask_constructed;
+    /* Finally, then we have depth_within_smask. This is used to keep the
+     * count of how many times we need to pop a transparency group to
+     * finish being governed by an smask. To keep this figure, we watch
+     * pushes of new transparency groups. On each such push, if
+     * smask_constructed is true, or depth_within_smask is already non-zero,
+     * we increment depth_within_smask. (i.e. whenever we start a group
+     * that will be governed by (and take possession of) an smask, or whenever
+     * we are already being governed by an smask, we know that we've just
+     * increased our nested depth by 1). Whenever we pop a group and it is
+     * non zero, we decrement it.
+     */
+    int depth_within_smask;
+    const gx_color_map_procs *(*save_get_cmap_procs)(const gs_gstate *,
+                                                     const gx_device *);
     gx_device_color_info saved_target_color_info;
     dev_proc_encode_color(*saved_target_encode_color);
     dev_proc_decode_color(*saved_target_decode_color);
@@ -239,7 +265,7 @@ typedef struct pdf14_device_s {
     dev_proc_decode_color(*my_decode_color);
     dev_proc_get_color_mapping_procs(*my_get_color_mapping_procs);
     dev_proc_get_color_comp_index(*my_get_color_comp_index);
-    
+
     pdf14_parent_color_t *trans_group_parent_cmap_procs;
 
 } pdf14_device_t;
@@ -255,7 +281,7 @@ typedef	struct pdf14_device_s pdf14_clist_device;
 /*
  * Send a PDF 1.4 transparency compositor action to the specified device.
  */
-int send_pdf14trans(gs_imager_state * pis, gx_device * dev,
+int send_pdf14trans(gs_gstate * pgs, gx_device * dev,
     gx_device * * pcdev, gs_pdf14trans_params_t * pparams, gs_memory_t * mem);
 
 /*
@@ -270,15 +296,24 @@ int send_pdf14trans(gs_imager_state * pis, gx_device * dev,
  */
 int
 pdf14_put_devn_params(gx_device * pdev, gs_devn_params * pdevn_params,
-	       				gs_param_list * plist);
+                                        gs_param_list * plist);
 
 /* Used to passed along information about the buffer created by the
    pdf14 device.  This is used by the pattern accumulator when the
    pattern contains transparency */
-
-int pdf14_get_buffer_information(const gx_device * dev, gx_pattern_trans_t *transbuff);
+int pdf14_get_buffer_information(const gx_device * dev,
+                                 gx_pattern_trans_t *transbuff, gs_memory_t *mem,
+                                 bool free_device);
 
 /* Not static due to call from pattern logic */
 int pdf14_disable_device(gx_device * dev);
 
+/* Needed so that we can set the monitoring in the target device */
+int gs_pdf14_device_color_mon_set(gx_device *pdev, bool monitoring);
+
+/* When playing back the clist, we need to know if the buffer device is compatible */
+/* with the pdf14 compositor that was used when writing the clist. Colorspace and  */
+/* depth are critical since these must match when reading back colors.             */
+bool pdf14_ok_to_optimize(gx_device *bdev);
+
 #endif /* gdevp14_INCLUDED */
