@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -244,8 +244,8 @@ void
 gdev_vector_init(gx_device_vector * vdev)
 {
     gdev_vector_reset(vdev);
-    if (vdev->procs.dev_spec_op == NULL || vdev->procs.dev_spec_op == gx_default_dev_spec_op)
-        vdev->procs.dev_spec_op = gdev_vector_dev_spec_op;
+    if (dev_proc(vdev, dev_spec_op) == gx_default_dev_spec_op)
+        set_dev_proc(vdev, dev_spec_op, gdev_vector_dev_spec_op);
 
     vdev->scale.x = vdev->scale.y = 1.0;
     vdev->in_page = false;
@@ -630,6 +630,11 @@ gdev_vector_dopath_init(gdev_vector_dopath_state_t *state,
         gs_make_scaling(vdev->scale.x, vdev->scale.y, &state->scale_mat);
     }
     state->first = true;
+
+    /* This is purely to prevent Coverity from thinking gdev_vector_dopath()
+    could use uninitialised state->start.x. */
+    state->start.x = 0;
+    state->start.y = 0;
 }
 
 /*
@@ -826,7 +831,7 @@ gdev_vector_update_clip_path(gx_device_vector * vdev,
 int
 gdev_vector_close_file(gx_device_vector * vdev)
 {
-    FILE *f = vdev->file;
+    gp_file *f = vdev->file;
     int err;
 
     if (vdev->dash_pattern) {
@@ -850,7 +855,7 @@ gdev_vector_close_file(gx_device_vector * vdev)
     }
     vdev->file = 0;
     if (f) {
-        err = ferror(f);
+        err = gp_ferror(f);
         /* We prevented sclose from closing the file. */
         if (gx_device_close_output_file((gx_device *)vdev, vdev->fname, f) != 0
                 || err != 0)
