@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -26,9 +26,9 @@ const char gp_file_name_list_separator = ':';
 /* Define the string to be concatenated with the file mode */
 /* for opening files without end-of-line conversion. */
 #if (defined(__MINGW32__) && __MINGW32__ == 1) || (defined(__CYGWIN__) && __CYGWIN__ == 1)
-const char gp_fmode_binary_suffix[] = "b";
+const char* gp_fmode_binary_suffix = "b";
 #else
-const char gp_fmode_binary_suffix[] = "";
+const char* gp_fmode_binary_suffix = "";
 #endif
 
 
