@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,16 +9,39 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
-
-/* Prototypes for configuration definitions in gconfig.c. */
-
 #ifndef gscdefs_INCLUDED
 #  define gscdefs_INCLUDED
 
+
+#define GS_STRINGIZE2(s) #s
+#define GS_STRINGIZE(s) GS_STRINGIZE2(s)
+
+#ifndef GS_BUILDTIME
+#  define GS_BUILDTIME\
+        0			/* should be set in the makefile */
+#endif
+
+#ifndef GS_COPYRIGHT
+#  define GS_COPYRIGHT\
+        "Copyright (C) 2020 Artifex Software, Inc.  All rights reserved."
+#endif
+
+#ifndef GS_PRODUCTFAMILY
+#  define GS_PRODUCTFAMILY\
+        "GPL Ghostscript"
+#endif
+
+#ifndef GS_PRODUCT
+#  define GS_PRODUCT\
+        GS_PRODUCTFAMILY ""
+#endif
+
+/* Prototypes for configuration definitions in gconfig.c. */
+
 /*
  * This file may be #included in places that don't even have stdpre.h,
  * so it mustn't use any Ghostscript definitions in any code that is
@@ -76,4 +99,7 @@ extern const unsigned gx_io_device_table_count;
 #define extern_gs_find_compositor() \
   const gs_composite_type_t * gs_find_compositor(int comp_id)
 
+#define extern_gs_get_fapi_server_inits() \
+  const gs_fapi_server_init_func * gs_get_fapi_server_inits(void)
+
 #endif /* gscdefs_INCLUDED */
