@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -49,7 +49,7 @@
 /* GC descriptors */
 gs_private_st_composite(st_color_space_DeviceN, gs_color_space,
      "gs_color_space_DeviceN", cs_DeviceN_enum_ptrs, cs_DeviceN_reloc_ptrs);
-private_st_device_n_attributes();
+private_st_device_n_colorant();
 private_st_device_n_map();
 
 /* Define the DeviceN color space type. */
@@ -83,20 +83,12 @@ const gs_color_space_type gs_color_space_type_DeviceN = {
 
 static
 ENUM_PTRS_BEGIN(cs_DeviceN_enum_ptrs)
-{
-    gs_device_n_params *params = &((gs_color_space *)vptr)->params.device_n;
-    if (index-3 < params->num_components)
-        return ENUM_NAME_INDEX(params->names[index-3]);
-    else
-        return 0;
-}
-ENUM_PTR(0, gs_color_space, params.device_n.names);
-ENUM_PTR(1, gs_color_space, params.device_n.map);
-ENUM_PTR(2, gs_color_space, params.device_n.colorants);
+    return 0;
+ENUM_PTR(0, gs_color_space, params.device_n.map);
+ENUM_PTR(1, gs_color_space, params.device_n.colorants);
 ENUM_PTRS_END
 static RELOC_PTRS_BEGIN(cs_DeviceN_reloc_ptrs)
 {
-    RELOC_PTR(gs_color_space, params.device_n.names);
     RELOC_PTR(gs_color_space, params.device_n.map);
     RELOC_PTR(gs_color_space, params.device_n.colorants);
 }
@@ -117,8 +109,8 @@ gs_cspace_new_DeviceN(
 {
     gs_color_space *pcs;
     gs_device_n_params *pcsdevn;
-    gs_separation_name *pnames;
-    int code;
+    char **pnames;
+    int i, code;
 
     if (palt_cspace == 0 || !palt_cspace->type->can_be_alt_space)
         return_error(gs_error_rangecheck);
@@ -130,6 +122,10 @@ gs_cspace_new_DeviceN(
     pcsdevn->names = NULL;
     pcsdevn->map = NULL;
     pcsdevn->colorants = NULL;
+    pcsdevn->named_color_supported = false;
+    pcsdevn->num_process_names = 0;
+    pcsdevn->process_names = NULL;
+    pcsdevn->mem = pmem->non_gc_memory;
 
     /* Allocate space for color names list. */
     code = alloc_device_n_map(&pcsdevn->map, pmem, "gs_cspace_build_DeviceN");
@@ -137,15 +133,14 @@ gs_cspace_new_DeviceN(
         gs_free_object(pmem, pcs, "gs_cspace_new_DeviceN");
         return code;
     }
-    /* Allocate space for color names list. */
-    pnames = (gs_separation_name *)
-        gs_alloc_byte_array(pmem, num_components, sizeof(gs_separation_name),
-                          ".gs_cspace_build_DeviceN(names)");
+    pnames = (char **)gs_alloc_bytes(pcsdevn->mem, num_components * sizeof(char *), "gs_cspace_new_DeviceN");
     if (pnames == 0) {
         gs_free_object(pmem, pcsdevn->map, ".gs_cspace_build_DeviceN(map)");
         gs_free_object(pmem, pcs, "gs_cspace_new_DeviceN");
         return_error(gs_error_VMerror);
     }
+    for (i=0; i<num_components; i++)
+        pnames[i] = NULL;		/* empty string for check_DeviceN_component_names */
     pcs->base_space = palt_cspace;
     rc_increment_cs(palt_cspace);
     pcsdevn->names = pnames;
@@ -181,10 +176,10 @@ alloc_device_n_map(gs_device_n_map ** ppmap, gs_memory_t * mem,
  * in the next gstate down in the gstate list (pgs->saved).
  */
 int
-gs_attachattributecolorspace(gs_separation_name sep_name, gs_gstate * pgs)
+gs_attachcolorant(char *sep_name, gs_gstate * pgs)
 {
     gs_color_space * pdevncs;
-    gs_device_n_attributes * patt;
+    gs_device_n_colorant * patt;
 
     /* Verify that we have a DeviceN color space */
     if (!pgs->saved)
@@ -193,8 +188,8 @@ gs_attachattributecolorspace(gs_separation_name sep_name, gs_gstate * pgs)
     if (pdevncs->type != &gs_color_space_type_DeviceN)
         return_error(gs_error_rangecheck);
 
-    /* Allocate an attribute list element for our linked list of attributes */
-    rc_alloc_struct_1(patt, gs_device_n_attributes, &st_device_n_attributes,
+    /* Allocate an attribute list element for our linked list of colorants */
+    rc_alloc_struct_1(patt, gs_device_n_colorant, &st_device_n_colorant,
                         pgs->memory, return_error(gs_error_VMerror),
                         "gs_attachattributrescolorspace");
 
@@ -530,10 +525,10 @@ gx_remap_concrete_DeviceN(const gs_color_space * pcs, const frac * pconc,
         if (dev_profile->spotnames != NULL && code >= 0) {
             temp_val = dev_profile->spotnames->equiv_cmyk_set;
             dev_profile->spotnames->equiv_cmyk_set = false;
-            gx_remap_concrete_devicen(pconc, pdc, pgs, dev, select);
+            gx_remap_concrete_devicen(pconc, pdc, pgs, dev, select, pcs);
             dev_profile->spotnames->equiv_cmyk_set = temp_val;
         } else {
-            gx_remap_concrete_devicen(pconc, pdc, pgs, dev, select);
+            gx_remap_concrete_devicen(pconc, pdc, pgs, dev, select, pcs);
         }
         return 0;
     }
@@ -547,11 +542,11 @@ gx_remap_concrete_DeviceN(const gs_color_space * pcs, const frac * pconc,
 static int
 check_DeviceN_component_names(const gs_color_space * pcs, gs_gstate * pgs)
 {
-    const gs_separation_name *names = pcs->params.device_n.names;
+    char **names = pcs->params.device_n.names;
     int num_comp = pcs->params.device_n.num_components;
     int i;
     int colorant_number;
-    byte * pname;
+    const char *pname;
     uint name_size;
     gs_devicen_color_map * pcolor_component_map
         = &pgs->color_component_map;
@@ -587,7 +582,10 @@ check_DeviceN_component_names(const gs_color_space * pcs, gs_gstate * pgs)
         /*
          * Get the character string and length for the component name.
          */
-        pcs->params.device_n.get_colorname_string(dev->memory, names[i], &pname, &name_size);
+        pname = names[i];
+        if (pname == NULL)
+            pname = "";
+        name_size = strlen(pname);
         /*
          * Compare the colorant name to the device's.  If the device's
          * compare routine returns GX_DEVICE_COLOR_MAX_COMPONENTS then the
@@ -595,13 +593,13 @@ check_DeviceN_component_names(const gs_color_space * pcs, gs_gstate * pgs)
          * SeparationOrder list.
          */
         colorant_number = (*dev_proc(dev, get_color_comp_index))
-                    (dev, (const char *)pname, name_size, SEPARATION_NAME);
+                                   (dev, pname, name_size, SEPARATION_NAME);
         if (colorant_number >= 0) {		/* If valid colorant name */
             pcolor_component_map->color_map[i] =
             (colorant_number == GX_DEVICE_COLOR_MAX_COMPONENTS) ? -1
                                                    : colorant_number;
         } else {
-            if (strncmp((char *) pname, "None", name_size) != 0) {
+            if (strncmp(pname, "None", name_size) != 0) {
                     non_match = true;
             } else {
                 /* Device N includes one or more None Entries. We can't reduce
@@ -619,6 +617,52 @@ check_DeviceN_component_names(const gs_color_space * pcs, gs_gstate * pgs)
     return 0;
 }
 
+/* Check if the colorants are all process colorants and of the same type */
+static separation_colors
+gx_check_process_names_DeviceN(gs_color_space * pcs, gs_gstate * pgs)
+{
+    int i, num_comp, num_spots = 0, num_rgb_process = 0;
+    int num_cmyk_process = 0, num_other = 0;
+    char **names;
+    const char *pname;
+    uint name_size;
+
+    names = pcs->params.device_n.names;
+    num_comp = pcs->params.device_n.num_components;
+
+    /* Step through the color space colorants */
+    for (i = 0; i < num_comp; i++) {
+        pname = names[i];
+        if (pname == NULL)
+            pname = "";
+        name_size = strlen(pname);
+
+        /* Classify */
+        if (strncmp(pname, "None", name_size) == 0) {
+            num_other++;
+        } else {
+            if (strncmp(pname, "Cyan", name_size) == 0 ||
+                strncmp(pname, "Magenta", name_size) == 0 ||
+                strncmp(pname, "Yellow", name_size) == 0 ||
+                strncmp(pname, "Black", name_size) == 0) {
+                num_cmyk_process++;
+            } else if (strncmp(pname, "Red", name_size) == 0 ||
+                strncmp(pname, "Green", name_size) == 0 ||
+                strncmp(pname, "Blue", name_size) == 0) {
+                num_rgb_process++;
+            } else {
+                num_spots++;
+            }
+        }
+    }
+
+    if (num_cmyk_process > 0 && num_rgb_process == 0 && num_spots == 0)
+        return SEP_PURE_CMYK;
+    if (num_rgb_process > 0 && num_cmyk_process == 0 && num_spots == 0)
+        return SEP_PURE_RGB;
+    return SEP_MIX;
+}
+
 /* Install a DeviceN color space. */
 static int
 gx_install_DeviceN(gs_color_space * pcs, gs_gstate * pgs)
@@ -627,15 +671,22 @@ gx_install_DeviceN(gs_color_space * pcs, gs_gstate * pgs)
     code = check_DeviceN_component_names(pcs, pgs);
     if (code < 0)
        return code;
+
+    if (pgs->icc_manager->device_named != NULL) {
+        pcs->params.device_n.named_color_supported =
+            gsicc_support_named_color(pcs, pgs);
+    }
+    pcs->params.device_n.color_type = gx_check_process_names_DeviceN(pcs, pgs);
+
     /* See if we have an ICC profile that we can associate with
        this DeviceN color space */
     if (pgs->icc_manager->device_n != NULL) {
         /* An nclr profile is in the manager.  Grab one that matches. */
         cmm_profile_t *profdata = gsicc_finddevicen(pcs, pgs->icc_manager);
         if (profdata != NULL)
-            rc_increment(profdata);
+            gsicc_adjust_profile_rc(profdata, 1, "gx_install_DeviceN");
         if (pcs->cmm_icc_profile_data != NULL)
-            rc_decrement(pcs->cmm_icc_profile_data, "gx_install_DeviceN");
+            gsicc_adjust_profile_rc(pcs->cmm_icc_profile_data, -1, "gx_install_DeviceN");
         pcs->cmm_icc_profile_data = profdata;
     }
     /* {csrc} was pgs->color_space->params.device_n.use_alt_cspace */
@@ -650,9 +701,9 @@ gx_install_DeviceN(gs_color_space * pcs, gs_gstate * pgs)
         /* Need to install the nclr cspace */
         code = gs_cspace_build_ICC(&nclr_pcs, NULL, pgs->memory);
         nclr_pcs->cmm_icc_profile_data = pcs->cmm_icc_profile_data;
-        rc_increment(pcs->cmm_icc_profile_data);
-        rc_increment_cs(nclr_pcs); /* Suspicious - RJW */
-        rc_decrement_cs(pcs->base_space, "gx_install_DeviceN");
+        gsicc_adjust_profile_rc(pcs->cmm_icc_profile_data, 1, "gx_install_DeviceN");
+        rc_increment(nclr_pcs);				/* FIXME: Suspicious - RJW */
+        rc_decrement(pcs->base_space, "gx_install_DeviceN");
         pcs->base_space = nclr_pcs;
     }
     /*
@@ -687,15 +738,18 @@ gx_set_overprint_DeviceN(const gs_color_space * pcs, gs_gstate * pgs)
         if ( base_type->index == gs_color_space_index_DeviceCMYK )
             return base_type->set_overprint( pcs->base_space, pgs );
         else
-            return gx_spot_colors_set_overprint( pcs->base_space, pgs);
-    }
-    else {
+            return gx_set_no_overprint(pgs);
+    } else {
         gs_overprint_params_t   params;
 
-        if ((params.retain_any_comps = pgs->overprint)) {
+
+        params.retain_any_comps = (pgs->overprint && pgs->is_fill_color) ||
+                                  (pgs->stroke_overprint && !pgs->is_fill_color);
+
+        if (params.retain_any_comps) {
             int     i, ncomps = pcs->params.device_n.num_components;
 
-            params.retain_spot_comps = false;
+            params.is_fill_color = pgs->is_fill_color;	/* for fill_stroke */
             params.drawn_comps = 0;
             for (i = 0; i < ncomps; i++) {
                 int mcomp = pcmap->color_map[i];
@@ -705,7 +759,9 @@ gx_set_overprint_DeviceN(const gs_color_space * pcs, gs_gstate * pgs)
         }
 
         /* Only DeviceCMYK can use overprint mode */
-        pgs->effective_overprint_mode = 0;
+        params.effective_opm = pgs->color[0].effective_opm = 0;
+        params.op_state = OP_STATE_NONE;
+        params.is_fill_color =pgs->is_fill_color;
         return gs_gstate_update_overprint(pgs, &params);
     }
 }
@@ -714,11 +770,27 @@ gx_set_overprint_DeviceN(const gs_color_space * pcs, gs_gstate * pgs)
 static void
 gx_final_DeviceN(const gs_color_space * pcs)
 {
-    gs_device_n_attributes * pnextatt, * patt = pcs->params.device_n.colorants;
+    gs_device_n_colorant * pnextatt, * patt = pcs->params.device_n.colorants;
+    uint num_proc_names = pcs->params.device_n.num_process_names;
+    gs_memory_t *mem = pcs->params.device_n.mem->non_gc_memory;
+    char **proc_names = pcs->params.device_n.process_names;
+    int k;
+
+    for (k = 0; k < pcs->params.device_n.num_components; k++)
+        gs_free_object(mem, pcs->params.device_n.names[k], "gx_final_DeviceN");
+    gs_free_object(mem, pcs->params.device_n.names, "gx_final_DeviceN");
+
+    if (num_proc_names > 0 && proc_names != NULL) {
+        for (k = 0; k < num_proc_names; k++) {
+            gs_free_object(mem, proc_names[k], "gx_final_DeviceN");
+        }
+        gs_free_object(mem, proc_names, "gx_final_DeviceN");
+    }
 
     rc_decrement_only(pcs->params.device_n.map, "gx_adjust_DeviceN");
     while (patt != NULL) {
         pnextatt = patt->next;
+        gs_free_object(mem, patt->colorant_name, "gx_final_DeviceN");
         rc_decrement_cs(patt->cspace, "gx_final_DeviceN");
         rc_decrement(patt, "gx_adjust_DeviceN");
         patt = pnextatt;
@@ -742,7 +814,7 @@ static int
 gx_serialize_DeviceN(const gs_color_space * pcs, stream * s)
 {
     const gs_device_n_params * p = &pcs->params.device_n;
-    uint n;
+    uint n, m;
     int code = gx_serialize_cspace_type(pcs, s);
 
     if (code < 0)
@@ -750,9 +822,14 @@ gx_serialize_DeviceN(const gs_color_space * pcs, stream * s)
     code = sputs(s, (const byte *)&p->num_components, sizeof(p->num_components), &n);
     if (code < 0)
         return code;
-    code = sputs(s, (const byte *)&p->names[0], sizeof(p->names[0]) * p->num_components, &n);
-    if (code < 0)
-        return code;
+    for (n=0;n < p->num_components;n++) {
+        const char *name = p->names[n];
+        if (name == NULL)
+            name = "";
+        code = sputs(s, (const byte *)name, strlen(name) + 1, &m);
+        if (code < 0)
+            return code;
+    }
     code = cs_serialize(pcs->base_space, s);
     if (code < 0)
         return code;
