@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -43,8 +43,7 @@ gs_setcolorspace_only(gs_gstate * pgs, gs_color_space * pcs)
     if (pcs->id != cs_old->id) {
         rc_increment_cs(pcs);
         pgs->color[0].color_space = pcs;
-        if ( (code = pcs->type->install_cspace(pcs, pgs)) < 0          ||
-              (pgs->overprint && (code = gs_do_set_overprint(pgs)) < 0)  ) {
+        if ( (code = pcs->type->install_cspace(pcs, pgs)) < 0) {
             pgs->color[0].color_space = cs_old;
             rc_decrement_only_cs(pcs, "gs_setcolorspace");
         } else {
@@ -86,6 +85,7 @@ gs_setcolor(gs_gstate * pgs, const gs_client_color * pcc)
     gs_client_color     cc_old = *pgs->color[0].ccolor;
     gx_device_color *dev_color = pgs->color[0].dev_color;
     bool do_unset  = true;
+    bool op;
 
     if (pgs->in_cachedevice)
         return_error(gs_error_undefined); /* PLRM3 page 215. */
@@ -105,6 +105,18 @@ gs_setcolor(gs_gstate * pgs, const gs_client_color * pcc)
     (*pcs->type->restrict_color)(pgs->color[0].ccolor, pcs);
     (*pcs->type->adjust_color_count)(&cc_old, pcs, -1);
 
+    /* If we are in a situation where we are doing overprint
+       with OPM, then we need to update the overprint
+       compositor as these values can affect what is drawn */
+    op = pgs->is_fill_color ? pgs->overprint : pgs->stroke_overprint;
+    if (pgs->color[0].effective_opm && op) {
+        if (pgs->is_fill_color) {
+            gs_setfilloverprint(pgs, op);
+        } else {
+            gs_setstrokeoverprint(pgs, op);
+        }
+    }
+
     return 0;
 }
 
@@ -267,8 +279,7 @@ gx_polarity_Indexed(const gs_color_space * pcs)
 static int
 gx_install_Indexed(gs_color_space * pcs, gs_gstate * pgs)
 {
-    return (*pcs->base_space->type->install_cspace)
-        (pcs->base_space, pgs);
+    return 0;
 }
 
 /* Color space overprint setting ditto. */
@@ -484,7 +495,7 @@ gx_concrete_space_Indexed(const gs_color_space * pcs,
 
     if (gs_color_space_is_PSCIE(pcs->base_space)) {
         if (pcs->base_space->icc_equivalent == NULL) {
-            gs_colorspace_set_icc_equivalent(pcs->base_space,
+            (void)gs_colorspace_set_icc_equivalent(pcs->base_space,
                                                 &is_lab, pgs->memory);
         }
         return (pcs->base_space->icc_equivalent);
@@ -524,19 +535,25 @@ gs_color_select_t select)
         return code;
 
     pconcs = cs_concrete_space(pcs, pgs);
-    /* Now see if we can do the named color replacement */
-    mapped = gx_remap_named_color(&cc, pconcs, pdc, pgs, dev, select);
+    if (pconcs) {
+        /* Now see if we can do the named color replacement */
+        mapped = gx_remap_named_color(&cc, pconcs, pdc, pgs, dev, select);
 
-    if (!mapped) {
-        /* Named color remap failed perhaps due to colorant not found. Do the 
-           old approach of concretize of the base space and remap concrete color */
-        const gs_color_space *pbcs =
-            (const gs_color_space *)pcs->base_space;
+        if (!mapped) {
+            /* Named color remap failed perhaps due to colorant not found. Do the
+               old approach of concretize of the base space and remap concrete color */
+            const gs_color_space *pbcs =
+                (const gs_color_space *)pcs->base_space;
+            cmm_dev_profile_t *dev_profile;
+            code = dev_proc(dev, get_profile)(dev, &dev_profile);
+            if (code < 0)
+                return code;
 
-        code = (*pbcs->type->concretize_color) (&cc, pbcs, conc, pgs, dev);
-        if (code < 0)
-            return code;
-        code = (*pconcs->type->remap_concrete_color)(conc, pconcs, pdc, pgs, dev, select);
+            code = (*pbcs->type->concretize_color) (&cc, pbcs, conc, pgs, dev);
+            if (code < 0)
+                return code;
+            code = (*pconcs->type->remap_concrete_color)(pconcs, conc, pdc, pgs, dev, select, dev_profile);
+        }
     }
 
     /* Save original color space and color info into dev color */
