@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -33,6 +33,9 @@
 #include "gsnamecl.h"  /* Custom color call back define */
 #include "gsicc.h"
 #include "gsicc_manage.h"
+#include "string_.h"
+#include "strmio.h"         /* needed for sfclose */
+#include "gsicc_cache.h"    /* Needed for gsicc_get_icc_buff_hash */
 
 static cs_proc_install_cspace(gx_install_DeviceGray);
 static cs_proc_install_cspace(gx_install_DeviceRGB);
@@ -99,7 +102,8 @@ gs_cspace_final(const gs_memory_t *cmem, void *vptr)
         pcs->type->final(pcs);
     if_debug2m('c', cmem, "[c]cspace final %p %d\n", pcs, (int)pcs->id);
     rc_decrement_only_cs(pcs->base_space, "gs_cspace_final");
-
+    if (pcs->params.device_n.devn_process_space != NULL)
+        rc_decrement_only_cs(pcs->params.device_n.devn_process_space, "gs_cspace_final");
     /* No need to decrement the ICC profile data.  It is handled
        by the finalize of the ICC space which is called above using
        pcs->type->final(pcs);  */
@@ -122,6 +126,7 @@ gs_cspace_alloc_with_id(gs_memory_t *mem, ulong id,
     pcs->pclient_color_space_data = NULL;
     pcs->cmm_icc_profile_data = NULL;
     pcs->icc_equivalent = NULL;
+    pcs->params.device_n.devn_process_space = NULL;
     return pcs;
 }
 
@@ -162,6 +167,57 @@ gs_cspace_new_DeviceCMYK(gs_memory_t *mem)
 }
 
 /* For use in initializing ICC color spaces for XPS */
+gs_color_space *
+gs_cspace_new_scrgb(gs_memory_t *pmem, gs_gstate * pgs)
+{
+    gs_color_space *pcspace = gs_cspace_alloc(pmem, &gs_color_space_type_ICC);
+    cmm_profile_t *profile;
+    stream *str;
+    int code;
+
+    if (pcspace == NULL)
+        return pcspace;
+
+    code = gsicc_open_search(SCRGB, strlen(SCRGB), pmem, pmem->gs_lib_ctx->profiledir,
+        pmem->gs_lib_ctx->profiledir_len, &str);
+
+    if (code < 0 || str == NULL) {
+        rc_decrement(pcspace, "gs_cspace_new_scrgb");
+        return NULL;
+    }
+
+    pcspace->cmm_icc_profile_data = gsicc_profile_new(str, pmem, SCRGB, strlen(SCRGB));
+    code = sfclose(str);
+    if (pcspace->cmm_icc_profile_data == NULL) {
+        rc_decrement(pcspace, "gs_cspace_new_scrgb");
+        return NULL;
+    }
+
+    /* Get the profile handle */
+    pcspace->cmm_icc_profile_data->profile_handle =
+        gsicc_get_profile_handle_buffer(pcspace->cmm_icc_profile_data->buffer,
+            pcspace->cmm_icc_profile_data->buffer_size, pmem);
+    if (!pcspace->cmm_icc_profile_data->profile_handle) {
+        rc_decrement(pcspace, "gs_cspace_new_scrgb");
+        return NULL;
+    }
+    profile = pcspace->cmm_icc_profile_data;
+
+    /* Compute the hash code of the profile. Everything in the
+    ICC manager will have it's hash code precomputed */
+    gsicc_get_icc_buff_hash(profile->buffer, &(profile->hashcode),
+        profile->buffer_size);
+    profile->hash_is_valid = true;
+    profile->num_comps =
+        gscms_get_input_channel_count(profile->profile_handle, profile->memory);
+    profile->num_comps_out =
+        gscms_get_output_channel_count(profile->profile_handle, profile->memory);
+    profile->data_cs =
+        gscms_get_profile_data_space(profile->profile_handle, profile->memory);
+    gsicc_set_icc_range(&profile);
+    return pcspace;
+}
+
 gs_color_space *
 gs_cspace_new_ICC(gs_memory_t *pmem, gs_gstate * pgs, int components)
 {
@@ -200,7 +256,7 @@ gs_cspace_new_ICC(gs_memory_t *pmem, gs_gstate * pgs, int components)
         case 4: pcspace->cmm_icc_profile_data = icc_manage->default_cmyk; break;
         default: rc_decrement(pcspace,"gs_cspace_new_ICC"); return NULL;
     }
-    rc_increment(pcspace->cmm_icc_profile_data);
+    gsicc_adjust_profile_rc(pcspace->cmm_icc_profile_data, 1, "gs_cspace_new_ICC");
     return pcspace;
 }
 
@@ -278,7 +334,7 @@ gx_install_DeviceGray(gs_color_space * pcs, gs_gstate * pgs)
 
     /* pcs takes a reference to the default_gray profile data */
     pcs->cmm_icc_profile_data = pgs->icc_manager->default_gray;
-    rc_increment(pgs->icc_manager->default_gray);
+    gsicc_adjust_profile_rc(pgs->icc_manager->default_gray, 1, "gx_install_DeviceGray");
     pcs->type = &gs_color_space_type_ICC;
     return 0;
 }
@@ -327,6 +383,12 @@ gs_cspace_base_space(const gs_color_space * pcspace)
     return pcspace->base_space;
 }
 
+const gs_color_space *
+gs_cspace_devn_process_space(const gs_color_space * pcspace)
+{
+    return pcspace->params.device_n.devn_process_space;
+}
+
 /* Abstract the reference counting for color spaces
    so that we can also increment the ICC profile
    if there is one associated with the color space */
@@ -382,7 +444,7 @@ gx_install_DeviceRGB(gs_color_space * pcs, gs_gstate * pgs)
 
     /* pcs takes a reference to default_rgb */
     pcs->cmm_icc_profile_data = pgs->icc_manager->default_rgb;
-    rc_increment(pcs->cmm_icc_profile_data);
+    gsicc_adjust_profile_rc(pcs->cmm_icc_profile_data, 1, "gx_install_DeviceRGB");
     pcs->type = &gs_color_space_type_ICC;
     return 0;
 }
@@ -401,32 +463,53 @@ gx_install_DeviceCMYK(gs_color_space * pcs, gs_gstate * pgs)
 
     /* pcs takes a reference to default_cmyk */
     pcs->cmm_icc_profile_data = pgs->icc_manager->default_cmyk;
-    rc_increment(pcs->cmm_icc_profile_data);
+    gsicc_adjust_profile_rc(pcs->cmm_icc_profile_data, 1, "gx_install_DeviceCMYK");
     pcs->type = &gs_color_space_type_ICC;
     return 0;
 }
 
+/*
+ * Communicate to the overprint compositor that this particular
+ * state overprint is not enabled.  This could be due to a
+ * mismatched color space, or that overprint is false or the
+ * device does not support it.
+ */
+int
+gx_set_no_overprint(gs_gstate* pgs)
+{
+    gs_overprint_params_t   params = { 0 };
+
+    params.retain_any_comps = false;
+    params.op_state = OP_STATE_NONE;
+    params.is_fill_color = pgs->is_fill_color;
+    params.effective_opm = pgs->color[0].effective_opm = 0;
+
+    return gs_gstate_update_overprint(pgs, &params);
+}
+
 /*
  * Push an overprint compositor onto the current device indicating that,
  * at most, the spot color parameters are to be preserved.
  *
  * This routine should be used for all Device, CIEBased, and ICCBased
- * color spaces, except for DeviceCMKY. Also, it would not be used for
- * DeviceRGB if we have simulated overprint turned on.
- * These latter cases requires a
- * special verson that supports overprint mode.
+ * color spaces, except for DeviceCMKY.
  */
 int
 gx_spot_colors_set_overprint(const gs_color_space * pcs, gs_gstate * pgs)
 {
-    gs_overprint_params_t   params;
+    gs_overprint_params_t   params = {0};
+    bool op = pgs->is_fill_color ? pgs->overprint : pgs->stroke_overprint;
 
-    if ((params.retain_any_comps = pgs->overprint)) {
-        params.retain_spot_comps = true;
+    if (!op)
         params.retain_any_comps = false;
-    }
+    else
+        params.retain_any_comps = true;
+
+    params.is_fill_color = pgs->is_fill_color;
+    params.op_state = OP_STATE_NONE;
+
     /* Only DeviceCMYK case can have overprint mode set to true */
-    pgs->effective_overprint_mode = 0;
+    params.effective_opm = pgs->color[0].effective_opm = 0;
     return gs_gstate_update_overprint(pgs, &params);
 }
 
@@ -557,7 +640,7 @@ gx_set_overprint_DeviceCMYK(const gs_color_space * pcs, gs_gstate * pgs)
    overprint simulation.  This is seen with AR when doing output preview,
    overprint simulation enabled of the file overprint_icc.pdf (see our
    test files) which has SWOP ICC based CMYK fills.  In AR, if we use a
-   simluation ICC profile that is different than the source profile,
+   simulation ICC profile that is different than the source profile,
    overprinting is no longer previewed. We follow the same logic here.
    If the source and destination ICC profiles do not match, then there is
    effectively no overprinting enabled.  This is bug 692433 */
@@ -566,13 +649,17 @@ int gx_set_overprint_cmyk(const gs_color_space * pcs, gs_gstate * pgs)
     gx_device *             dev = pgs->device;
     gx_device_color_info *  pcinfo = (dev == 0 ? 0 : &dev->color_info);
     gx_color_index          drawn_comps = 0;
-    gs_overprint_params_t   params;
+    gs_overprint_params_t   params = { 0 };
     gx_device_color        *pdc;
     cmm_dev_profile_t      *dev_profile;
     cmm_profile_t          *output_profile = 0;
     int                     code;
     bool                    profile_ok = false;
     gsicc_rendering_param_t        render_cond;
+    bool                    eop;
+
+    if_debug0m(gs_debug_flag_overprint, pgs->memory,
+        "[overprint] gx_set_overprint_cmyk\n");
 
     if (dev) {
         code = dev_proc(dev, get_profile)(dev, &dev_profile);
@@ -588,6 +675,10 @@ int gx_set_overprint_cmyk(const gs_color_space * pcs, gs_gstate * pgs)
         else
             drawn_comps = pcinfo->process_comps;
     }
+
+    if_debug1m(gs_debug_flag_overprint, pgs->memory,
+        "[overprint] gx_set_overprint_cmyk. drawn_comps = 0x%x\n", drawn_comps);
+
     if (drawn_comps == 0)
         return gx_spot_colors_set_overprint(pcs, pgs);
 
@@ -600,15 +691,26 @@ int gx_set_overprint_cmyk(const gs_color_space * pcs, gs_gstate * pgs)
         }
     }
 
-    pdc = gs_currentdevicecolor_inline(pgs);
-    if (color_is_set(pdc) && profile_ok && pgs->overprint_mode == 1) {
+    eop = gs_currentcolor_eopm(pgs);
+
+    if_debug3m(gs_debug_flag_overprint, pgs->memory,
+        "[overprint] gx_set_overprint_cmyk. is_fill_color = %d, pgs->color[0].effective_opm = %d pgs->color[1].effective_opm = %d\n",
+        pgs->is_fill_color, pgs->color[0].effective_opm, pgs->color[1].effective_opm);
+
+    if (profile_ok && eop) {
         gx_color_index  nz_comps, one, temp;
         int             code;
         int             num_colorant[4], k;
         bool            colorant_ok;
-
         dev_color_proc_get_nonzero_comps((*procp));
 
+        if_debug0m(gs_debug_flag_overprint, pgs->memory,
+            "[overprint] gx_set_overprint_cmyk. color_is_set, profile_ok and eop\n");
+
+        code = gx_set_dev_color(pgs);
+        if (code < 0)
+            return code;
+        pdc = gs_currentdevicecolor_inline(pgs);
         procp = pdc->type->get_nonzero_comps;
         if (pdc->ccolor_valid) {
             /* If we have the source colors, then use those in making the
@@ -629,7 +731,9 @@ int gx_set_overprint_cmyk(const gs_color_space * pcs, gs_gstate * pgs)
             one = 1;
             colorant_ok = true;
             for (k = 0; k < 4; k++) {
-                if (pdc->ccolor.paint.values[k] != 0) {
+                /* Note: AR assumes the value is zero if it
+                   is less than 0.5 out of 255 */
+                if (pdc->ccolor.paint.values[k] > (0.5 / 255.0)) {
                     if (num_colorant[k] == -1) {
                         colorant_ok = false;
                     } else {
@@ -649,13 +753,20 @@ int gx_set_overprint_cmyk(const gs_color_space * pcs, gs_gstate * pgs)
         }
         drawn_comps &= nz_comps;
     }
+    params.is_fill_color = pgs->is_fill_color;
     params.retain_any_comps = true;
-    params.retain_spot_comps = false;
     params.drawn_comps = drawn_comps;
-    /* We are in CMYK, the profiles match and overprint is true.  Set effective
-       overprint mode to overprint mode */
-    pgs->effective_overprint_mode = pgs->overprint_mode;
+    params.op_state = OP_STATE_NONE;
 
+    if_debug2m(gs_debug_flag_overprint, pgs->memory,
+        "[overprint] gx_set_overprint_cmyk. retain_any_comps = %d, drawn_comps = 0x%x\n",
+        params.retain_any_comps, params.drawn_comps);
+
+    /* We are in CMYK, the profiles match and overprint is true.  Set effective
+       overprint mode to overprint mode but only if effective has not already
+       been set to 0 */
+    params.effective_opm = pgs->color[0].effective_opm =
+        pgs->overprint_mode && gs_currentcolor_eopm(pgs);
     return gs_gstate_update_overprint(pgs, &params);
 }
 
@@ -877,7 +988,9 @@ ENUM_PTRS_BEGIN_PROC(color_space_enum_ptrs)
         return ENUM_OBJ(pcs->pclient_color_space_data);
     if (index == 2)
         return ENUM_OBJ(pcs->icc_equivalent);
-    return ENUM_USING(*pcs->type->stype, vptr, size, index - 3);
+    if (index == 3)
+        return ENUM_OBJ(pcs->params.device_n.devn_process_space);
+    return ENUM_USING(*pcs->type->stype, vptr, size, index - 4);
     ENUM_PTRS_END_PROC
 }
 static
@@ -886,6 +999,7 @@ RELOC_PTRS_WITH(color_space_reloc_ptrs, gs_color_space *pcs)
     RELOC_VAR(pcs->base_space);
     RELOC_VAR(pcs->pclient_color_space_data);
     RELOC_VAR(pcs->icc_equivalent);
+    RELOC_VAR(pcs->params.device_n.devn_process_space);
     RELOC_USING(*pcs->type->stype, vptr, size);
 }
 RELOC_PTRS_END
