@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -51,11 +51,11 @@ gx_device_finalize(const gs_memory_t *cmem, void *vptr)
     gx_device * const dev = (gx_device *)vptr;
     (void)cmem; /* unused */
 
+    discard(gs_closedevice(dev));
+
     if (dev->icc_struct != NULL) {
         rc_decrement(dev->icc_struct, "gx_device_finalize(icc_profile)");
     }
-    if (dev->finalize)
-        dev->finalize(dev);
 
     /* Deal with subclassed devices. Ordinarily these should not be a problem, we
      * will never see them, but if ths is a end of job restore we can end up
@@ -71,7 +71,9 @@ gx_device_finalize(const gs_memory_t *cmem, void *vptr)
         dev->PageList = 0;
     }
 
-    discard(gs_closedevice(dev));
+    if (dev->finalize)
+        dev->finalize(dev);
+
     if (dev->stype_is_dynamic)
         gs_free_const_object(dev->memory->non_gc_memory, dev->stype,
                              "gx_device_finalize");
@@ -195,8 +197,8 @@ gs_output_page(gs_gstate * pgs, int num_copies, int flush)
         return code;
     if (((code = gs_newpath(pgs)) < 0) ||
         ((code = gs_moveto(pgs, 0.0, 0.0)) < 0) ||
-	((code = gs_setgray(pgs, 0.0) < 0)) ||
-	((code = gs_fill(pgs)) < 0))
+    ((code = gs_setgray(pgs, 0.0)) < 0) ||
+    ((code = gs_fill(pgs)) < 0))
     {
         gs_grestore(pgs);
 	return code;
@@ -498,7 +500,9 @@ gs_setdevice_no_erase(gs_gstate * pgs, gx_device * dev)
     if (libctx->io_device_table != NULL) {
         cmm_dev_profile_t *dev_profile;
         if (pgs->icc_manager->lab_profile == NULL) {  /* pick one not set externally */
-            gsicc_init_iccmanager(pgs);
+            code = gsicc_init_iccmanager(pgs);
+            if (code < 0)
+                return(code);
         }
         /* Also, if the device profile is not yet set then take care of that
            before we start filling pages, if we can */
@@ -601,7 +605,7 @@ gs_setdevice_no_init(gs_gstate * pgs, gx_device * dev)
     }
     rc_assign(pgs->device, dev, "gs_setdevice_no_init");
     gs_gstate_update_device(pgs, dev);
-    return pgs->overprint ? gs_do_set_overprint(pgs) : 0;
+    return 0;
 }
 
 /* Initialize a just-allocated device. */
@@ -691,14 +695,17 @@ int
 gs_nulldevice(gs_gstate * pgs)
 {
     int code = 0;
-
-    if (pgs->device == 0 || !gx_device_is_null(pgs->device)) {
+    gs_gstate *spgs;
+    bool saveLockSafety = false;
+    if (pgs->device == NULL || !gx_device_is_null(pgs->device)) {
         gx_device *ndev;
         code = gs_copydevice(&ndev, (const gx_device *)&gs_null_device,
                                  pgs->memory);
 
         if (code < 0)
             return code;
+        if (gs_currentdevice_inline(pgs) != NULL)
+            saveLockSafety = gs_currentdevice_inline(pgs)->LockSafetyParams;
         /*
          * Internal devices have a reference count of 0, not 1,
          * aside from references from graphics states.
@@ -716,8 +723,22 @@ gs_nulldevice(gs_gstate * pgs)
             set_dev_proc(ndev, get_profile, gx_default_get_profile);
         }
 
-        if ((code = gs_setdevice_no_erase(pgs, ndev)) < 0)
+        if (gs_setdevice_no_erase(pgs, ndev) < 0) {
             gs_free_object(pgs->memory, ndev, "gs_copydevice(device)");
+            /* We are out of options: find the device we installed in
+               the initial graphics state, and put that in place.
+               We just need something so we can end this job cleanly.
+             */
+            spgs = pgs->saved;
+            if (spgs != NULL) {
+                while (spgs->saved) spgs = spgs->saved;
+                gs_currentdevice_inline(pgs) = gs_currentdevice_inline(spgs);
+                rc_increment(gs_currentdevice_inline(pgs));
+            }
+            code = gs_note_error(gs_error_Fatal);
+        }
+        if (gs_currentdevice_inline(pgs) != NULL)
+            gs_currentdevice_inline(pgs)->LockSafetyParams = saveLockSafety;
     }
     return code;
 }
@@ -1170,10 +1191,16 @@ done:
     return(code);
 }
 
+static int
+noclose(FILE *f)
+{
+    return 0;
+}
+
 /* Open the output file for a device. */
 int
 gx_device_open_output_file(const gx_device * dev, char *fname,
-                           bool binary, bool positionable, FILE ** pfile)
+                           bool binary, bool positionable, gp_file ** pfile)
 {
     gs_parsed_file_name_t parsed;
     const char *fmt;
@@ -1183,7 +1210,7 @@ gx_device_open_output_file(const gx_device * dev, char *fname,
     if (pfname == NULL) {
         code = gs_note_error(gs_error_VMerror);
 	goto done;
-     }
+    }
 
     if (strlen(fname) == 0) {
         code = gs_note_error(gs_error_undefinedfilename);
@@ -1200,9 +1227,14 @@ gx_device_open_output_file(const gx_device * dev, char *fname,
             code = gs_note_error(gs_error_undefinedfilename);
 	    goto done;
 	}
-        *pfile = dev->memory->gs_lib_ctx->fstdout;
+        *pfile = gp_file_FILE_alloc(dev->memory);
+        if (*pfile == NULL) {
+            code = gs_note_error(gs_error_VMerror);
+            goto done;
+        }
+        gp_file_FILE_set(*pfile, dev->memory->gs_lib_ctx->core->fstdout, noclose);
         /* Force stdout to binary. */
-        code = gp_setmode_binary(*pfile, true);
+        code = gp_setmode_binary_impl(dev->memory->gs_lib_ctx->core->fstdout, true);
 	goto done;
     } else if (parsed.iodev && !strcmp(parsed.iodev->dname, "%pipe%")) {
         positionable = false;
@@ -1224,7 +1256,8 @@ gx_device_open_output_file(const gx_device * dev, char *fname,
         parsed.fname = pfname;
         parsed.len = strlen(parsed.fname);
     }
-    if (positionable || (parsed.iodev && parsed.iodev != iodev_default(dev->memory))) {
+    if (parsed.iodev &&
+        (positionable || parsed.iodev != iodev_default(dev->memory))) {
         char fmode[4];
 
         if (!parsed.fname) {
@@ -1235,13 +1268,12 @@ gx_device_open_output_file(const gx_device * dev, char *fname,
         if (positionable)
             strcat(fmode, "+");
         code = parsed.iodev->procs.gp_fopen(parsed.iodev, parsed.fname, fmode,
-                                         pfile, NULL, 0);
+                                            pfile, NULL, 0, dev->memory);
         if (code)
             emprintf1(dev->memory,
                       "**** Could not open the file %s .\n",
                       parsed.fname);
-    }
-    else {
+    } else {
         *pfile = gp_open_printer(dev->memory, (pfname[0] ? pfname : fname), binary);
         if (!(*pfile)) {
             emprintf1(dev->memory, "**** Could not open the file '%s'.\n", (pfname[0] ? pfname : fname));
@@ -1260,7 +1292,7 @@ done:
 /* Close the output file for a device. */
 int
 gx_device_close_output_file(const gx_device * dev, const char *fname,
-                            FILE *file)
+                            gp_file *file)
 {
     gs_parsed_file_name_t parsed;
     const char *fmt;
@@ -1276,7 +1308,7 @@ gx_device_close_output_file(const gx_device * dev, const char *fname,
         if (parsed.iodev != iodev_default(dev->memory))
             return parsed.iodev->procs.fclose(parsed.iodev, file);
     }
-    gp_close_printer(dev->memory, file, (parsed.fname ? parsed.fname : fname));
+    gp_close_printer(file, (parsed.fname ? parsed.fname : fname));
     return 0;
 }
 
