@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -288,6 +288,10 @@ gs_font_dir_finalize(const gs_memory_t *cmem, void *vptr)
         cmem->gs_lib_ctx->font_dir = NULL;
     }
 
+    /* free character cache machinery */
+    gs_free_object(pdir->memory, pdir->fmcache.mdata, "gs_font_dir_finalize");
+    gs_free_object(pdir->memory, pdir->ccache.table, "gs_font_dir_finalize");
+
     /* free the circular list of memory chunks */
     while (chunk) {
         if (start_chunk == chunk->next) {
@@ -304,7 +308,6 @@ gs_font_dir_finalize(const gs_memory_t *cmem, void *vptr)
     pdir->ccache.chunks = NULL;
 }
 
-
 /* Allocate and minimally initialize a font. */
 gs_font *
 gs_font_alloc(gs_memory_t *mem, gs_memory_type_ptr_t pstype,
