@@ -1,17 +1,19 @@
-/* Copyright (C) 2001-2006 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
    implied.
 
-   This software is distributed under license and may not be copied, modified
-   or distributed except as expressly authorized under the terms of that
-   license.  Refer to licensing information at http://www.artifex.com/
-   or contact Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134,
-   San Rafael, CA  94903, U.S.A., +1(415)492-9861, for further information.
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
-/* $Id: gshtscr.c 11058 2010-04-13 12:15:42Z robin $ */
+
 /* Screen (Type 1) halftone processing for Ghostscript library */
 #include "math_.h"
 #include "gx.h"
@@ -21,7 +23,6 @@
 #include "gzstate.h"
 #include "gxdevice.h"           /* for gzht.h */
 #include "gzht.h"
-#include "gswts.h"
 
 /* Define whether to force all halftones to be strip halftones, */
 /* for debugging. */
@@ -72,22 +73,6 @@ gs_currentaccuratescreens(gs_memory_t *mem)
     return ctx->screen_accurate_screens;
 }
 
-void
-gs_setusewts(gs_memory_t *mem, bool use_wts)
-{
-    gs_lib_ctx_t *ctx = gs_lib_ctx_get_interp_instance(mem);
-
-    ctx->screen_use_wts = use_wts;
-}
-
-bool
-gs_currentusewts(gs_memory_t *mem)
-{
-    gs_lib_ctx_t *ctx = gs_lib_ctx_get_interp_instance(mem);
-
-    return ctx->screen_use_wts;
-}
-
 void
 gs_setminscreenlevels(gs_memory_t *mem, uint levels)
 {
@@ -182,7 +167,7 @@ gx_compute_cell_values(gx_ht_cell_params_t * phcp)
 /* Forward references */
 static int pick_cell_size(gs_screen_halftone * ph,
      const gs_matrix * pmat, ulong max_size, uint min_levels, bool accurate,
-			   gx_ht_cell_params_t * phcp);
+                           gx_ht_cell_params_t * phcp);
 
 /* Allocate a screen enumerator. */
 gs_screen_enum *
@@ -193,7 +178,7 @@ gs_screen_enum_alloc(gs_memory_t * mem, client_name_t cname)
 
 /* Set up for halftone sampling. */
 int
-gs_screen_init(gs_screen_enum * penum, gs_state * pgs,
+gs_screen_init(gs_screen_enum * penum, gs_gstate * pgs,
                gs_screen_halftone * phsp)
 {
     gs_lib_ctx_t *ctx = gs_lib_ctx_get_interp_instance(pgs->memory);
@@ -202,7 +187,7 @@ gs_screen_init(gs_screen_enum * penum, gs_state * pgs,
                                    ctx->screen_accurate_screens);
 }
 int
-gs_screen_init_memory(gs_screen_enum * penum, gs_state * pgs,
+gs_screen_init_memory(gs_screen_enum * penum, gs_gstate * pgs,
                 gs_screen_halftone * phsp, bool accurate, gs_memory_t * mem)
 {
     int code =
@@ -247,7 +232,7 @@ gs_screen_order_alloc(gx_ht_order *porder, gs_memory_t *mem)
     return code;
 }
 int
-gs_screen_order_init_memory(gx_ht_order * porder, const gs_state * pgs,
+gs_screen_order_init_memory(gx_ht_order * porder, const gs_gstate * pgs,
                             gs_screen_halftone * phsp, bool accurate,
                             gs_memory_t * mem)
 {
@@ -361,6 +346,8 @@ pick_cell_size(gs_screen_halftone * ph, const gs_matrix * pmat, ulong max_size,
         return_error(gs_error_rangecheck);
     while ((fabs(u0) + fabs(v0)) * rt < 4)
         ++rt;
+    phcp->C = 0;
+
   try_size:
     better = false;
     {
@@ -373,11 +360,18 @@ pick_cell_size(gs_screen_halftone * ph, const gs_matrix * pmat, ulong max_size,
         p.R = p.R1 = rt;
         for (p.M = m0 + 1; p.M >= m0; p.M--)
             for (p.N = n0 + 1; p.N >= n0; p.N--) {
-                long raster, wt, wt_size;
+                long raster, wt;
+#ifdef DEBUG
+                long wt_size;
+#endif
                 double fr, ar, ft, at, f_diff, a_diff, f_err, a_err;
 
                 p.M1 = (int)floor(p.M / T + 0.5);
                 p.N1 = (int)floor(p.N * T + 0.5);
+
+                if (p.M1 == 0 && p.N1 == 0)
+                    return_error(gs_error_rangecheck);
+
                 gx_compute_cell_values(&p);
                 if_debug3('h', "[h]trying m=%d, n=%d, r=%d\n", p.M, p.N, rt);
                 wt = p.W;
@@ -388,7 +382,9 @@ pick_cell_size(gs_screen_halftone * ph, const gs_matrix * pmat, ulong max_size,
                 raster = bitmap_raster(wt);
                 if (raster > max_size / p.D || raster > max_long / wt)
                     continue;
+#ifdef DEBUG
                 wt_size = raster * wt;
+#endif
 
                 /* Compute the corresponding values of F and A. */
 
@@ -420,18 +416,18 @@ pick_cell_size(gs_screen_halftone * ph, const gs_matrix * pmat, ulong max_size,
 
                 {
                     /*
-		     * Compute the error in position between ideal location.
-		     * and the current integer location.
-		     */
+                     * Compute the error in position between ideal location.
+                     * and the current integer location.
+                     */
 
-		    double error =
-			(fn0 - p.N) * (fn0 - p.N) + (fm0 - p.M) * (fm0 - p.M);
-		    /*
-		     * Adjust the error by the length of the vector.  This gives
-		     * a slight bias toward larger cell sizzes.
-		     */
-		    error /= p.N * p.N + p.M * p.M;
-		    error = sqrt(error); /* The previous calcs. gave value squared */
+                    double error =
+                        (fn0 - p.N) * (fn0 - p.N) + (fm0 - p.M) * (fm0 - p.M);
+                    /*
+                     * Adjust the error by the length of the vector.  This gives
+                     * a slight bias toward larger cell sizzes.
+                     */
+                    error /= p.N * p.N + p.M * p.M;
+                    error = sqrt(error); /* The previous calcs. gave value squared */
                     if (error > e_best)
                         continue;
                     e_best = error;
@@ -449,6 +445,15 @@ pick_cell_size(gs_screen_halftone * ph, const gs_matrix * pmat, ulong max_size,
                     goto done;
             }
     }
+    /* It is possible (for ridiculous halftone screens) to reach this point without ever
+     * having been able to find a suitable screen. In that case we will not have set
+     * phcp and so phcp->C will be undefined. We can detect this condition by checking rt
+     * is 1 (so its the first attempt to find a screen) and better is false (we didn't find
+     * a better match). If that happens, exit with an error.
+     */
+    if (rt == 1 && !better)
+        return_error(gs_error_rangecheck);
+
     if (phcp->C < min_levels) { /* We don't have enough levels yet.  Keep going. */
         ++rt;
         goto try_size;
@@ -481,9 +486,11 @@ pick_cell_size(gs_screen_halftone * ph, const gs_matrix * pmat, ulong max_size,
 /* This is the second half of gs_screen_init_accurate. */
 int
 gs_screen_enum_init_memory(gs_screen_enum * penum, const gx_ht_order * porder,
-                           gs_state * pgs, const gs_screen_halftone * phsp,
+                           gs_gstate * pgs, const gs_screen_halftone * phsp,
                            gs_memory_t * mem)
 {
+    int code;
+
     penum->pgs = pgs;           /* ensure clean for GC */
     if (&penum->order != porder) /* Pacify Valgrind */
         penum->order = *porder;
@@ -492,47 +499,45 @@ gs_screen_enum_init_memory(gs_screen_enum * penum, const gx_ht_order * porder,
     penum->halftone.params.screen = *phsp;
     penum->x = penum->y = 0;
 
-    if (porder->wse == NULL) {
-	penum->strip = porder->num_levels / porder->width;
-	penum->shift = porder->shift;
-	/*
-	 * We want a transformation matrix that maps the parallelogram
-	 * (0,0), (U,V), (U-V',V+U'), (-V',U') to the square (+/-1, +/-1).
-	 * If the coefficients are [a b c d e f] and we let
-	 *      u = U = M/R, v = V = N/R,
-	 *      r = -V' = -N'/R', s = U' = M'/R',
-	 * then we just need to solve the equations:
-	 *      a*0 + c*0 + e = -1      b*0 + d*0 + f = -1
-	 *      a*u + c*v + e = 1       b*u + d*v + f = 1
-	 *      a*r + c*s + e = -1      b*r + d*s + f = 1
-	 * This has the following solution:
-	 *      Q = 2 / (M*M' + N*N')
-	 *      a = Q * R * M'
-	 *      b = -Q * R' * N
-	 *      c = Q * R * N'
-	 *      d = Q * R' * M
-	 *      e = -1
-	 *      f = -1
-	 */
-	{
-	    const int M = porder->params.M, N = porder->params.N, R = porder->params.R;
-	    const int M1 = porder->params.M1, N1 = porder->params.N1, R1 = porder->params.R1;
-	    double Q = 2.0 / ((long)M * M1 + (long)N * N1);
+    penum->strip = porder->num_levels / porder->width;
+    penum->shift = porder->shift;
+    /*
+     * We want a transformation matrix that maps the parallelogram
+     * (0,0), (U,V), (U-V',V+U'), (-V',U') to the square (+/-1, +/-1).
+     * If the coefficients are [a b c d e f] and we let
+     *      u = U = M/R, v = V = N/R,
+     *      r = -V' = -N'/R', s = U' = M'/R',
+     * then we just need to solve the equations:
+     *      a*0 + c*0 + e = -1      b*0 + d*0 + f = -1
+     *      a*u + c*v + e = 1       b*u + d*v + f = 1
+     *      a*r + c*s + e = -1      b*r + d*s + f = 1
+     * This has the following solution:
+     *      Q = 2 / (M*M' + N*N')
+     *      a = Q * R * M'
+     *      b = -Q * R' * N
+     *      c = Q * R * N'
+     *      d = Q * R' * M
+     *      e = -1
+     *      f = -1
+     */
+    {
+        const int M = porder->params.M, N = porder->params.N, R = porder->params.R;
+        const int M1 = porder->params.M1, N1 = porder->params.N1, R1 = porder->params.R1;
+        double Q = 2.0 / ((long)M * M1 + (long)N * N1);
 
-	    penum->mat.xx = Q * (R * M1);
-	    penum->mat.xy = Q * (-R1 * N);
-	    penum->mat.yx = Q * (R * N1);
-	    penum->mat.yy = Q * (R1 * M);
-	    penum->mat.tx = -1.0;
-	    penum->mat.ty = -1.0;
-	    gs_matrix_invert(&penum->mat, &penum->mat_inv);
-	}
-	if_debug7('h', "[h]Screen: (%dx%d)/%d [%f %f %f %f]\n",
-		  porder->width, porder->height, porder->params.R,
-		  penum->mat.xx, penum->mat.xy,
-		  penum->mat.yx, penum->mat.yy);
+        penum->mat.xx = Q * (R * M1);
+        penum->mat.xy = Q * (-R1 * N);
+        penum->mat.yx = Q * (R * N1);
+        penum->mat.yy = Q * (R1 * M);
+        penum->mat.tx = -1.0;
+        penum->mat.ty = -1.0;
+        code = gs_matrix_invert(&penum->mat, &penum->mat_inv);
     }
-    return 0;
+    if_debug7('h', "[h]Screen: (%dx%d)/%d [%f %f %f %f]\n",
+              porder->width, porder->height, porder->params.R,
+              penum->mat.xx, penum->mat.xy,
+              penum->mat.yx, penum->mat.yy);
+    return code;
 }
 
 /* Report current point for sampling */
@@ -544,12 +549,6 @@ gs_screen_currentpoint(gs_screen_enum * penum, gs_point * ppt)
     double sx, sy; /* spot center in spot coords (integers) */
     gs_point spot_center; /* device coords */
 
-    if (penum->order.wse) {
-	int code;
-	code = gs_wts_screen_enum_currentpoint(penum->order.wse, ppt);
-	return code;
-    }
-
     if (penum->y >= penum->strip) {     /* all done */
         gx_ht_construct_spot_order(&penum->order);
         return 1;
@@ -592,32 +591,28 @@ gs_screen_currentpoint(gs_screen_enum * penum, gs_point * ppt)
 
 /* Record next halftone sample */
 int
-gs_screen_next(gs_screen_enum * penum, floatp value)
+gs_screen_next(gs_screen_enum * penum, double value)
 {
-    if (penum->order.wse) {
-	return gs_wts_screen_enum_next (penum->order.wse, value);
-    } else {
-	ht_sample_t sample;
-	int width = penum->order.width;
-	gx_ht_bit *bits = (gx_ht_bit *)penum->order.bit_data;
+    ht_sample_t sample;
+    int width = penum->order.width;
+    gx_ht_bit *bits = (gx_ht_bit *)penum->order.bit_data;
 
-	if (value < -1.0 || value > 1.0)
-	    return_error(gs_error_rangecheck);
-	sample = (ht_sample_t) ((value + 1) * max_ht_sample);
-#ifdef DEBUG
-	if (gs_debug_c('H')) {
-	    gs_point pt;
+    if (value < -1.0 || value > 1.0)
+        return_error(gs_error_rangecheck);
+    sample = (ht_sample_t) ((value + 1) * max_ht_sample);
+#if defined(DEBUG) && !defined(GS_THREADSAFE)
+    if (gs_debug_c('H')) {
+        gs_point pt;
 
-	    gs_screen_currentpoint(penum, &pt);
-	    dlprintf6("[H]sample x=%d y=%d (%f,%f): %f -> %u\n",
-		      penum->x, penum->y, pt.x, pt.y, value, sample);
-	}
-#endif
-	bits[penum->y * width + penum->x].mask = sample;
-	if (++(penum->x) >= width)
-	    penum->x = 0, ++(penum->y);
-	return 0;
+        gs_screen_currentpoint(penum, &pt);
+        dlprintf6("[H]sample x=%d y=%d (%f,%f): %f -> %u\n",
+                  penum->x, penum->y, pt.x, pt.y, value, sample);
     }
+#endif
+    bits[penum->y * width + penum->x].mask = sample;
+    if (++(penum->x) >= width)
+        penum->x = 0, ++(penum->y);
+    return 0;
 }
 
 /* Install a fully constructed screen in the gstate. */
