@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -25,9 +25,7 @@
 #include "gsicc_cms.h"
 #include "gxdevice.h"
 
-#ifndef MEMENTO_SQUEEZE_BUILD
 #define USE_LCMS2_LOCKING
-#endif
 
 #ifdef USE_LCMS2_LOCKING
 #include "gxsync.h"
@@ -54,7 +52,9 @@ static
 void *gs_lcms2_malloc(cmsContext id, unsigned int size)
 {
     void *ptr;
+#if !(defined(SHARE_LCMS) && SHARE_LCMS==1)
     gs_memory_t *mem = (gs_memory_t *)cmsGetContextUserData(id);
+#endif
 
 #if defined(SHARE_LCMS) && SHARE_LCMS==1
     ptr = malloc(size);
@@ -71,7 +71,10 @@ void *gs_lcms2_malloc(cmsContext id, unsigned int size)
 static
 void gs_lcms2_free(cmsContext id, void *ptr)
 {
+#if !(defined(SHARE_LCMS) && SHARE_LCMS==1)
     gs_memory_t *mem = (gs_memory_t *)cmsGetContextUserData(id);
+#endif
+
     if (ptr != NULL) {
 #if DEBUG_LCMS_MEM
         gs_warn1("lcms free at 0x%x",ptr);
@@ -88,7 +91,9 @@ void gs_lcms2_free(cmsContext id, void *ptr)
 static
 void *gs_lcms2_realloc(cmsContext id, void *ptr, unsigned int size)
 {
+#if !(defined(SHARE_LCMS) && SHARE_LCMS==1)
     gs_memory_t *mem = (gs_memory_t *)cmsGetContextUserData(id);
+#endif
     void *ptr2;
 
     if (ptr == 0)
@@ -306,7 +311,7 @@ gscms_transform_color_buffer(gx_device *dev, gsicc_link_t *icclink,
     int planar,numbytes, big_endian, hasalpha, k;
     unsigned char *inputpos, *outputpos;
 #if DUMP_CMS_BUFFER
-    FILE *fid_in, *fid_out;
+    gp_file *fid_in, *fid_out;
 #endif
     /* Although little CMS does  make assumptions about data types in its
        transformations you can change it after the fact.  */
@@ -438,8 +443,8 @@ gscms_transform_color_buffer(gx_device *dev, gsicc_link_t *icclink,
         }
     }
 #if DUMP_CMS_BUFFER
-    fid_in = gp_fopen("CM_Input.raw","ab");
-    fid_out = gp_fopen("CM_Output.raw","ab");
+    fid_in = gp_fopen(dev->memory,"CM_Input.raw","ab");
+    fid_out = gp_fopen(dev->memory,"CM_Output.raw","ab");
     fwrite((unsigned char*) inputbuffer,sizeof(unsigned char),
                             input_buff_desc->row_stride,fid_in);
     fwrite((unsigned char*) outputbuffer,sizeof(unsigned char),
