@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -65,7 +65,7 @@ static void clip_stack_rc_adjust(gx_clip_stack_t *cs, int delta, client_name_t c
  *      is independent of the gstate.  These are garbage collected, not
  *      reference counted, so we don't need to do anything special with them
  *      when manipulating gstates.  Currently this includes:
- *              font, device
+ *              font
  *
  * (3) Objects that are referenced directly by the gstate, may be shared
  *      among gstates, and should disappear when no gstates reference them.
@@ -110,7 +110,7 @@ static void clip_stack_rc_adjust(gx_clip_stack_t *cs, int delta, client_name_t c
  *      The individual routines that manipulate these are responsible
  *      for doing the right kind of reference counting or whatever.
  *      Currently:
- *              path, clip_path, and (if different from both clip_path
+ *              devices, path, clip_path, and (if different from both clip_path
  *                and view_clip) effective_clip.path require
  *                gx_path_assign/free, which uses a reference count;
  *              color_space and ccolor require cs_adjust_color/cspace_count
@@ -233,6 +233,9 @@ gs_gstate_alloc(gs_memory_t * mem)
     if (code < 0)
         goto fail;
     gs_setalpha(pgs, 1.0);
+    gs_setfillconstantalpha(pgs, 1.0);
+    gs_setstrokeconstantalpha(pgs, 1.0);
+    gs_setalphaisshape(pgs, false);
     gs_settransfer(pgs, gs_identity_transfer);
     gs_setflat(pgs, 1.0);
     gs_setfilladjust(pgs, 0.3, 0.3);
@@ -351,6 +354,14 @@ gs_gsave_for_save(gs_gstate * pgs, gs_gstate ** psaved)
     /* Cut the stack so we can't grestore past here. */
     *psaved = pgs->saved;
     pgs->saved = 0;
+
+    code = gs_gsave(pgs);
+    if (code < 0) {
+        pgs->saved = *psaved;
+        *psaved = NULL;
+        gs_grestore(pgs);
+        return code;
+    }
     return code;
 fail:
     if (new_cpath)
@@ -366,7 +377,6 @@ gs_grestore_only(gs_gstate * pgs)
     gs_gstate tmp_gstate;
     void *pdata = pgs->client_data;
     void *sdata;
-    bool prior_overprint = pgs->overprint;
 
     if_debug2m('g', pgs->memory, "[g]grestore 0x%lx, level was %d\n",
                (ulong) saved, pgs->level);
@@ -388,11 +398,6 @@ gs_grestore_only(gs_gstate * pgs)
     *saved = tmp_gstate;            /* restore "freed" state (pointers zeroed after contents freed) */
     gs_free_object(pgs->memory, saved, "gs_grestore");
 
-    /* update the overprint compositor, if necessary */
-    if (prior_overprint || pgs->overprint)
-    {
-        return gs_do_set_overprint(pgs);
-    }
     return 0;
 }
 
@@ -521,12 +526,7 @@ gs_setgstate(gs_gstate * pgs, const gs_gstate * pfrom)
     pgs->view_clip = view_clip;
     pgs->show_gstate =
         (pgs->show_gstate == pfrom ? pgs : saved_show);
-
-    /* update the overprint compositor, unconditionally. Unlike grestore, this */
-    /* may skip over states where overprint was set, so the prior state can    */
-    /* not be relied on to avoid this call. setgstate is not as commonly used  */
-    /* as grestore, so the overhead of the compositor call is acceptable.      */
-    return(gs_do_set_overprint(pgs));
+    return 0;
 }
 
 /* Get the allocator pointer of a graphics state. */
@@ -632,21 +632,39 @@ gs_do_set_overprint(gs_gstate * pgs)
 
     if (cs_num_components(pcs) < 0 && pcc->pattern != 0)
         code = pcc->pattern->type->procs.set_color(pcc, pgs);
-    else
+    else {
+        gx_device* dev = pgs->device;
+        cmm_dev_profile_t* dev_profile;
+
+        dev_proc(dev, get_profile)(dev, &dev_profile);
+        if (!dev_profile->sim_overprint || dev_profile->device_profile[0]->data_cs != gsCMYK)
+            return code;
+
+        /* The spaces that do not allow opm (e.g. ones that are not ICC or DeviceCMYK)
+           will blow away any true setting later. But we have to be prepared
+           in case this is a CMYK ICC space for example. Hence we set effective mode
+           to mode here (Bug 698721)*/
+        pgs->color[0].effective_opm = pgs->overprint_mode;
+
+        if_debug2m(gs_debug_flag_overprint, pgs->memory,
+            "[overprint] gs_do_set_overprint. Preset effective mode. pgs->color[0].effective_opm = %d pgs->color[1].effective_opm = %d\n",
+            pgs->color[0].effective_opm, pgs->color[1].effective_opm);
+
         pcs->type->set_overprint(pcs, pgs);
+    }
     return code;
 }
 
-/* setoverprint */
+/* setoverprint (non-stroke case) interpreter code
+   ensures that this is called when appropriate. This
+   should only be coming when we are doing PS files.
+   As they don't have separate stroke and fill overprint
+   controls */
 void
 gs_setoverprint(gs_gstate * pgs, bool ovp)
 {
-    bool    prior_ovp = pgs->overprint;
-
     pgs->overprint = ovp;
     pgs->stroke_overprint = ovp;
-    if (prior_ovp != ovp)
-        (void)gs_do_set_overprint(pgs);
 }
 
 /* currentoverprint */
@@ -674,11 +692,7 @@ gs_currentstrokeoverprint(const gs_gstate * pgs)
 void
 gs_setfilloverprint(gs_gstate * pgs, bool ovp)
 {
-    bool    prior_ovp = pgs->overprint;
-
     pgs->overprint = ovp;
-    if (prior_ovp != ovp)
-        (void)gs_do_set_overprint(pgs);
 }
 
 /* currentstrokeoverprint */
@@ -692,15 +706,11 @@ gs_currentfilloverprint(const gs_gstate * pgs)
 int
 gs_setoverprintmode(gs_gstate * pgs, int mode)
 {
-    int     prior_mode = pgs->effective_overprint_mode;
-    int     code = 0;
-
     if (mode < 0 || mode > 1)
         return_error(gs_error_rangecheck);
     pgs->overprint_mode = mode;
-    if (pgs->overprint && prior_mode != mode)
-        code = gs_do_set_overprint(pgs);
-    return code;
+
+    return 0;
 }
 
 /* currentoverprintmode */
@@ -715,7 +725,7 @@ gs_setcpsimode(gs_memory_t *mem, bool mode)
 {
     gs_lib_ctx_t *libctx = gs_lib_ctx_get_interp_instance(mem);
 
-    libctx->CPSI_mode = mode;
+    libctx->core->CPSI_mode = mode;
 }
 
 /* currentcpsimode */
@@ -724,7 +734,7 @@ gs_currentcpsimode(const gs_memory_t * mem)
 {
     gs_lib_ctx_t *libctx = gs_lib_ctx_get_interp_instance(mem);
 
-    return libctx->CPSI_mode;
+    return libctx->core->CPSI_mode;
 }
 
 /* The edgebuffer based scanconverter can only cope with values of 0
@@ -748,7 +758,7 @@ gs_setscanconverter(gs_gstate * gs, int converter)
 {
     gs_lib_ctx_t *libctx = gs_lib_ctx_get_interp_instance(gs->memory);
 
-    libctx->scanconverter = converter;
+    libctx->core->scanconverter = converter;
 
     sanitize_fill_adjust(gs);
 }
@@ -759,7 +769,7 @@ gs_getscanconverter(const gs_memory_t * mem)
 {
     gs_lib_ctx_t *libctx = gs_lib_ctx_get_interp_instance(mem);
 
-    return libctx->scanconverter;
+    return libctx->core->scanconverter;
 }
 
 /* setrenderingintent
@@ -835,12 +845,16 @@ gs_initgraphics(gs_gstate * pgs)
         if (pcs1 == NULL)
             return_error(gs_error_unknownerror);
 
-        if (pgs->color[0].color_space != NULL)
+        if (pgs->color[0].color_space != NULL) {
             gs_setcolorspace(pgs, pcs1);
-        else {
+            rc_decrement_cs(pcs1, "gs_initgraphics");
+        } else {
             pgs->color[0].color_space = pcs1;
             gs_setcolorspace(pgs, pcs1);
         }
+        code = gx_set_dev_color(pgs);
+        if (code < 0)
+            return code;
 
         gs_swapcolors_quick(pgs); /* To color 1 */
 
@@ -848,41 +862,56 @@ gs_initgraphics(gs_gstate * pgs)
         if (pcs2 == NULL)
             return_error(gs_error_unknownerror);
 
-        if (pgs->color[0].color_space != NULL)
+        if (pgs->color[0].color_space != NULL) {
             gs_setcolorspace(pgs, pcs2);
-        else {
+            rc_decrement_cs(pcs2, "gs_initgraphics");
+        } else {
             pgs->color[0].color_space = pcs2;
             gs_setcolorspace(pgs, pcs2);
         }
+        code = gx_set_dev_color(pgs);
 
         gs_swapcolors_quick(pgs); /* To color 0 */
+
+        if (code < 0)
+            return code;
+
     } else {
         gs_color_space  *pcs1, *pcs2;
 
-        pcs1 = gs_cspace_new_ICC(pgs->memory, pgs, 1);;
+        pcs1 = gs_cspace_new_ICC(pgs->memory, pgs, 1);
         if (pcs1 == NULL)
             return_error(gs_error_unknownerror);
 
-        if (pgs->color[0].color_space != NULL)
+        if (pgs->color[0].color_space != NULL) {
             gs_setcolorspace(pgs, pcs1);
-        else {
+            rc_decrement_cs(pcs1, "gs_initgraphics");
+        } else {
             pgs->color[0].color_space = pcs1;
             gs_setcolorspace(pgs, pcs1);
         }
+        code = gx_set_dev_color(pgs);
+        if (code < 0)
+            return code;
 
         gs_swapcolors_quick(pgs); /* To color 1 */
-        pcs2 = gs_cspace_new_ICC(pgs->memory, pgs, 1);;
+        pcs2 = gs_cspace_new_ICC(pgs->memory, pgs, 1);
         if (pcs2 == NULL)
             return_error(gs_error_unknownerror);
 
-        if (pgs->color[0].color_space != NULL)
+        if (pgs->color[0].color_space != NULL) {
             gs_setcolorspace(pgs, pcs2);
-        else {
+            rc_decrement_cs(pcs2, "gs_initgraphics");
+        } else {
             pgs->color[0].color_space = pcs2;
             gs_setcolorspace(pgs, pcs2);
         }
+        code = gx_set_dev_color(pgs);
 
         gs_swapcolors_quick(pgs); /* To color 0 */
+
+        if (code < 0)
+            return code;
     }
     pgs->in_cachedevice = 0;
 
@@ -1179,11 +1208,11 @@ gstate_alloc(gs_memory_t * mem, client_name_t cname, const gs_gstate * pfrom)
 
 /* Copy the dash pattern from one gstate to another. */
 static int
-gstate_copy_dash(gs_gstate * pto, const gs_gstate * pfrom)
+gstate_copy_dash(gs_memory_t *mem, gx_dash_params *dash , const gs_gstate * pfrom)
 {
-    return gs_setdash(pto, pfrom->line_params.dash.pattern,
+    return gx_set_dash(dash, pfrom->line_params.dash.pattern,
                       pfrom->line_params.dash.pattern_size,
-                      pfrom->line_params.dash.offset);
+                      pfrom->line_params.dash.offset, mem);
 }
 
 /* Clone an existing graphics state. */
@@ -1196,28 +1225,35 @@ gstate_clone(gs_gstate * pfrom, gs_memory_t * mem, client_name_t cname,
 {
     gs_gstate *pgs = gstate_alloc(mem, cname, pfrom);
     gs_gstate_parts parts;
+    void *pdata = NULL;
+    gx_dash_params dash;
 
-    if (pgs == 0)
+    if (pgs == NULL)
         return 0;
     GSTATE_ASSIGN_PARTS(&parts, pgs);
-    *pgs = *pfrom;
-    /* Copy the dash pattern if necessary. */
-    if (pgs->line_params.dash.pattern) {
-        int code;
+    if (pfrom->client_data != NULL) {
+        pdata = (*pfrom->client_procs.alloc) (mem);
 
-        pgs->line_params.dash.pattern = 0;      /* force allocation */
-        code = gstate_copy_dash(pgs, pfrom);
-        if (code < 0)
-            goto fail;
-    }
-    if (pgs->client_data != 0) {
-        void *pdata = pgs->client_data = (*pgs->client_procs.alloc) (mem);
-
-        if (pdata == 0 ||
-         gstate_copy_client_data(pgs, pdata, pfrom->client_data, reason) < 0
+        if (pdata == NULL ||
+         gstate_copy_client_data(pfrom, pdata, pfrom->client_data, reason) < 0
             )
             goto fail;
     }
+    /* Copy the dash and dash pattern if necessary. */
+    dash = gs_currentlineparams_inline(pfrom)->dash;
+    if (pfrom->line_params.dash.pattern) {
+        int code;
+
+        dash.pattern = NULL; /* Ensures a fresh allocation */
+        code = gstate_copy_dash(mem, &dash, pfrom);
+        if (code < 0)
+            goto fail;
+    }
+    *pgs = *pfrom;
+    pgs->client_data = pdata;
+    gs_currentlineparams_inline(pgs)->dash = dash;
+    pgs->memory = mem;
+
     gs_gstate_copied(pgs);
     /* Don't do anything to clip_stack. */
 
@@ -1242,6 +1278,8 @@ gstate_clone(gs_gstate * pfrom, gs_memory_t * mem, client_name_t cname,
     cs_adjust_counts_icc(pgs, 1);
     return pgs;
   fail:
+    if (pdata != NULL)
+        (*pfrom->client_procs.free) (pdata, mem);
     memset(pgs->color, 0, 2*sizeof(gs_gstate_color));
     gs_free_object(mem, pgs->line_params.dash.pattern, cname);
     GSTATE_ASSIGN_PARTS(pgs, &parts);
@@ -1320,7 +1358,8 @@ gstate_copy(gs_gstate * pto, const gs_gstate * pfrom,
     GSTATE_ASSIGN_PARTS(&parts, pto);
     /* Copy the dash pattern if necessary. */
     if (pfrom->line_params.dash.pattern || pto->line_params.dash.pattern) {
-        int code = gstate_copy_dash(pto, pfrom);
+        int code = gstate_copy_dash(pto->memory,
+                             &(gs_currentlineparams_inline(pto)->dash), pfrom);
 
         if (code < 0)
             return code;
@@ -1399,8 +1438,13 @@ gs_id gx_get_clip_path_id(gs_gstate *pgs)
     return pgs->clip_path->id;
 }
 
-void gs_swapcolors_quick(gs_gstate *pgs)
+void gs_swapcolors_quick(const gs_gstate *cpgs)
 {
+    union {
+        const gs_gstate *cpgs;
+        gs_gstate *pgs;
+    } const_breaker;
+    gs_gstate *pgs;
     struct gx_cie_joint_caches_s *tmp_cie;
     gs_devicen_color_map          tmp_ccm;
     gs_client_color              *tmp_cc;
@@ -1408,6 +1452,11 @@ void gs_swapcolors_quick(gs_gstate *pgs)
     gx_device_color              *tmp_dc;
     gs_color_space               *tmp_cs;
 
+    /* Break const just once, neatly, here rather than
+     * hackily in every caller. */
+    const_breaker.cpgs = cpgs;
+    pgs = const_breaker.pgs;
+
     tmp_cc               = pgs->color[0].ccolor;
     pgs->color[0].ccolor = pgs->color[1].ccolor;
     pgs->color[1].ccolor = tmp_cc;
@@ -1420,6 +1469,11 @@ void gs_swapcolors_quick(gs_gstate *pgs)
     pgs->color[0].color_space = pgs->color[1].color_space;
     pgs->color[1].color_space = tmp_cs;
 
+    /* Overprint and effective_op vary with stroke/fill and cs */
+    tmp                         = pgs->color[0].effective_opm;
+    pgs->color[0].effective_opm = pgs->color[1].effective_opm;
+    pgs->color[1].effective_opm = tmp;
+
     /* Swap the bits of the gs_gstate that depend on the current color */
     tmp_cie                   = pgs->cie_joint_caches;
     pgs->cie_joint_caches     = pgs->cie_joint_caches_alt;
@@ -1429,41 +1483,5 @@ void gs_swapcolors_quick(gs_gstate *pgs)
     pgs->color_component_map     = pgs->color_component_map_alt;
     pgs->color_component_map_alt = tmp_ccm;
 
-    tmp                = pgs->overprint;
-    pgs->overprint     = pgs->stroke_overprint;
-    pgs->stroke_overprint = tmp;
-
-    tmp                     = pgs->overprint_mode;
-    pgs->overprint_mode     = pgs->overprint_mode_alt;
-    pgs->overprint_mode_alt = tmp;
-
-    tmp                               = pgs->effective_overprint_mode;
-    pgs->effective_overprint_mode     = pgs->effective_overprint_mode_alt;
-    pgs->effective_overprint_mode_alt = tmp;
-
-}
-
-int gs_swapcolors(gs_gstate *pgs)
-{
-    int prior_overprint = pgs->overprint;
-
-    gs_swapcolors_quick(pgs);
-
-    /* The following code will only call gs_do_set_overprint when we
-     * have a change:
-     * if ((prior_overprint != pgs->overprint) ||
-     *    ((prior_mode != pgs->effective_overprint_mode) &&
-     *     (pgs->overprint)))
-     *    return gs_do_set_overprint(pgs);
-     * Sadly, that's no good, as we need to call when we have swapped
-     * image space types too (separation <-> non separation for example).
-     *
-     * So instead, we call whenever at least one of them had overprint
-     * turned on.
-     */
-    if (prior_overprint || pgs->overprint)
-    {
-        return gs_do_set_overprint(pgs);
-    }
-    return 0;
+    pgs->is_fill_color = !(pgs->is_fill_color);	/* used by overprint for fill_stroke */
 }
