@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -31,6 +31,7 @@
 #include "gxtext.h"
 #include "gzstate.h"
 #include "gsutil.h"
+#include "gxdevsop.h"
 
 /* GC descriptors */
 public_st_gs_text_params();
@@ -149,6 +150,9 @@ gx_device_text_begin(gx_device * dev, gs_gstate * pgs,
 static int
 gs_text_enum_init_dynamic(gs_text_enum_t *pte, gs_font *font)
 {
+    uint operation = pte->text.operation;
+    bool propagate_charpath = (operation & TEXT_DO_DRAW) != 0;
+
     pte->current_font = font;
     pte->index = 0;
     pte->xy_index = 0;
@@ -157,6 +161,24 @@ gs_text_enum_init_dynamic(gs_text_enum_t *pte, gs_font *font)
     pte->device_disabled_grid_fitting = 0;
     pte->outer_CID = GS_NO_GLYPH;
     pte->single_byte_space = false;
+    pte->cc = NULL;
+
+    /* We need to set the charpath_flag, as the PCL interpreter calls the
+     * graphics lib to do some measurement operations, which relies on the
+     * charpath_flag. See bug 700577 for more details. */
+    if (operation & TEXT_DO_ANY_CHARPATH)
+        pte->charpath_flag =
+            (operation & TEXT_DO_FALSE_CHARPATH ? cpm_false_charpath :
+             operation & TEXT_DO_TRUE_CHARPATH ? cpm_true_charpath :
+             operation & TEXT_DO_FALSE_CHARBOXPATH ? cpm_false_charboxpath :
+             operation & TEXT_DO_TRUE_CHARBOXPATH ? cpm_true_charboxpath :
+             operation & TEXT_DO_CHARWIDTH ? cpm_charwidth :
+             cpm_show /* can't happen */ );
+    else
+        pte->charpath_flag =
+        (propagate_charpath ? (pte->pgs ? pte->pgs->in_charpath : 0) : cpm_show);
+    pte->is_pure_color = pte->pgs ? gs_color_writes_pure(pte->pgs) : 0;
+
     return font->procs.init_fstack(pte, font);
 }
 int
@@ -227,6 +249,7 @@ gs_text_enum_copy_dynamic(gs_text_enum_t *pto, const gs_text_enum_t *pfrom,
 
     pto->current_font = pfrom->current_font;
     pto->index = pfrom->index;
+    pto->bytes_decoded = pfrom->bytes_decoded;
     pto->xy_index = pfrom->xy_index;
     pto->fstack.depth = depth;
     pto->FontBBox_as_Metrics2 = pfrom->FontBBox_as_Metrics2;
@@ -249,6 +272,12 @@ gs_text_begin(gs_gstate * pgs, const gs_text_params_t * text,
 {
     gx_clip_path *pcpath = 0;
     int code;
+    gs_overprint_params_t op_params = { 0 };
+    bool op_active = dev_proc(pgs->device, dev_spec_op)(pgs->device, gxdso_overprint_active, NULL, 0);
+    bool text_op_fill = ((pgs->overprint || (!pgs->overprint && op_active)) &&
+        (pgs->text_rendering_mode == 0));
+    bool text_op_stroke = ((pgs->stroke_overprint || (!pgs->stroke_overprint && op_active)) &&
+        (pgs->text_rendering_mode == 1));
 
     /*
      * Detect nocurrentpoint now, even if the string is empty, for Adobe
@@ -263,6 +292,7 @@ gs_text_begin(gs_gstate * pgs, const gs_text_params_t * text,
        to compute a text width.
        Note : FontType 3 throws error in setcachedevice. */
     if (pgs->font->FontType != ft_user_defined &&
+        pgs->font->FontType != ft_PDF_user_defined &&
         pgs->font->FontType != ft_GL2_stick_user_defined &&
         pgs->font->FontType != ft_PCL_user_defined &&
         pgs->font->FontMatrix.xx == 0 && pgs->font->FontMatrix.xy == 0 &&
@@ -279,7 +309,7 @@ gs_text_begin(gs_gstate * pgs, const gs_text_params_t * text,
        Unfortunately we can't effectively know a leaf font type here,
        so we load the color unconditionally . */
     /* Processing a text object operation */
-    dev_proc(pgs->device, set_graphics_type_tag)(pgs->device, GS_TEXT_TAG);
+    ensure_tag_is_set(pgs, pgs->device, GS_TEXT_TAG);	/* NB: may unset_dev_color */
 
     code = gx_set_dev_color(pgs);
     if (code != 0)
@@ -287,6 +317,40 @@ gs_text_begin(gs_gstate * pgs, const gs_text_params_t * text,
     code = gs_gstate_color_load(pgs);
     if (code < 0)
         return code;
+
+    if (text_op_stroke) {
+        if_debug0m(gs_debug_flag_overprint, pgs->memory,
+            "[overprint] Stroke Text Overprint\n");
+        code = gs_do_set_overprint(pgs);
+        if (code < 0)
+            return code;
+    } else if (text_op_fill) {
+        if_debug0m(gs_debug_flag_overprint, pgs->memory,
+            "[overprint] Fill Text Overprint\n");
+        code = gs_do_set_overprint(pgs);
+        if (code < 0)
+            return code;
+    }
+
+    /* If overprint is true, push the compositor action to set the op device state */
+    if ((pgs->overprint  && pgs->text_rendering_mode == 0) ||
+        (pgs->stroke_overprint && pgs->text_rendering_mode == 1) ||
+        op_active) {
+        gx_device* dev = pgs->device;
+        cmm_dev_profile_t* dev_profile;
+
+        dev_proc(dev, get_profile)(dev, &dev_profile);
+        if (dev_profile->sim_overprint && dev_profile->device_profile[0]->data_cs == gsCMYK) {
+            if (pgs->text_rendering_mode == 0) {
+                op_params.op_state = OP_STATE_FILL;
+                gs_gstate_update_overprint(pgs, &op_params);
+            } else if (pgs->text_rendering_mode == 1) {
+                op_params.op_state = OP_STATE_STROKE;
+                gs_gstate_update_overprint(pgs, &op_params);
+            }
+        }
+    }
+
     pgs->device->sgr.stroke_stored = false;
     return gx_device_text_begin(pgs->device, pgs,
                                 text, pgs->font, pgs->path,
@@ -310,7 +374,7 @@ gs_text_update_dev_color(gs_gstate * pgs, gs_text_enum_t * pte)
      * well.
      */
     /* Processing a text object operation */
-    dev_proc(pgs->device, set_graphics_type_tag)(pgs->device, GS_TEXT_TAG);
+    ensure_tag_is_set(pgs, pgs->device, GS_TEXT_TAG);	/* NB: may unset_dev_color */
 
     if (pte->pdcolor != 0) {
         int code = gx_set_dev_color(pgs);
