@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -36,6 +36,7 @@ static dev_proc_open_device(accum_open_device);
 static dev_proc_close_device(accum_close);
 static dev_proc_fill_rectangle(accum_fill_rectangle);
 static dev_proc_dev_spec_op(accum_dev_spec_op);
+static dev_proc_get_clipping_box(accum_get_clipping_box);
 
 /* GC information */
 extern_st(st_clip_list);
@@ -94,7 +95,7 @@ static const gx_device_cpath_accum gs_cpath_accum_device =
   gx_default_end_image,
   NULL,
   NULL,
-  gx_get_largest_clipping_box,
+  accum_get_clipping_box,
   gx_default_begin_typed_image,
   NULL,
   NULL,
@@ -160,6 +161,24 @@ gx_cpath_accum_set_cbox(gx_device_cpath_accum * padev,
     }
 }
 
+static void
+accum_get_clipping_box(gx_device *dev, gs_fixed_rect *pbox)
+{
+    gx_device_cpath_accum * padev = (gx_device_cpath_accum *)dev;
+
+    if (padev->list.transpose) {
+        pbox->p.x = int2fixed(padev->clip_box.p.y);
+        pbox->p.y = int2fixed(padev->clip_box.p.x);
+        pbox->q.x = int2fixed(padev->clip_box.q.y+1)-1;
+        pbox->q.y = int2fixed(padev->clip_box.q.x+1)-1;
+    } else {
+        pbox->p.x = int2fixed(padev->clip_box.p.x);
+        pbox->p.y = int2fixed(padev->clip_box.p.y);
+        pbox->q.x = int2fixed(padev->clip_box.q.x+1)-1;
+        pbox->q.y = int2fixed(padev->clip_box.q.y+1)-1;
+    }
+}
+
 /* Finish accumulating a clipping path. */
 /* NB: After this the padev bbox will be restored to "normal" untransposed */
 int
@@ -208,8 +227,9 @@ gx_cpath_accum_end(gx_device_cpath_accum * padev, gx_clip_path * pcpath)
     gx_cpath_set_outer_box(&apath);
     apath.path_valid = false;
     apath.id = gs_next_ids(padev->list_memory, 1);	/* path changed => change id */
-    gx_cpath_assign_free(pcpath, &apath);
-    return 0;
+    apath.cached = NULL;
+    code = gx_cpath_assign_free(pcpath, &apath);
+    return code;
 }
 
 /* Discard an accumulator in case of error. */
@@ -576,7 +596,6 @@ top:
             rsplit->ymax = ye;
             rsplit = rsplit->prev;
         }
-        ymax = ye;
     }
     /* Now ye = ymax.  If necessary, split off the part of the */
     /* existing band that is below the new band. */
