@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -150,7 +150,7 @@ clist_fill_mask(gx_device * dev,
     int y0;
     byte copy_op =
         (depth > 1 ? cmd_op_copy_color_alpha :
-         cmd_op_copy_mono_planes + cmd_copy_ht_color);  /* Plane not needed here */
+         cmd_op_copy_mono_planes);  /* Plane not needed here */
     bool slow_rop =
         cmd_slow_rop(dev, lop_know_S_0(lop), pdcolor) ||
         cmd_slow_rop(dev, lop_know_S_1(lop), pdcolor);
@@ -204,29 +204,29 @@ clist_fill_mask(gx_device * dev,
 
         RECT_STEP_INIT(re);
         code = cmd_update_lop(cdev, re.pcls, lop);
-        if (code < 0 && SET_BAND_CODE(code))
-            return re.band_code;
+        if (code < 0)
+            return code;
         if (depth > 1 && !re.pcls->color_is_alpha) {
             byte *dp;
 
             code = set_cmd_put_op(&dp, cdev, re.pcls, cmd_opv_set_copy_alpha, 1);
-            if (code < 0 && SET_BAND_CODE(code))
-                return re.band_code;
+            if (code < 0)
+                return code;
             re.pcls->color_is_alpha = 1;
         }
         code = cmd_do_write_unknown(cdev, re.pcls, clip_path_known);
         if (code >= 0)
             code = cmd_do_enable_clip(cdev, re.pcls, pcpath != NULL);
-        if (code < 0 && SET_BAND_CODE(code))
-            return re.band_code;
+        if (code < 0)
+            return code;
         code = cmd_put_drawing_color(cdev, re.pcls, pdcolor, &re,
-                                     devn_not_tile);
+                                     devn_not_tile_fill);
         if (code == gs_error_unregistered)
             return code;
         if (depth > 1 && code >= 0)
             code = cmd_set_color1(cdev, re.pcls, pdcolor->colors.pure);
-        if (code < 0 && SET_BAND_CODE(code))
-            return re.band_code;
+        if (code < 0)
+            return code;
         re.pcls->color_usage.slow_rop |= slow_rop;
         /* Put it in the cache if possible. */
         if (!cls_has_tile_id(cdev, re.pcls, id, offset_temp)) {
@@ -240,8 +240,6 @@ clist_fill_mask(gx_device * dev,
             tile.id = id;
             tile.num_planes = 1;
             code = clist_change_bits(cdev, re.pcls, &tile, depth);
-            if (code == gs_error_VMerror && SET_BAND_CODE(code))	/* only VMerror exits here */
-                return re.band_code;
             if (code < 0) {
                 /* Something went wrong; just copy the bits. */
                 goto copy;
@@ -280,8 +278,8 @@ clist_fill_mask(gx_device * dev,
                     cmd_putxy(rect, &dp);
                 }
             }
-            if (code < 0 && SET_BAND_CODE(code))
-                return re.band_code;
+            if (code < 0)
+                return code;
             re.pcls->rect = rect;
         }
     } while ((re.y += re.height) < re.yend);
@@ -539,7 +537,8 @@ clist_begin_typed_image(gx_device * dev, const gs_gstate * pgs,
             indexed = false;
             num_components = gs_color_space_num_components(pcs);
         }
-        uses_color = pim->CombineWithColor && rop3_uses_T(pgs->log_op);
+        uses_color = pim->CombineWithColor &&
+                    (rop3_uses_T(pgs->log_op) || rop3_uses_S(pgs->log_op));
     }
     code = gx_image_enum_common_init((gx_image_enum_common_t *) pie,
                                      (const gs_data_image_t *) pim,
@@ -757,8 +756,6 @@ clist_begin_typed_image(gx_device * dev, const gs_gstate * pgs,
             goto use_default;
     }
     if (pim->Interpolate) {
-        if (gx_device_is_pattern_clist(dev))
-            goto use_default;
         pie->support.x = pie->support.y = MAX_ISCALE_SUPPORT + 1;
     } else {
         pie->support.x = pie->support.y = 0;
@@ -1001,7 +998,9 @@ clist_image_plane_data(gx_image_enum_common_t * info,
     sbox.p.y = (y0 = y_orig) - pie->support.y;
     sbox.q.x = pie->rect.q.x + pie->support.x;
     sbox.q.y = (y1 = pie->y += yh_used) + pie->support.y;
-    gs_bbox_transform(&sbox, &pie->matrix, &dbox);
+    code = gs_bbox_transform(&sbox, &pie->matrix, &dbox);
+    if (code < 0)
+        return code;
     /*
      * In order to keep the band list consistent, we must write out
      * the image data in precisely those bands whose begin_image
@@ -1108,8 +1107,8 @@ clist_image_plane_data(gx_image_enum_common_t * info,
                 code = cmd_do_enable_clip(cdev, re.pcls, pie->pcpath != NULL);
             if (code >= 0)
                 code = cmd_update_lop(cdev, re.pcls, lop);
-            if (code < 0 && SET_BAND_CODE(code))
-                return re.band_code;
+            if (code < 0)
+                return code;
             if (pie->uses_color) {
                 /* We want to write the color taking into account the entire image so */
                 /* we set re.rect_nbands from pie->ymin and pie->ymax so that we will */
@@ -1119,9 +1118,9 @@ clist_image_plane_data(gx_image_enum_common_t * info,
                 re.rect_nbands = ((pie->ymax + re.band_height - 1) / re.band_height) -
                                  ((pie->ymin) / re.band_height);
                 code = cmd_put_drawing_color(cdev, re.pcls, &pie->dcolor,
-                                             &re, devn_not_tile);
-                if (code < 0 && SET_BAND_CODE(code))
-                    return re.band_code;
+                                             &re, devn_not_tile_fill);
+                if (code < 0)
+                    return code;
             }
             if (entire_box.p.x != 0 || entire_box.p.y != 0 ||
                 entire_box.q.x != pie->image.Width ||
@@ -1135,8 +1134,8 @@ clist_image_plane_data(gx_image_enum_common_t * info,
             len = bp - pie->begin_image_command;
             code =
                 set_cmd_put_op(&dp, cdev, re.pcls, image_op, 1 + len);
-            if (code < 0 && SET_BAND_CODE(code))
-                return re.band_code;
+            if (code < 0)
+                return code;
             memcpy(dp + 1, pie->begin_image_command, len);
 
             /* Mark band's begin_image as known */
@@ -1206,8 +1205,8 @@ clist_image_plane_data(gx_image_enum_common_t * info,
                                                     bytes_per_plane, offsets,
                                                     xoff - xskip, nrows);
                     }
-                    if (code < 0 && SET_BAND_CODE(code))
-                        return re.band_code;
+                    if (code < 0)
+                        return code;
                     for (i = 0; i < num_planes; ++i)
                         offsets[i] += planes[i].raster * nrows;
                 }
@@ -1217,8 +1216,8 @@ clist_image_plane_data(gx_image_enum_common_t * info,
                     code = cmd_image_plane_data(cdev, re.pcls, planes, info,
                                                 bytes_per_plane, offsets,
                                                 xoff - xskip, nrows);
-                    if (code < 0 && SET_BAND_CODE(code))
-                        return re.band_code;
+                    if (code < 0)
+                        return code;
                     for (i = 0; i < num_planes; ++i)
                         offsets[i] += planes[i].raster * nrows;
                 }
@@ -1270,6 +1269,8 @@ clist_create_compositor(gx_device * dev,
     int code = pcte->type->procs.write(pcte, 0, &size, cdev);
     int temp_cropping_min, temp_cropping_max;
 
+    CMD_CHECK_LAST_OP_BLOCK_DEFINED(cdev);
+
     /* determine the amount of space required */
     if (code < 0 && code != gs_error_rangecheck)
         return code;
@@ -1281,8 +1282,12 @@ clist_create_compositor(gx_device * dev,
     if (code < 0)
         return code;
 
+    CMD_CHECK_LAST_OP_BLOCK_DEFINED(cdev);
+
     code = pcte->type->procs.get_cropping(pcte, &ry, &rheight, cdev->cropping_min, cdev->cropping_max);
 
+    CMD_CHECK_LAST_OP_BLOCK_DEFINED(cdev);
+
     if (code < 0)
         return code;
 
@@ -1372,8 +1377,8 @@ clist_create_compositor(gx_device * dev,
                 dp[2] = pcte->type->comp_id;
                 code = pcte->type->procs.write(pcte, dp + 3, &size_dummy, cdev);
             }
-            if (code < 0 && SET_BAND_CODE(code))
-                return re.band_code;
+            if (code < 0)
+                return code;
         } while ((re.y += re.height) < re.yend);
     }
     if (cropping_op == POPCROP) {
@@ -2038,8 +2043,8 @@ write_image_end_all(gx_device *dev, const clist_image_enum *pie)
         if (re.pcls->known & begin_image_known) {
             if_debug1m('L', dev->memory, "[L]image_end for band %d\n", re.band);
             code = set_cmd_put_op(&dp, cdev, re.pcls, cmd_opv_image_data, 2);
-            if (code < 0 && SET_BAND_CODE(code))
-                return re.band_code;
+            if (code < 0)
+                return code;
             dp[1] = 0;      /* EOD */
             re.pcls->known ^= begin_image_known;
         }
