@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -111,7 +111,10 @@ const gx_device_mask_clip gs_mask_clip_device =
   gx_forward_set_graphics_type_tag,
   mask_clip_strip_copy_rop2,
   mask_clip_strip_tile_rect_devn,
-  mask_clip_copy_alpha_hl_color
+  mask_clip_copy_alpha_hl_color,
+  NULL,
+  gx_default_transform_pixel_region,
+  gx_forward_fill_stroke_path,
  }
 };
 
