@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -32,9 +32,7 @@
 #include "gsicc_cache.h"
 #include "gxdevsop.h"
 
-#ifdef PACIFY_VALGRIND
 #include "valgrind.h"
-#endif
 
 extern dev_proc_open_device(pattern_clist_open_device);
 
@@ -203,6 +201,8 @@ const gx_device_procs gs_clist_device_procs = {
     clist_strip_tile_rect_devn,
     clist_copy_alpha_hl_color,
     clist_process_page,
+    gx_default_transform_pixel_region,
+    clist_fill_stroke_path,
 };
 
 /*------------------- Choose the implementation -----------------------
@@ -396,6 +396,7 @@ clist_init_data(gx_device * dev, byte * init_data, uint data_size)
     gx_device_memory bdev;
     gx_device *pbdev = (gx_device *)&bdev;
     int code;
+    int align = 1 << (target->log2_align_mod > log2_align_bitmap_mod ? target->log2_align_mod : log2_align_bitmap_mod);
 
     /* the clist writer has its own color info that depends upon the
        transparency group color space (if transparency exists).  The data that is
@@ -444,6 +445,13 @@ clist_init_data(gx_device * dev, byte * init_data, uint data_size)
             } else {
                 bits_size = cdev->space_params.band.tile_cache_size;
             }
+            /* The top of the tile_cache is the bottom of the imagable band buffer,
+             * which needs to be appropriately aligned. Because the band height is
+             * fixed, we must round *down* the size of the cache to a appropriate
+             * value. See clist_render_thread() and clist_rasterize_lines()
+             * for where the value is used.
+             */
+            bits_size = ROUND_DOWN(bits_size, align);
         } else {
             int adjusted;
             /*
@@ -452,6 +460,13 @@ clist_init_data(gx_device * dev, byte * init_data, uint data_size)
              */
             bits_size = clist_tile_cache_size(target, band_space);
             bits_size = min(bits_size, data_size >> 1);
+            /* The top of the tile_cache is the bottom of the imagable band buffer,
+             * which needs to be appropriately aligned. Because the band height is
+             * fixed, here we round up the size of the cache, since the band height
+             * is variable, and it should only be a few bytes. See clist_render_thread()
+             * and clist_rasterize_lines() for where the value is used.
+             */
+            bits_size = ROUND_UP(bits_size, align);
             band_height = gdev_mem_max_height(&bdev, band_width,
                               band_space - bits_size, page_uses_transparency);
             if (band_height == 0) {
@@ -563,7 +578,8 @@ clist_reset(gx_device * dev)
     cdev->cropping_level = 0;
     cdev->mask_id_count = cdev->mask_id = cdev->temp_mask_id = 0;
     cdev->icc_table = NULL;
-    cdev->icc_cache_cl = NULL;
+    cdev->op_fill_active = false;
+    cdev->op_stroke_active = false;
     return 0;
 }
 /*
@@ -623,8 +639,7 @@ clist_open_output_file(gx_device *dev)
     code = clist_init(dev);
     if (code < 0)
         return code;
-    strcpy(fmode, "w+");
-    strcat(fmode, gp_fmode_binary_suffix);
+    snprintf(fmode, sizeof(fmode), "w+%s", gp_fmode_binary_suffix);
     cdev->page_cfname[0] = 0;   /* create a new file */
     cdev->page_bfname[0] = 0;   /* ditto */
     clist_reset_page(cdev);
@@ -690,15 +705,22 @@ clist_open(gx_device *dev)
 
     code = clist_init(dev);
     if (code < 0)
-        return code;
+        goto errxit;
+
     cdev->icc_cache_list_len = 0;
     cdev->icc_cache_list = NULL;
     code = clist_open_output_file(dev);
     if ( code >= 0)
         code = clist_emit_page_header(dev);
-    if (code >= 0)
-       dev->is_open = save_is_open;
-
+    if (code >= 0) {
+        dev->is_open = save_is_open;
+        return code;		/* success */
+    }
+    /* fall through to clean up and return error code */
+errxit:
+    /* prevent leak */
+    gs_free_object(cdev->memory->non_gc_memory, cdev->cache_chunk, "free tile cache for clist");
+    cdev->cache_chunk = NULL;
     return code;
 }
 
@@ -752,9 +774,9 @@ clist_finish_page(gx_device *dev, bool flush)
     if (!CLIST_IS_WRITER((gx_device_clist *)dev)) {
         gx_device_clist_reader * const crdev =  &((gx_device_clist *)dev)->reader;
 
+        clist_teardown_render_threads(dev);
         gs_free_object(cdev->memory, crdev->color_usage_array, "clist_color_usage_array");
         crdev->color_usage_array = NULL;
-        clist_teardown_render_threads(dev);
 
        /* Free the icc table associated with this device.
            The threads that may have pointed to this were destroyed in
@@ -763,14 +785,16 @@ clist_finish_page(gx_device *dev, bool flush)
            be any issues. */
         clist_free_icc_table(crdev->icc_table, crdev->memory);
         crdev->icc_table = NULL;
-        rc_decrement(crdev->icc_cache_cl,"clist_finish_page");
-        crdev->icc_cache_cl = NULL;
     }
     if (flush) {
-        if (cdev->page_cfile != 0)
-            cdev->page_info.io_procs->rewind(cdev->page_cfile, true, cdev->page_cfname);
-        if (cdev->page_bfile != 0)
-            cdev->page_info.io_procs->rewind(cdev->page_bfile, true, cdev->page_bfname);
+        if (cdev->page_cfile != 0) {
+            code = cdev->page_info.io_procs->rewind(cdev->page_cfile, true, cdev->page_cfname);
+            if (code < 0) return code;
+        }
+        if (cdev->page_bfile != 0) {
+            code = cdev->page_info.io_procs->rewind(cdev->page_bfile, true, cdev->page_bfname);
+            if (code < 0) return code;
+        }
         cdev->page_info.bfile_end_pos = 0;
         clist_reset_page(cdev);
     } else {
@@ -959,7 +983,7 @@ clist_free_icc_table_contents(clist_icctable_t *icc_table)
     curr_entry = icc_table->head;
     for (k = 0; k < number_entries; k++) {
         next_entry = curr_entry->next;
-        rc_decrement(curr_entry->icc_profile, "clist_free_icc_table");
+        gsicc_adjust_profile_rc(curr_entry->icc_profile, -1, "clist_free_icc_table");
         gs_free_object(icc_table->memory, curr_entry, "clist_free_icc_table");
         curr_entry = next_entry;
     }
@@ -1006,7 +1030,7 @@ clist_icc_writetable(gx_device_clist_writer *cldev)
         curr_entry->serial_data.file_position = clist_icc_addprofile(cldev, curr_entry->icc_profile, &size_data);
         curr_entry->icc_profile->rend_is_valid = rend_is_valid;
         curr_entry->serial_data.size = size_data;
-        rc_decrement(curr_entry->icc_profile, "clist_icc_writetable");
+        gsicc_adjust_profile_rc(curr_entry->icc_profile, -1, "clist_icc_writetable");
         curr_entry->icc_profile = NULL;
         curr_entry = curr_entry->next;
     }
@@ -1098,7 +1122,7 @@ clist_icc_addentry(gx_device_clist_writer *cdev, int64_t hashcode_in, cmm_profil
         entry->serial_data.file_position = -1;
         entry->icc_profile = icc_profile;
         entry->render_is_valid = icc_profile->rend_is_valid;
-        rc_increment(icc_profile);
+        gsicc_adjust_profile_rc(icc_profile, 1, "clist_icc_addentry");
         icc_table = gs_alloc_struct(stable_mem, clist_icctable_t,
                                     &st_clist_icctable, "clist_icc_addentry");
         if (icc_table == NULL)
@@ -1138,7 +1162,7 @@ clist_icc_addentry(gx_device_clist_writer *cdev, int64_t hashcode_in, cmm_profil
         entry->serial_data.file_position = -1;
         entry->icc_profile = icc_profile;
         entry->render_is_valid = icc_profile->rend_is_valid;
-        rc_increment(icc_profile);
+        gsicc_adjust_profile_rc(icc_profile, 1, "clist_icc_addentry");
         icc_table->final->next = entry;
         icc_table->final = entry;
         icc_table->tablesize++;
@@ -1200,7 +1224,7 @@ clist_writer_push_no_cropping(gx_device_clist_writer *cdev)
 
     if (buf == NULL)
         return_error(gs_error_VMerror);
-    if_debug4m('v', cdev->memory, "[v]push cropping[%d], min=%d, max=%d, buf=%0x\n",
+    if_debug4m('v', cdev->memory, "[v]push cropping[%d], min=%d, max=%d, buf=%p\n",
                cdev->cropping_level, cdev->cropping_min, cdev->cropping_max, buf);
     buf->next = cdev->cropping_stack;
     cdev->cropping_stack = buf;
@@ -1237,7 +1261,7 @@ clist_writer_pop_cropping(gx_device_clist_writer *cdev)
     cdev->temp_mask_id = buf->temp_mask_id;
     cdev->cropping_stack = buf->next;
     cdev->cropping_level--;
-    if_debug4m('v', cdev->memory, "[v]pop cropping[%d] min=%d, max=%d, buf=%0x\n",
+    if_debug4m('v', cdev->memory, "[v]pop cropping[%d] min=%d, max=%d, buf=%p\n",
                cdev->cropping_level, cdev->cropping_min, cdev->cropping_max, buf);
     gs_free_object(cdev->memory, buf, "clist_writer_transparency_pop");
     return 0;
