@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -314,13 +314,14 @@ clist_fillpage(gx_device * dev, gs_gstate *pgs, gx_drawing_color *pdcolor)
     int code;
 
     /* flush previous contents */
-    if ((code = clist_close_writer_and_init_reader(cldev) < 0) ||
-        (code = clist_finish_page(dev, true)) < 0)
+    if ((code = clist_close_writer_and_init_reader(cldev) < 0))
+        return code;;
+    if ((code = clist_finish_page(dev, true)) < 0)
         return code;;
 
     pcls = cdev->states; /* Use any. */
 
-    code = cmd_put_drawing_color(cdev, pcls, pdcolor, NULL, devn_not_tile);
+    code = cmd_put_drawing_color(cdev, pcls, pdcolor, NULL, devn_not_tile_fill);
     if (code >= 0)
         code = cmd_write_page_rect_cmd(cdev, cmd_op_fill_rect);
     return code;
@@ -413,7 +414,7 @@ clist_fill_rectangle_hl_color(gx_device *dev, const gs_fixed_rect *rect,
         re.pcls->color_usage.or |= color_usage;
         code = cmd_disable_lop(cdev, re.pcls);
         code = cmd_put_drawing_color(cdev, re.pcls, pdcolor, &re,
-                                     devn_not_tile);
+                                     devn_not_tile_fill);
         if (code >= 0) {
             code = cmd_write_rect_hl_cmd(cdev, re.pcls, cmd_op_fill_rect_hl,
                                          rx, re.y, rwidth, re.height, false);
@@ -425,21 +426,6 @@ clist_fill_rectangle_hl_color(gx_device *dev, const gs_fixed_rect *rect,
     return 0;
 }
 
-static void update_color_use_frac_array(uchar num_colors, const frac31 *color,
-    cmd_rects_enum_t *re)
-{
-    uchar k;
-
-    if (color == NULL)
-        return;
-
-    for (k = 0; k < num_colors; k++){
-        if (color[k] != 0) {
-            re->pcls->color_usage.or |= (1<<k);
-        }
-    }
-}
-
 static inline int
 clist_write_fill_trapezoid(gx_device * dev,
     const gs_fixed_edge *left, const gs_fixed_edge *right,
@@ -506,7 +492,7 @@ clist_write_fill_trapezoid(gx_device * dev,
         RECT_STEP_INIT(re);
         if (pdcolor != NULL) {
             code = cmd_put_drawing_color(cdev, re.pcls, pdcolor, &re,
-                                         devn_not_tile);
+                                         devn_not_tile_fill);
             if (code == gs_error_unregistered)
                 return code;
             if (code < 0) {
@@ -517,14 +503,11 @@ clist_write_fill_trapezoid(gx_device * dev,
         } else {
             /* Even with pdcolor NULL, we may still have colors packed in
                c0, c1, c2 or c3 */
-            update_color_use_frac_array(dev->color_info.num_components, c0, &re);
-            update_color_use_frac_array(dev->color_info.num_components, c1, &re);
-            update_color_use_frac_array(dev->color_info.num_components, c2, &re);
-            update_color_use_frac_array(dev->color_info.num_components, c3, &re);
+            re.pcls->color_usage.or |= gx_color_usage_all(cdev);
             code = 0;
         }
         if (code >= 0) {
-            /* Dont't want to shorten the trapezoid by the band boundary,
+            /* Don't want to shorten the trapezoid by the band boundary,
                keeping in mind a further optimization with writing same data to all bands. */
             code = cmd_write_trapezoid_cmd(cdev, re.pcls, cmd_opv_fill_trapezoid, left, right,
                                       ybot, ytop, options, fa, c0, c1, c2, c3);
@@ -635,6 +618,10 @@ clist_dev_spec_op(gx_device *pdev, int dev_spec_op, void *data, int size)
             ibox->q.y = cwdev->cropping_max;
         return 0;
     }
+    if (dev_spec_op == gxdso_overprint_active) {
+        gx_device_clist_writer* cwdev = &((gx_device_clist*)pdev)->writer;
+        return cwdev->op_fill_active || cwdev->op_stroke_active;
+    }
     /* forward to the appropriate super class */
     if (cdev->is_printer)
         return gdev_prn_forwarding_dev_spec_op(pdev, dev_spec_op, data, size);
@@ -920,6 +907,8 @@ clist_copy_planes(gx_device * dev,
     gx_bitmap_id orig_id = id;
     cmd_rects_enum_t re;
     int bpc = dev->color_info.depth / dev->color_info.num_components;
+    int pix_in_byte = bpc <= 8 ? 8/bpc : 1;
+    int byte_in_pix = bpc >= 8 ? bpc/8 : 1;
 
     if (rwidth <= 0 || rheight <= 0)
         return 0;
@@ -956,9 +945,9 @@ clist_copy_planes(gx_device * dev,
         byte *dp, *dp2;
         uint csize;
         byte op = (byte) cmd_op_copy_mono_planes;
-        int dx = data_x % (8/bpc);
+        int dx = data_x % pix_in_byte;
         int w1 = dx + rwidth;
-        const byte *row = data + (re.y - y0) * raster + (data_x / (8/bpc));
+        const byte *row = data + (re.y - y0) * raster + (data_x * byte_in_pix / pix_in_byte);
         int bytes_row = ((w1*bpc+7)/8 + 7) & ~7;
         int maxheight = data_bits_size / bytes_row / cdev->color_info.num_components;
         int plane;
@@ -1231,7 +1220,7 @@ clist_copy_alpha_hl_color(gx_device * dev, const byte * data, int data_x,
             re.pcls->color_is_alpha = 1;
         }
         /* Set the color */
-        code = cmd_put_drawing_color(cdev, re.pcls, pdcolor, &re, devn_not_tile);
+        code = cmd_put_drawing_color(cdev, re.pcls, pdcolor, &re, devn_not_tile_fill);
 copy:{
             gx_cmd_rect rect;
             int rsize;
@@ -1557,7 +1546,7 @@ clist_strip_copy_rop2(gx_device * dev,
                         tile_with_id.id = gs_next_ids(dev->memory, 1);
                         tiles = &tile_with_id;
                     }
-                    code = clist_change_tile(cdev, re.pcls, tiles,
+                    code = clist_change_bits(cdev, re.pcls, tiles,
                                             (tcolors != 0 ? 1 :
                                                  cdev->clist_color_info.depth));
                     if (code < 0 && !(code == gs_error_limitcheck))
