@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -37,6 +37,15 @@
 #include "gsicc.h"
 #include "gxdevsop.h"
 
+/* If enabled, this makes use of the alternate transform
+   ICC profile for mapping separation and
+   DeviceN colorants from CMYK to output
+   ICC color space iff the profiles make
+   sense.  We should probably make this yet
+   another color command line option. Disabling
+   it for now for the current release. */
+#define USE_ALT_MAP 0
+
 /* Structure descriptor */
 public_st_device_color();
 static
@@ -561,9 +570,6 @@ gx_remap_color(gs_gstate * pgs)
                                           pcs, gs_currentdevicecolor_inline(pgs),
                                           (gs_gstate *) pgs, pgs->device,
                                           gs_color_select_texture);
-    /* if overprint mode is in effect, update the overprint information */
-   if (code >= 0 && pgs->overprint)
-        code = gs_do_set_overprint(pgs);
     return code;
 }
 
@@ -598,7 +604,6 @@ gx_device_color * pdc, const gs_gstate * pgs, gx_device * dev,
 gs_color_select_t select)
 {
     gx_color_value device_values[GX_DEVICE_COLOR_MAX_COMPONENTS];
-    const gs_separation_name name = pcs->params.separation.sep_name;
     byte *pname;
     uint name_size;
     gsicc_rendering_param_t rendering_params;
@@ -623,13 +628,11 @@ gs_color_select_t select)
     rendering_params.cmm = gsCMM_DEFAULT;
 
     if (type == gs_color_space_index_Separation) {
-        pcs->params.separation.get_colorname_string(pgs->memory, name,
-            &pname, &name_size);
-        named_color_sep.colorant_name = (char*)pname;
-        named_color_sep.name_size = name_size;
+        named_color_sep.colorant_name = pcs->params.separation.sep_name;
+        named_color_sep.name_size = strlen(pcs->params.separation.sep_name);
         named_color_ptr = &named_color_sep;
     } else if (type == gs_color_space_index_DeviceN) {
-        const gs_separation_name *names = pcs->params.device_n.names;
+        char **names = pcs->params.device_n.names;
         num_src_comps = pcs->params.device_n.num_components;
         /* Allocate and initialize name structure */
         named_color_devn =
@@ -639,8 +642,8 @@ gs_color_select_t select)
         if (named_color_devn == NULL)
             return false; /* Clearly a bigger issue. But lets not end here */
         for (k = 0; k < num_src_comps; k++) {
-            pcs->params.device_n.get_colorname_string(dev->memory, names[k],
-                &pname, &name_size);
+            pname = (byte *)names[k];
+            name_size = strlen(names[k]);
             named_color_devn[k].colorant_name = (char*)pname;
             named_color_devn[k].name_size = name_size;
         }
@@ -691,7 +694,7 @@ gs_color_select_t select)
                         return false;
                     dev_profile->spotnames->equiv_cmyk_set = true;
                 }
-                gx_remap_concrete_devicen(conc, pdc, pgs, dev, select);
+                gx_remap_concrete_devicen(conc, pdc, pgs, dev, select, pcs);
             } else {
                 gs_gstate temp_state = *((const gs_gstate *)pgs);
 
@@ -702,10 +705,10 @@ gs_color_select_t select)
                 for (k = 0; k < dev->color_info.num_components; k++)
                     temp_state.color_component_map.color_map[k] = k;
                 temp_state.color_component_map.num_components = dev->color_info.num_components;
-                gx_remap_concrete_devicen(conc, pdc, &temp_state, dev, select);
+                gx_remap_concrete_devicen(conc, pdc, &temp_state, dev, select, pcs);
             }
         } else {
-            gx_remap_concrete_devicen(conc, pdc, pgs, dev, select);
+            gx_remap_concrete_devicen(conc, pdc, pgs, dev, select, pcs);
         }
         /* Save original color space and color info into dev color */
         i = any_abs(i);
@@ -791,7 +794,7 @@ gx_remap_DeviceGray(const gs_client_color * pc, const gs_color_space * pcs,
     if (pgs->icc_manager->default_gray != NULL) {
         gs_color_space *pcs_notconst = (gs_color_space*) pcs;
         pcs_notconst->cmm_icc_profile_data = pgs->icc_manager->default_gray;
-        rc_increment(pgs->icc_manager->default_gray);
+        gsicc_adjust_profile_rc(pgs->icc_manager->default_gray, 1, "gx_remap_DeviceGray");
         pcs_notconst->type = &gs_color_space_type_ICC;
         code =
             (*pcs_notconst->type->remap_color)(gs_currentcolor_inline(pgs),
@@ -1134,8 +1137,6 @@ cmap_cmyk_direct(frac c, frac m, frac y, frac k, gx_device_color * pdc,
             gx_color_load_select(pdc, pgs, dev, select);
         return;
     }
-    /* if output device supports devn, we need to make sure we send it the
-       proper color type */
     /* if output device supports devn, we need to make sure we send it the
        proper color type */
     if (dev_proc(dev, dev_spec_op)(dev, gxdso_supports_devn, NULL, 0)) {
@@ -1290,14 +1291,145 @@ map_components_to_colorants(const frac * pcc,
     }
 }
 
+static bool
+named_color_supported(const gs_gstate * pgs)
+{
+    gs_color_space *pcs = gs_currentcolorspace_inline(pgs);
+    gs_color_space_index type = gs_color_space_get_index(pcs);
+
+    if (pgs->icc_manager->device_named == NULL)
+        return false;
+
+    if (type == gs_color_space_index_Separation && pcs->params.separation.named_color_supported)
+        return true;
+
+    if (type == gs_color_space_index_DeviceN && pcs->params.device_n.named_color_supported)
+        return true;
+
+    return false;
+}
+
+/* Routines for handling CM of CMYK components of a DeviceN color space */
+static bool
+devicen_has_cmyk(gx_device * dev, cmm_profile_t *des_profile)
+{
+    gs_devn_params *devn_params;
+
+    devn_params = dev_proc(dev, ret_devn_params)(dev);
+    if (devn_params == NULL) {
+        if (des_profile != NULL && des_profile->data_cs == gsCMYK)
+            return true;
+        else
+            return false;
+    }
+    return(devn_params->num_std_colorant_names == 4);
+}
+
+static void
+devicen_sep_icc_cmyk(frac cm_comps[], const gs_gstate * pgs,
+    const gs_color_space * pcs, gx_device *dev)
+{
+    gsicc_link_t *icc_link;
+    gsicc_rendering_param_t rendering_params;
+    unsigned short psrc[GS_CLIENT_COLOR_MAX_COMPONENTS];
+    unsigned short psrc_cm[GS_CLIENT_COLOR_MAX_COMPONENTS];
+    int k, code;
+    unsigned short *psrc_temp;
+    gsicc_rendering_param_t render_cond;
+    cmm_dev_profile_t *dev_profile = NULL;
+    cmm_profile_t *des_profile = NULL;
+    cmm_profile_t *src_profile = pgs->icc_manager->default_cmyk;
+
+    code = dev_proc(dev, get_profile)(dev, &dev_profile);
+
+    /* If we can't transform them, we will just leave them as is. */
+    if (code < 0)
+        return;
+
+    gsicc_extract_profile(dev->graphics_type_tag,
+        dev_profile, &des_profile, &render_cond);
+    /* Define the rendering intents. */
+    rendering_params.black_point_comp = pgs->blackptcomp;
+    rendering_params.graphics_type_tag = dev->graphics_type_tag;
+    rendering_params.override_icc = false;
+    rendering_params.preserve_black = gsBKPRESNOTSPECIFIED;
+    rendering_params.rendering_intent = pgs->renderingintent;
+    rendering_params.cmm = gsCMM_DEFAULT;
+    /* Sigh, frac to full 16 bit.  Need to clean this up */
+    for (k = 0; k < 4; k++) {
+        psrc[k] = frac2cv(cm_comps[k]);
+    }
+
+    /* Determine what src profile to use.  First choice is the attributes
+       process color space if it is the correct type.  Second choice is
+       the alternate tint transform color space if it is the correct type.
+       Third type is default_cmyk.  If we have an issue with bad profiles then
+       the color values will just remain as they were from the source */
+    if (gs_color_space_get_index(pcs) == gs_color_space_index_DeviceN) {
+        if (pcs->params.device_n.devn_process_space != NULL &&
+            pcs->params.device_n.devn_process_space->cmm_icc_profile_data != NULL &&
+            pcs->params.device_n.devn_process_space->cmm_icc_profile_data->data_cs == gsCMYK) {
+            src_profile = pcs->params.device_n.devn_process_space->cmm_icc_profile_data;
+        } else if (pcs->base_space != NULL &&
+            pcs->base_space->cmm_icc_profile_data != NULL &&
+            pcs->base_space->cmm_icc_profile_data->data_cs == gsCMYK &&
+            USE_ALT_MAP) {
+            src_profile = pcs->base_space->cmm_icc_profile_data;
+        }
+    } else if (gs_color_space_get_index(pcs) == gs_color_space_index_Separation) {
+        if (pcs->base_space != NULL &&
+            pcs->base_space->cmm_icc_profile_data != NULL &&
+            pcs->base_space->cmm_icc_profile_data->data_cs == gsCMYK &&
+            USE_ALT_MAP) {
+            src_profile = pcs->base_space->cmm_icc_profile_data;
+        }
+    }
+
+    icc_link = gsicc_get_link_profile(pgs, dev, src_profile, des_profile,
+        &rendering_params, pgs->memory, dev_profile->devicegraytok);
+
+    if (icc_link == NULL && src_profile != pgs->icc_manager->default_cmyk) {
+        icc_link = gsicc_get_link_profile(pgs, dev,
+            pgs->icc_manager->default_cmyk, des_profile,
+            &rendering_params, pgs->memory, dev_profile->devicegraytok);
+    }
+
+    /* If we can't transform them, we will just leave them as is. */
+    if (icc_link == NULL)
+        return;
+
+    /* Transform the color */
+    if (icc_link->is_identity) {
+        psrc_temp = &(psrc[0]);
+    } else {
+        /* Transform the color */
+        psrc_temp = &(psrc_cm[0]);
+        (icc_link->procs.map_color)(dev, icc_link, psrc, psrc_temp, 2);
+    }
+    /* This needs to be optimized */
+    for (k = 0; k < 4; k++) {
+        cm_comps[k] = float2frac(((float)psrc_temp[k]) / 65535.0);
+    }
+    /* Release the link */
+    gsicc_release_link(icc_link);
+}
+
 static void
 cmap_separation_halftoned(frac all, gx_device_color * pdc,
-     const gs_gstate * pgs, gx_device * dev, gs_color_select_t select)
+     const gs_gstate * pgs, gx_device * dev, gs_color_select_t select,
+     const gs_color_space *pcs)
 {
     uchar i, ncomps = dev->color_info.num_components;
     bool additive = dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE;
     frac comp_value = all;
     frac cm_comps[GX_DEVICE_COLOR_MAX_COMPONENTS];
+    gsicc_rendering_param_t render_cond;
+    cmm_dev_profile_t *dev_profile = NULL;
+    cmm_profile_t *des_profile = NULL;
+
+    dev_proc(dev, get_profile)(dev, &dev_profile);
+    gsicc_extract_profile(dev->graphics_type_tag,
+        dev_profile, &des_profile, &render_cond);
 
     if (pgs->color_component_map.sep_type == SEP_ALL) {
         /*
@@ -1310,12 +1442,17 @@ cmap_separation_halftoned(frac all, gx_device_color * pdc,
         /* Use the "all" value for all components */
         for (i = 0; i < pgs->color_component_map.num_colorants; i++)
             cm_comps[i] = comp_value;
-    }
-    else {
+    } else {
         /* map to the color model */
         map_components_to_colorants(&all, &(pgs->color_component_map), cm_comps);
     }
 
+    if (devicen_has_cmyk(dev, des_profile) &&
+        des_profile->data_cs == gsCMYK &&
+        !named_color_supported(pgs)) {
+        devicen_sep_icc_cmyk(cm_comps, pgs, pcs, dev);
+    }
+
     /* apply the transfer function(s); convert to color values */
     if (pgs->effective_transfer_non_identity_count != 0) {
         if (additive)
@@ -1335,7 +1472,7 @@ cmap_separation_halftoned(frac all, gx_device_color * pdc,
 
 static void
 cmap_separation_direct(frac all, gx_device_color * pdc, const gs_gstate * pgs,
-                 gx_device * dev, gs_color_select_t select)
+                 gx_device * dev, gs_color_select_t select, const gs_color_space *pcs)
 {
     uchar i, ncomps = dev->color_info.num_components;
     bool additive = dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE;
@@ -1377,6 +1514,20 @@ cmap_separation_direct(frac all, gx_device_color * pdc, const gs_gstate * pgs,
         map_components_to_colorants(&comp_value, &(pgs->color_component_map), cm_comps);
     }
 
+    /* Check if we have the standard colorants.  If yes, then we will apply
+      ICC color management to those colorants. */
+    if (devicen_has_cmyk(dev, des_profile) && des_profile->data_cs == gsCMYK &&
+        !named_color_supported(pgs)) {
+        /* We need to do a CMYK to CMYK conversion here.  This will always
+           use the default CMYK profile and the device's output profile.
+           We probably need to add some checking here
+           and possibly permute the colorants, much as is done on the input
+           side for the case when we add DeviceN icc source profiles for use
+           in PDF and PS data. Also, don't do this if we are doing mapping
+           through the named color mapping.  */
+        devicen_sep_icc_cmyk(cm_comps, pgs, pcs, dev);
+    }
+
     /* apply the transfer function(s); convert to color values */
     if (pgs->effective_transfer_non_identity_count == 0)
         for (i = 0; i < ncomps; i++)
@@ -1444,69 +1595,6 @@ cmap_separation_direct(frac all, gx_device_color * pdc, const gs_gstate * pgs,
         gx_color_load_select(pdc, pgs, dev, select);
 }
 
-/* Routines for handling CM of CMYK components of a DeviceN color space */
-static bool
-devicen_has_cmyk(gx_device * dev)
-{
-    gs_devn_params *devn_params;
-
-    devn_params = dev_proc(dev, ret_devn_params)(dev);
-    if (devn_params == NULL) {
-        return false;
-    }
-    return(devn_params->num_std_colorant_names == 4);
-}
-
-static int
-devicen_icc_cmyk(frac cm_comps[], const gs_gstate * pgs, gx_device *dev)
-{
-    gsicc_link_t *icc_link;
-    gsicc_rendering_param_t rendering_params;
-    unsigned short psrc[GS_CLIENT_COLOR_MAX_COMPONENTS];
-    unsigned short psrc_cm[GS_CLIENT_COLOR_MAX_COMPONENTS];
-    int k, code;
-    unsigned short *psrc_temp;
-    gsicc_rendering_param_t render_cond;
-    cmm_dev_profile_t *dev_profile = NULL;
-    cmm_profile_t *des_profile = NULL;
-
-    code = dev_proc(dev, get_profile)(dev,  &dev_profile);
-    if (code < 0)
-        return code;
-
-    gsicc_extract_profile(dev->graphics_type_tag,
-                          dev_profile, &des_profile, &render_cond);
-    /* Define the rendering intents. */
-    rendering_params.black_point_comp = pgs->blackptcomp;
-    rendering_params.graphics_type_tag = dev->graphics_type_tag;
-    rendering_params.override_icc = false;
-    rendering_params.preserve_black = gsBKPRESNOTSPECIFIED;
-    rendering_params.rendering_intent = pgs->renderingintent;
-    rendering_params.cmm = gsCMM_DEFAULT;
-    /* Sigh, frac to full 16 bit.  Need to clean this up */
-    for (k = 0; k < 4; k++){
-        psrc[k] = frac2cv(cm_comps[k]);
-    }
-    icc_link = gsicc_get_link_profile(pgs, dev, pgs->icc_manager->default_cmyk,
-                des_profile, &rendering_params, pgs->memory,
-                dev_profile->devicegraytok);
-    /* Transform the color */
-    if (icc_link->is_identity) {
-        psrc_temp = &(psrc[0]);
-    } else {
-        /* Transform the color */
-        psrc_temp = &(psrc_cm[0]);
-        (icc_link->procs.map_color)(dev, icc_link, psrc, psrc_temp, 2);
-    }
-    /* This needs to be optimized */
-    for (k = 0; k < 4; k++){
-        cm_comps[k] = float2frac(((float) psrc_temp[k])/65535.0);
-    }
-    /* Release the link */
-    gsicc_release_link(icc_link);
-    return(0);
-}
-
 /* ------ DeviceN color mapping */
 
 /*
@@ -1516,7 +1604,7 @@ devicen_icc_cmyk(frac cm_comps[], const gs_gstate * pgs, gx_device *dev)
 static void
 cmap_devicen_halftoned(const frac * pcc,
     gx_device_color * pdc, const gs_gstate * pgs, gx_device * dev,
-    gs_color_select_t select)
+    gs_color_select_t select, const gs_color_space *pcs)
 {
     uchar i, ncomps = dev->color_info.num_components;
     frac cm_comps[GX_DEVICE_COLOR_MAX_COMPONENTS];
@@ -1530,9 +1618,10 @@ cmap_devicen_halftoned(const frac * pcc,
     /* map to the color model */
     map_components_to_colorants(pcc, &(pgs->color_component_map), cm_comps);
     /* See comments in cmap_devicen_direct for details on below operations */
-    if (devicen_has_cmyk(dev) &&
-        des_profile->data_cs == gsCMYK) {
-        devicen_icc_cmyk(cm_comps, pgs, dev);
+    if (devicen_has_cmyk(dev, des_profile) &&
+        des_profile->data_cs == gsCMYK &&
+        !named_color_supported(pgs)) {
+        devicen_sep_icc_cmyk(cm_comps, pgs, pcs, dev);
     }
     /* apply the transfer function(s); convert to color values */
     if (pgs->effective_transfer_non_identity_count != 0) {
@@ -1559,7 +1648,7 @@ cmap_devicen_halftoned(const frac * pcc,
 static void
 cmap_devicen_direct(const frac * pcc,
     gx_device_color * pdc, const gs_gstate * pgs, gx_device * dev,
-    gs_color_select_t select)
+    gs_color_select_t select, const gs_color_space *pcs)
 {
     uchar i, ncomps = dev->color_info.num_components;
     frac cm_comps[GX_DEVICE_COLOR_MAX_COMPONENTS];
@@ -1591,14 +1680,16 @@ cmap_devicen_direct(const frac * pcc,
        as a CMYK color that will be color managed and specified with 10% C,
        20% M 0% Y 0% K. Hence the CMYK values should go through the same
        color management as a stand alone CMYK value.  */
-    if (devicen_has_cmyk(dev) && des_profile->data_cs == gsCMYK) {
+    if (devicen_has_cmyk(dev, des_profile) && des_profile->data_cs == gsCMYK &&
+        !named_color_supported(pgs)) {
         /* We need to do a CMYK to CMYK conversion here.  This will always
            use the default CMYK profile and the device's output profile.
            We probably need to add some checking here
            and possibly permute the colorants, much as is done on the input
            side for the case when we add DeviceN icc source profiles for use
-           in PDF and PS data. */
-        devicen_icc_cmyk(cm_comps, pgs, dev);
+           in PDF and PS data. Also, don't do this if we are doing mapping
+           through the named color mapping.  */
+        devicen_sep_icc_cmyk(cm_comps, pgs, pcs, dev);
     }
     /* apply the transfer function(s); convert to color values.
        assign directly if output device supports devn */
@@ -1997,6 +2088,238 @@ gx_unit_frac(float fvalue)
     return f;
 }
 
+static void
+cmapper_transfer_halftone_add(gx_cmapper_t *data)
+{
+    gx_color_value *pconc = &data->conc[0];
+    const gs_gstate * pgs = data->pgs;
+    gx_device * dev = data->dev;
+    gs_color_select_t select = data->select;
+    uchar ncomps = dev->color_info.num_components;
+    frac frac_value;
+    uchar i;
+    frac cv_frac[GX_DEVICE_COLOR_MAX_COMPONENTS];
+
+    /* apply the transfer function(s) */
+    for (i = 0; i < ncomps; i++) {
+        frac_value = cv2frac(pconc[i]);
+        cv_frac[i] = gx_map_color_frac(pgs, frac_value, effective_transfer[i]);
+    }
+    /* Halftoning */
+    if (gx_render_device_DeviceN(&(cv_frac[0]), &data->devc, dev,
+                    pgs->dev_ht, &pgs->screen_phase[select]) == 1)
+        gx_color_load_select(&data->devc, pgs, dev, select);
+}
+
+static void
+cmapper_transfer_halftone_op(gx_cmapper_t *data)
+{
+    gx_color_value *pconc = &data->conc[0];
+    const gs_gstate * pgs = data->pgs;
+    gx_device * dev = data->dev;
+    gs_color_select_t select = data->select;
+    uchar ncomps = dev->color_info.num_components;
+    frac frac_value;
+    uchar i;
+    frac cv_frac[GX_DEVICE_COLOR_MAX_COMPONENTS];
+
+    /* apply the transfer function(s) */
+    uint k = dev->color_info.black_component;
+    for (i = 0; i < ncomps; i++) {
+        frac_value = cv2frac(pconc[i]);
+        if (i == k) {
+            cv_frac[i] = frac_1 - gx_map_color_frac(pgs,
+                (frac)(frac_1 - frac_value), effective_transfer[i]);
+        } else {
+            cv_frac[i] = frac_value;  /* Ignore transfer, see PLRM3 p. 494 */
+        }
+    }
+    /* Halftoning */
+    if (gx_render_device_DeviceN(&(cv_frac[0]), &data->devc, dev,
+                    pgs->dev_ht, &pgs->screen_phase[select]) == 1)
+        gx_color_load_select(&data->devc, pgs, dev, select);
+}
+
+static void
+cmapper_transfer_halftone_sub(gx_cmapper_t *data)
+{
+    gx_color_value *pconc = &data->conc[0];
+    const gs_gstate * pgs = data->pgs;
+    gx_device * dev = data->dev;
+    gs_color_select_t select = data->select;
+    uchar ncomps = dev->color_info.num_components;
+    frac frac_value;
+    uchar i;
+    frac cv_frac[GX_DEVICE_COLOR_MAX_COMPONENTS];
+
+    /* apply the transfer function(s) */
+    for (i = 0; i < ncomps; i++) {
+        frac_value = cv2frac(pconc[i]);
+        cv_frac[i] = frac_1 - gx_map_color_frac(pgs,
+                    (frac)(frac_1 - frac_value), effective_transfer[i]);
+    }
+    /* Halftoning */
+    if (gx_render_device_DeviceN(&(cv_frac[0]), &data->devc, dev,
+                    pgs->dev_ht, &pgs->screen_phase[select]) == 1)
+        gx_color_load_select(&data->devc, pgs, dev, select);
+}
+
+static void
+cmapper_transfer_add(gx_cmapper_t *data)
+{
+    gx_color_value *pconc = &data->conc[0];
+    const gs_gstate * pgs = data->pgs;
+    gx_device * dev = data->dev;
+    uchar ncomps = dev->color_info.num_components;
+    frac frac_value;
+    uchar i;
+    gx_color_index color;
+
+    /* apply the transfer function(s) */
+    for (i = 0; i < ncomps; i++) {
+        frac_value = cv2frac(pconc[i]);
+        frac_value = gx_map_color_frac(pgs,
+                                frac_value, effective_transfer[i]);
+        pconc[i] = frac2cv(frac_value);
+    }
+    /* Halftoning */
+    color = dev_proc(dev, encode_color)(dev, &(pconc[0]));
+    /* check if the encoding was successful; we presume failure is rare */
+    if (color != gx_no_color_index)
+        color_set_pure(&data->devc, color);
+}
+
+static void
+cmapper_transfer_op(gx_cmapper_t *data)
+{
+    gx_color_value *pconc = &data->conc[0];
+    const gs_gstate * pgs = data->pgs;
+    gx_device * dev = data->dev;
+    frac frac_value;
+    gx_color_index color;
+
+    uint k = dev->color_info.black_component;
+    /* Ignore transfer for non blacks, see PLRM3 p. 494 */
+    frac_value = cv2frac(pconc[k]);
+    frac_value = frac_1 - gx_map_color_frac(pgs,
+                (frac)(frac_1 - frac_value), effective_transfer[k]);
+    pconc[k] = frac2cv(frac_value);
+    /* Halftoning */
+    color = dev_proc(dev, encode_color)(dev, &(pconc[0]));
+    /* check if the encoding was successful; we presume failure is rare */
+    if (color != gx_no_color_index)
+        color_set_pure(&data->devc, color);
+}
+
+static void
+cmapper_transfer_sub(gx_cmapper_t *data)
+{
+    gx_color_value *pconc = &data->conc[0];
+    const gs_gstate * pgs = data->pgs;
+    gx_device * dev = data->dev;
+    uchar ncomps = dev->color_info.num_components;
+    frac frac_value;
+    uchar i;
+    gx_color_index color;
+
+    /* apply the transfer function(s) */
+    for (i = 0; i < ncomps; i++) {
+        frac_value = cv2frac(pconc[i]);
+        frac_value = frac_1 - gx_map_color_frac(pgs,
+                    (frac)(frac_1 - frac_value), effective_transfer[i]);
+        pconc[i] = frac2cv(frac_value);
+    }
+    /* Halftoning */
+    color = dev_proc(dev, encode_color)(dev, &(pconc[0]));
+    /* check if the encoding was successful; we presume failure is rare */
+    if (color != gx_no_color_index)
+        color_set_pure(&data->devc, color);
+}
+
+/* This is used by image color render to handle the cases where we need to
+   perform either a transfer function or halftoning on the color values
+   during an ICC color flow.  In this case, the color is already in the
+   device color space but in 16bpp color values. */
+static void
+cmapper_halftone(gx_cmapper_t *data)
+{
+    gx_color_value *pconc = &data->conc[0];
+    const gs_gstate * pgs = data->pgs;
+    gx_device * dev = data->dev;
+    gs_color_select_t select = data->select;
+    uchar ncomps = dev->color_info.num_components;
+    uchar i;
+    frac cv_frac[GX_DEVICE_COLOR_MAX_COMPONENTS];
+
+    /* We need this to be in frac form */
+    for (i = 0; i < ncomps; i++) {
+        cv_frac[i] = cv2frac(pconc[i]);
+    }
+    if (gx_render_device_DeviceN(&(cv_frac[0]), &data->devc, dev,
+                pgs->dev_ht, &pgs->screen_phase[select]) == 1)
+        gx_color_load_select(&data->devc, pgs, dev, select);
+}
+
+/* This is used by image color render to handle the cases where we need to
+   perform either a transfer function or halftoning on the color values
+   during an ICC color flow.  In this case, the color is already in the
+   device color space but in 16bpp color values. */
+static void
+cmapper_vanilla(gx_cmapper_t *data)
+{
+    gx_color_value *pconc = &data->conc[0];
+    gx_device * dev = data->dev;
+    gx_color_index color;
+
+    color = dev_proc(dev, encode_color)(dev, &(pconc[0]));
+    /* check if the encoding was successful; we presume failure is rare */
+    if (color != gx_no_color_index)
+        color_set_pure(&data->devc, color);
+}
+
+void
+gx_get_cmapper(gx_cmapper_t *data, const gs_gstate *pgs,
+               gx_device *dev, bool has_transfer, bool has_halftone,
+               gs_color_select_t select)
+{
+    memset(&(data->conc[0]), 0, sizeof(gx_color_value[GX_DEVICE_COLOR_MAX_COMPONENTS]));
+    data->pgs = pgs;
+    data->dev = dev;
+    data->select = select;
+    data->devc.type = gx_dc_type_none;
+    data->direct = 0;
+    if (has_transfer && dev->color_info.opmode == GX_CINFO_OPMODE_UNKNOWN)
+        check_cmyk_color_model_comps(dev);
+    if (pgs->effective_transfer_non_identity_count == 0)
+        has_transfer = 0;
+    if (has_transfer) {
+        if (dev->color_info.polarity == GX_CINFO_POLARITY_ADDITIVE) {
+            if (has_halftone)
+                data->set_color = cmapper_transfer_halftone_add;
+            else
+                data->set_color = cmapper_transfer_add;
+        } else if (dev->color_info.opmode == GX_CINFO_OPMODE) {
+            if (has_halftone)
+                data->set_color = cmapper_transfer_halftone_op;
+            else
+                data->set_color = cmapper_transfer_op;
+        } else {
+            if (has_halftone)
+                data->set_color = cmapper_transfer_halftone_sub;
+            else
+                data->set_color = cmapper_transfer_sub;
+        }
+    } else {
+        if (has_halftone)
+            data->set_color = cmapper_halftone;
+        else {
+            int code = dev_proc(dev, dev_spec_op)(dev, gxdso_is_encoding_direct, NULL, 0);
+            data->set_color = cmapper_vanilla;
+            data->direct = (code == 1);
+        }
+    }
+}
+
 /* This is used by image color render to handle the cases where we need to
    perform either a transfer function or halftoning on the color values
    during an ICC color flow.  In this case, the color is already in the
