@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -19,7 +19,7 @@
 #ifndef gxdevcli_INCLUDED
 #  define gxdevcli_INCLUDED
 
-#include "std.h"		/* for FILE */
+#include "gsdevice.h"
 #include "stdint_.h"
 #include "gscompt.h"
 #include "gsdcolor.h"
@@ -40,14 +40,12 @@
 #include "gp.h"
 #include "gscms.h"
 #include "gxrplane.h"
+#include "gxdda.h"
+#include "gxpath.h"
+#include "gsimage.h"
 
 /* See Drivers.htm for documentation of the driver interface. */
 
-#ifndef gx_device_DEFINED
-#  define gx_device_DEFINED
-typedef struct gx_device_s gx_device;
-#endif
-
 /* ---------------- Memory management ---------------- */
 
 /*
@@ -117,53 +115,9 @@ typedef struct gx_device_s gx_device;
 
 /* ---------------- Auxiliary types and structures ---------------- */
 
-/* We need at least an abstract type for a graphics state, */
-/* which is passed to the page device procedures. */
-#ifndef gs_gstate_DEFINED
-#  define gs_gstate_DEFINED
-typedef struct gs_gstate_s gs_gstate;
-#endif
-
-/* We need abstract types for paths and fill/stroke parameters, */
-/* for the path-oriented device procedures. */
-#ifndef gx_path_DEFINED
-#  define gx_path_DEFINED
-typedef struct gx_path_s gx_path;
-#endif
-#ifndef gx_clip_path_DEFINED
-#  define gx_clip_path_DEFINED
-typedef struct gx_clip_path_s gx_clip_path;
-#endif
-#ifndef gx_fill_params_DEFINED
-#  define gx_fill_params_DEFINED
-typedef struct gx_fill_params_s gx_fill_params;
-#endif
-#ifndef gx_stroke_params_DEFINED
-#  define gx_stroke_params_DEFINED
-typedef struct gx_stroke_params_s gx_stroke_params;
-#endif
-#ifndef gs_gstate_DEFINED
-#  define gs_gstate_DEFINED
-typedef struct gs_gstate_s gs_gstate;
-#endif
-#ifndef patch_fill_state_t_DEFINED
-#  define patch_fill_state_t_DEFINED
-typedef struct patch_fill_state_s  patch_fill_state_t;
-#endif
-
-/* We need an abstract type for the image enumeration state, */
-/* for begin[_typed]_image. */
-#ifndef gx_image_enum_common_t_DEFINED
-#  define gx_image_enum_common_t_DEFINED
-typedef struct gx_image_enum_common_s gx_image_enum_common_t;
-#endif
-
 /* We need an abstract type for the pattern instance, */
 /* for pattern_manage. */
-#ifndef gs_pattern1_instance_t_DEFINED
-#  define gs_pattern1_instance_t_DEFINED
 typedef struct gs_pattern1_instance_s gs_pattern1_instance_t;
-#endif
 
 /* Define the type for colors passed to the higher-level procedures. */
 typedef gx_device_color gx_drawing_color;
@@ -182,10 +136,7 @@ typedef struct gs_fixed_edge_s {
 } gs_fixed_edge;
 
 /* Define the parameters passed to get_bits_rectangle. */
-#ifndef gs_get_bits_params_DEFINED
-#  define gs_get_bits_params_DEFINED
 typedef struct gs_get_bits_params_s gs_get_bits_params_t;
-#endif
 
 /* Define the structure for device color capabilities. */
 typedef struct gx_device_anti_alias_info_s {
@@ -891,12 +842,6 @@ typedef enum FILTER_FLAGS {
 
 /* ---------------- Device procedures ---------------- */
 
-/* Define an opaque type for parameter lists. */
-#ifndef gs_param_list_DEFINED
-#  define gs_param_list_DEFINED
-typedef struct gs_param_list_s gs_param_list;
-#endif
-
 /*
  * Definition of device procedures.
  * Note that the gx_device * argument is not declared const,
@@ -1090,6 +1035,19 @@ typedef struct gs_param_list_s gs_param_list;
 #define dev_proc_stroke_path(proc)\
   dev_t_proc_stroke_path(proc, gx_device)
 
+                /* Added in release 9.22 */
+
+#define dev_t_proc_fill_stroke_path(proc, dev_t)\
+  int proc(dev_t *dev,\
+    const gs_gstate *pgs, gx_path *ppath,\
+    const gx_fill_params *fill_params,\
+    const gx_drawing_color *pdcolor_fill,\
+    const gx_stroke_params *stroke_params,\
+    const gx_drawing_color *pdcolor_stroke,\
+    const gx_clip_path *pcpath)
+#define dev_proc_fill_stroke_path(proc)\
+  dev_t_proc_fill_stroke_path(proc, gx_device)
+
                 /* Added in release 3.60 */
 
 #define dev_t_proc_fill_mask(proc, dev_t)\
@@ -1482,10 +1440,7 @@ typedef struct gs_fill_attributes_s {
  * return a pointer to the devn_params section of a device.  Return NULL
  * if this field is not present within the device.
  */
-#ifndef gs_devn_params_DEFINED
-#define gs_devn_params_DEFINED
 typedef struct gs_devn_params_s gs_devn_params;
-#endif
 
 #define dev_t_proc_ret_devn_params(proc, dev_t)\
   gs_devn_params * proc(dev_t *dev)
@@ -1514,7 +1469,7 @@ typedef struct gs_devn_params_s gs_devn_params;
   dev_t_proc_pop_transparency_state(proc, gx_device)
 
 #define dev_t_proc_put_image(proc, dev_t)\
-  int proc(gx_device *dev, const byte **buffers, int num_chan, int x, int y,\
+  int proc(gx_device *dev, gx_device *mdev,  const byte **buffers, int num_chan, int x, int y,\
             int width, int height, int row_stride,\
             int alpha_plane_index, int tag_plane_index)
 #define dev_proc_put_image(proc)\
@@ -1589,6 +1544,40 @@ struct gx_process_page_options_s
 #define dev_proc_process_page(proc)\
   dev_t_proc_process_page(proc, gx_device)
 
+typedef enum  {
+    transform_pixel_region_begin = 0,
+    transform_pixel_region_data_needed = 1,
+    transform_pixel_region_process_data = 2,
+    transform_pixel_region_end = 3
+} transform_pixel_region_reason;
+
+typedef struct {
+    void *state;
+    union {
+        struct {
+            const gs_int_rect *clip;
+            int w; /* source width */
+            int h; /* source height */
+            int spp;
+            const gx_dda_fixed_point *pixels; /* DDA to enumerate the destination positions of pixels across a row */
+            const gx_dda_fixed_point *rows; /* DDA to enumerate the starting position of each row */
+            gs_logical_operation_t lop;
+        } init;
+        struct {
+            const unsigned char *buffer[GX_DEVICE_COLOR_MAX_COMPONENTS];
+            int data_x;
+            gx_cmapper_t *cmapper;
+            const gs_gstate *pgs;
+        } process_data;
+    } u;
+} transform_pixel_region_data;
+
+#define dev_t_proc_transform_pixel_region(proc, dev_t)\
+  int proc(dev_t *dev, transform_pixel_region_reason reason, transform_pixel_region_data *data)
+#define dev_proc_transform_pixel_region(proc)\
+  dev_t_proc_transform_pixel_region(proc, gx_device)
+
+
 /* Define the device procedure vector template proper. */
 
 #define gx_device_proc_struct(dev_t)\
@@ -1665,6 +1654,8 @@ struct gx_process_page_options_s
         dev_t_proc_strip_tile_rect_devn((*strip_tile_rect_devn), dev_t);\
         dev_t_proc_copy_alpha_hl_color((*copy_alpha_hl_color), dev_t);\
         dev_t_proc_process_page((*process_page), dev_t);\
+        dev_t_proc_transform_pixel_region((*transform_pixel_region), dev_t);\
+        dev_t_proc_fill_stroke_path((*fill_stroke_path), dev_t);\
 }
 
 /*
@@ -1853,16 +1844,27 @@ void map_cmyk_subclass(const subclass_color_mappings scm, frac c, frac m, frac y
 }
 
 /* Test to see if the device wants to use tags */
-static inline bool device_encodes_tags(gx_device *dev)
+static inline bool device_encodes_tags(const gx_device *dev)
 {
     return (dev->graphics_type_tag & GS_DEVICE_ENCODES_TAGS) != 0;
 }
 
+static inline bool device_is_deep(const gx_device *dev)
+{
+    bool has_tags = device_encodes_tags(dev);
+    int bits_per_comp = ((dev->color_info.depth - has_tags*8) /
+                         dev->color_info.num_components);
+    if (bits_per_comp > 16)
+        return 1;
+    if (bits_per_comp == 16 && dev->color_info.num_components > 1)
+        return 1;
+    if (bits_per_comp == 8)
+        return 0;
+    return (dev->color_info.max_color > 255 ||
+            dev->color_info.max_gray > 255);
+}
+
 /* A null device.  This is used to temporarily disable output. */
-#ifndef gx_device_null_DEFINED
-#  define gx_device_null_DEFINED
-typedef struct gx_device_null_s gx_device_null;
-#endif
 struct gx_device_null_s {
     gx_device_forward_common;
 };
