@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -19,6 +19,8 @@
 #ifndef gxdevsop_INCLUDED
 #  define gxdevsop_INCLUDED
 
+#include "gxdevcli.h"
+
 /* This file enumerates a series of device specific operations, that can be
  * performed using the 'dev_spec_op' procedure in the gs_device structure.
  *
@@ -312,7 +314,7 @@ enum {
      * device.   In this case, we may want to not use the alternate tint
      * tranform even if the blending color space is RGB or Gray. */
     gxdso_pdf14_sep_device,
-    /* Used only by pdfwrite to paa a Form Appearance Name, so that
+    /* Used only by pdfwrite to pass a Form Appearance Name, so that
      * we can use the name in a pdfmark.
      */
     gxdso_pdf_form_name,
@@ -327,6 +329,49 @@ enum {
     gxdso_JPEG_passthrough_data,
     gxdso_JPEG_passthrough_end,
     gxdso_supports_iccpostrender,
+    /* Retrieve the last device in a device chain
+       (either forwarding or subclass devices).
+     */
+    gxdso_current_output_device,
+    /* Should we call copy_color rather than resolving images to fill_rectangles? */
+    gxdso_copy_color_is_fast,
+    /* gxdso_is_encoding_direct:
+     *     data = NULL
+     *     size = 0
+     * Returns 1 if the device maps color byte values directly into the device,
+     * 0 otherwise.
+     */
+    gxdso_is_encoding_direct,
+    /* gxdso_event_info:
+     *     data = dev_param_req_t
+     *     size = sizeof(dev_param-req_t
+     * Passes a single name in request->Param, naming the event which occurred.
+     * Used to send a warning to pdfwrite that some event has happened we want to know about.
+     * Currently this is used in pdf_font.ps to signal that a substittue font has been
+     * used. If we are emitting PDF/A then we need to abort it, as the Widths array of
+     * the PDF font may not match the widths of the glyphs in the font.
+     */
+    gxdso_event_info,
+    /* gxdso_overprint_active:
+    *     data = NULL
+    *     size = 0
+    * Returns 1 if the overprint device is active,
+    * 0 otherwise.
+    */
+    gxdso_overprint_active,
+    /* gxdso_in_smask:
+    *     data = NULL
+    *     size = 0
+    * Returns 1 if we are within an smask (either construction or usage),
+    * 0 otherwise.
+    */
+    gxdso_in_smask,
+
+    /* Debug only dsos follow here */
+#ifdef DEBUG
+    /* Private dso used to check that a printer device properly forwards to the default */
+    gxdso_debug_printer_check,
+#endif
     /* Add new gxdso_ keys above this. */
     gxdso_pattern__LAST
 };
