@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -297,6 +297,7 @@ gx_ht_read_component(
     if (data >= data_lim)
         return_error(gs_error_rangecheck);
     new_order.procs = &ht_order_procs_table[*data++];
+    new_order.threshold_inverted = 0;
 
     /* calculate the space required for levels and bit data */
     levels_size = new_order.num_levels * sizeof(new_order.levels[0]);
@@ -584,8 +585,10 @@ gx_ht_read_and_install(
         for (i = 0; i < num_dev_comps; i++)
             components_save[i] = components[i];
         code = gx_gstate_dev_ht_install(pgs, &dht, dht.type, dev);
-        for (i = 0; i < num_dev_comps; i++)
-            gx_ht_order_release(&components_save[i].corder, mem, false);
+        if (code >= 0) {
+            for (i = 0; i < num_dev_comps; i++)
+                gx_ht_order_release(&components_save[i].corder, mem, false);
+        }
     }
 
     /*
