@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -199,7 +199,7 @@ is_spotan_device(gx_device * dev)
      * is allocaded on the stack i.e. has no block header with a descriptor
      * but has dev->memory set like a heap-allocated device.
      */
-    return dev->procs.open_device == san_open;
+    return dev_proc(dev, open_device) == san_open;
 }
 
 /* Forward declarations */
@@ -299,6 +299,7 @@ gx_general_fill_path(gx_device * pdev, const gs_gstate * pgs,
     fill_options fo;
     line_list lst;
     int clipping = 0;
+    int scanconverter;
 
     *(const fill_options **)&lst.fo = &fo; /* break 'const'. */
     /*
@@ -380,8 +381,6 @@ gx_general_fill_path(gx_device * pdev, const gs_gstate * pgs,
             fo.adjust_above = fixed_half /* + fixed_epsilon */ ;        /* see above */
     else
         fo.adjust_below = fo.adjust_above = adjust.y;
-    /* Initialize the active line list. */
-    init_line_list(&lst, ppath->memory);
     sbox.p.x = ibox.p.x - adjust.x;
     sbox.p.y = ibox.p.y - adjust.y;
     sbox.q.x = ibox.q.x + adjust.x;
@@ -434,85 +433,45 @@ gx_general_fill_path(gx_device * pdev, const gs_gstate * pgs,
             fill_by_trapezoids = false; /* avoid double writing pixels */
     }
 
-    if (!fo.is_spotan && gs_getscanconverter(pdev->memory) >= GS_SCANCONVERTER_EDGEBUFFER) {
-        gx_edgebuffer eb = { 0 };
+    if (!fo.is_spotan && ((scanconverter = gs_getscanconverter(pdev->memory)) >= GS_SCANCONVERTER_EDGEBUFFER ||
+                          (scanconverter == GS_SCANCONVERTER_DEFAULT && GS_SCANCONVERTER_DEFAULT_IS_EDGEBUFFER))) {
+        gx_scan_converter_t *sc;
+        /* If we have a request for accurate curves, make sure we exactly
+         * match what we'd get for stroking. */
+        if (!big_path && pgs->accurate_curves && gx_path_has_curves(ppath))
+        {
+            gx_path_init_local(&ffpath, ppath->memory);
+            code = gx_path_copy_reducing(ppath, &ffpath, fo.fixed_flat, NULL,
+                                         pco_small_curves | pco_accurate);
+            if (code < 0)
+                return code;
+            ppath = &ffpath;
+        }
+
         if (fill_by_trapezoids && !lop_is_idempotent(lop))
             fill_by_trapezoids = 0;
         if (!fill_by_trapezoids)
         {
-            if (adjust.x == 0 && adjust.y == 0) {
-                code = gx_scan_convert(dev,
-                                       ppath,
-                                       &ibox,
-                                       &eb,
-                                       fo.fixed_flat);
-                if (code >= 0) {
-                    code = gx_filter_edgebuffer(dev,
-                                                &eb,
-                                                params->rule);
-                }
-                if (code >= 0) {
-                    code = gx_fill_edgebuffer(dev,
-                                              pdevc,
-                                              &eb,
-                                              fo.fill_direct ? -1 : (int)pgs->log_op);
-                }
-            } else {
-                code = gx_scan_convert_app(dev,
-                                           ppath,
-                                           &ibox,
-                                           &eb,
-                                           fo.fixed_flat);
-                if (code >= 0) {
-                    code = gx_filter_edgebuffer_app(dev,
-                                                    &eb,
-                                                    params->rule);
-                }
-                if (code >= 0) {
-                    code = gx_fill_edgebuffer_app(dev,
-                                                  pdevc,
-                                                  &eb,
-                                                  fo.fill_direct ? -1 : (int)pgs->log_op);
-                }
-            }
+            if (adjust.x == 0 && adjust.y == 0)
+                sc = &gx_scan_converter;
+            else
+                sc = &gx_scan_converter_app;
         } else {
-            if (adjust.x == 0 && adjust.y == 0) {
-                code = gx_scan_convert_tr(dev,
-                                          ppath,
-                                          &ibox,
-                                          &eb,
-                                          fo.fixed_flat);
-                if (code >= 0) {
-                    code = gx_filter_edgebuffer_tr(dev,
-                                                   &eb,
-                                                    params->rule);
-                }
-                if (code >= 0) {
-                    code = gx_fill_edgebuffer_tr(dev,
-                                                 pdevc,
-                                                 &eb,
-                                                 (int)pgs->log_op);
-                }
-            } else {
-                code = gx_scan_convert_tr_app(dev,
-                                              ppath,
-                                              &ibox,
-                                              &eb,
-                                              fo.fixed_flat);
-                if (code >= 0) {
-                    code = gx_filter_edgebuffer_tr_app(dev,
-                                                       &eb,
-                                                       params->rule);
-                }
-                if (code >= 0) {
-                    code = gx_fill_edgebuffer_tr_app(dev,
-                                                     pdevc,
-                                                     &eb,
-                                                     (int)pgs->log_op);
-                }
-            }
+            if (adjust.x == 0 && adjust.y == 0)
+                sc = &gx_scan_converter_tr;
+            else
+                sc = &gx_scan_converter_tr_app;
         }
-        gx_edgebuffer_fin(dev,&eb);
+        code = gx_scan_convert_and_fill(sc,
+                                        dev,
+                                        ppath,
+                                        &ibox,
+                                        fo.fixed_flat,
+                                        params->rule,
+                                        pdevc,
+                                       (!fill_by_trapezoids && fo.fill_direct) ? -1 : (int)pgs->log_op);
+        if (ppath == &ffpath)
+            gx_path_free(ppath, "gx_general_fill_path");
         return code;
     }
 
@@ -536,6 +495,8 @@ gx_general_fill_path(gx_device * pdev, const gs_gstate * pgs,
         }
     }
     fo.fill_by_trapezoids = fill_by_trapezoids;
+    /* Initialize the active line list. */
+    init_line_list(&lst, ppath->memory);
     if ((code = add_y_list(pfpath, &lst)) < 0)
         goto nope;
     {
@@ -702,6 +663,30 @@ gx_default_fill_path(gx_device * pdev, const gs_gstate * pgs,
     return code;
 }
 
+/*
+ * Fill/Stroke a path.  This is the default implementation of the driver
+ * fill_path procedure.
+ */
+int
+gx_default_fill_stroke_path(gx_device * pdev, const gs_gstate * pgs,
+                            gx_path * ppath,
+                            const gx_fill_params * params_fill,
+                            const gx_device_color * pdevc_fill,
+                            const gx_stroke_params * params_stroke,
+                            const gx_device_color * pdevc_stroke,
+                            const gx_clip_path * pcpath)
+{
+    int code = dev_proc(pdev, fill_path)(pdev, pgs, ppath, params_fill, pdevc_fill, pcpath);
+
+    if (code < 0)
+        return code;
+    /* Swap colors to make sure the pgs colorspace is correct for stroke */
+    gs_swapcolors_quick(pgs);
+    code = dev_proc(pdev, stroke_path)(pdev, pgs, ppath, params_stroke, pdevc_stroke, pcpath);
+    gs_swapcolors_quick(pgs);
+    return code;
+}
+
 /* Free the line list. */
 static void
 free_line_list(line_list *ll)
@@ -1318,8 +1303,7 @@ insert_x_new(active_line * alp, line_list *ll)
     alp->prev = prev;
     if (next != 0)
         next->prev = alp;
-    if (prev)
-        prev->next = alp;
+    prev->next = alp;
 }
 
 /* Insert a newly active line in the h list. */
@@ -1877,7 +1861,6 @@ intersect_al(line_list *ll, fixed y, fixed *y_top, int draw, bool all_bands)
                             nx = nx0;
                     }
                     endp->x_next = alp->x_next = nx;  /* Ensure same X. */
-                    draw = 0;
                     /* Can't guarantee same x for triple intersections here.
                        Will take care below */
                 }
