@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -20,6 +20,8 @@
 #ifndef gxhttile_INCLUDED
 #  define gxhttile_INCLUDED
 
+#include "gxbitmap.h"
+
 /*
  * A halftone tile is just an ordinary bitmap tile, plus a couple of other
  * items associated with managing its existence in a tile cache.
@@ -27,12 +29,8 @@
  * clients of gx_device_color can access it.
  */
 
-#ifndef gx_ht_tile_DEFINED
-#  define gx_ht_tile_DEFINED
 typedef struct gx_ht_tile_s gx_ht_tile;
 
-#endif
-
 struct gx_ht_tile_s {
     gx_strip_bitmap tiles;	/* the currently rendered tile */
     int level;			/* the cached gray level, i.e. */
