@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -58,21 +58,126 @@ typedef union {
 iclass_proc(gs_image_class_4_color);
 
 static irender_proc(image_render_color_DeviceN);
-static irender_proc(image_render_color_icc_portrait);
-static irender_proc(image_render_color_icc_landscape);
-static irender_proc(image_render_color_icc_skew);
+static irender_proc(image_render_color_icc_tpr);
+#ifndef WITH_CAL
 static irender_proc(image_render_color_thresh);
+#else
 
-irender_proc_t
-gs_image_class_4_color(gx_image_enum * penum)
+#include "cal.h"
+
+static irender_proc(image_render_color_ht_cal);
+
+static int
+image_render_color_ht_cal_skip_line(gx_image_enum *penum,
+                                    gx_device     *dev)
 {
-    bool std_cmap_procs;
+    return !cal_halftone_next_line_required(penum->cal_ht);
+}
+
+static void
+color_halftone_callback(cal_halftone_data_t *ht, void *arg)
+{
+    gx_device *dev = arg;
+    gx_color_index dev_white = gx_device_white(dev);
+    gx_color_index dev_black = gx_device_black(dev);
+
+    if (dev->is_planar) {
+        (*dev_proc(dev, copy_planes)) (dev, ht->data, ht->x + (ht->offset_x<<3), ht->raster,
+            gx_no_bitmap_id, ht->x, ht->y, ht->w, ht->h,
+            ht->plane_raster);
+    } else {
+        (*dev_proc(dev, copy_mono)) (dev, ht->data, ht->x + (ht->offset_x<<3), ht->raster,
+            gx_no_bitmap_id, ht->x, ht->y, ht->w, ht->h, dev_white,
+            dev_black);
+    }
+}
+
+static cal_halftone*
+color_halftone_init(gx_image_enum *penum)
+{
+    cal_halftone *cal_ht = NULL;
+    gx_dda_fixed dda_ht;
+    cal_context *ctx = penum->memory->gs_lib_ctx->core->cal_ctx;
+    int k;
+    gx_ht_order *d_order;
     int code;
+    byte *cache = (penum->color_cache != NULL ? penum->color_cache->device_contone : NULL);
+    cal_matrix matrix;
+    int clip_x, clip_y;
+
+    if (!gx_device_must_halftone(penum->dev))
+        return NULL;
+
+    if (penum->pgs == NULL || penum->pgs->dev_ht == NULL)
+        return NULL;
+    dda_ht = penum->dda.pixel0.x;
+    if (penum->dxx > 0)
+        dda_translate(dda_ht, -fixed_epsilon);
+    matrix.xx = penum->matrix.xx;
+    matrix.xy = penum->matrix.xy;
+    matrix.yx = penum->matrix.yx;
+    matrix.yy = penum->matrix.yy;
+    matrix.tx = penum->matrix.tx + matrix.xx * penum->rect.x + matrix.yx * penum->rect.y;
+    matrix.ty = penum->matrix.ty + matrix.xy * penum->rect.x + matrix.yy * penum->rect.y;
+    clip_x = fixed2int(penum->clip_outer.p.x);
+    clip_y = fixed2int(penum->clip_outer.p.y);
+    cal_ht = cal_halftone_init(ctx,
+                               penum->memory->non_gc_memory,
+                               penum->rect.w,
+                               penum->rect.h,
+                               &matrix,
+                               penum->dev->color_info.num_components,
+                               cache,
+                               clip_x,
+                               clip_y,
+                               fixed2int_ceiling(penum->clip_outer.q.x) - clip_x,
+                               fixed2int_ceiling(penum->clip_outer.q.y) - clip_y,
+                               penum->adjust);
+    if (cal_ht == NULL)
+        goto fail;
+
+    for (k = 0; k < penum->pgs->dev_ht->num_comp; k++) {
+        d_order = &(penum->pgs->dev_ht->components[k].corder);
+        code = gx_ht_construct_threshold(d_order, penum->dev, penum->pgs, k);
+        if (code < 0)
+            goto fail;
+        if (cal_halftone_add_screen(ctx,
+                                    penum->memory->non_gc_memory,
+                                    cal_ht,
+                                    penum->pgs->dev_ht->components[k].corder.threshold_inverted,
+                                    penum->pgs->dev_ht->components[k].corder.width,
+                                    penum->pgs->dev_ht->components[k].corder.full_height,
+                                    -penum->pgs->screen_phase[k].x,
+                                    -penum->pgs->screen_phase[k].y,
+                                    penum->pgs->dev_ht->components[k].corder.threshold) < 0)
+            goto fail;
+    }
+
+    return cal_ht;
+
+fail:
+    cal_halftone_fin(cal_ht, penum->memory->non_gc_memory);
+    return NULL;
+}
+#endif
+
+static int image_skip_color_icc_tpr(gx_image_enum *penum, gx_device *dev);
+
+int
+gs_image_class_4_color(gx_image_enum * penum, irender_proc_t *render_fn)
+{
+    int code = 0;
 #if USE_FAST_HT_CODE
     bool use_fast_thresh = true;
 #else
     bool use_fast_thresh = false;
 #endif
+    const gs_color_space *pcs;
+    gsicc_rendering_param_t rendering_params;
+    int k;
+    int src_num_comp = cs_num_components(penum->pcs);
+    int des_num_comp, bpc;
+    cmm_dev_profile_t *dev_profile;
 
     if (penum->use_mask_color) {
         /*
@@ -109,100 +214,130 @@ gs_image_class_4_color(gx_image_enum * penum)
        then we will need to use those and go through pixel by pixel instead
        of blasting through buffers.  This is true for example with many of
        the color spaces for CUPs */
-    std_cmap_procs = gx_device_uses_std_cmap_procs(penum->dev, penum->pgs);
     if ( (gs_color_space_get_index(penum->pcs) == gs_color_space_index_DeviceN &&
-        penum->pcs->cmm_icc_profile_data == NULL) || penum->use_mask_color ||
-        !std_cmap_procs) {
-        return &image_render_color_DeviceN;
+        penum->pcs->cmm_icc_profile_data == NULL) || penum->use_mask_color) {
+         *render_fn = &image_render_color_DeviceN;
+         return 0;
+    }
+
+    /* Set up the link now */
+    code = dev_proc(penum->dev, get_profile)(penum->dev, &dev_profile);
+    if (code < 0)
+        return code;
+
+    des_num_comp = gsicc_get_device_profile_comps(dev_profile);
+    bpc = penum->dev->color_info.depth / des_num_comp;	/* bits per component */
+    penum->icc_setup.need_decode = false;
+    /* Check if we need to do any decoding.  If yes, then that will slow us down */
+    for (k = 0; k < src_num_comp; k++) {
+        if ( penum->map[k].decoding != sd_none ) {
+            penum->icc_setup.need_decode = true;
+            break;
+        }
+    }
+    /* Define the rendering intents */
+    rendering_params.black_point_comp = penum->pgs->blackptcomp;
+    rendering_params.graphics_type_tag = GS_IMAGE_TAG;
+    rendering_params.override_icc = false;
+    rendering_params.preserve_black = gsBKPRESNOTSPECIFIED;
+    rendering_params.rendering_intent = penum->pgs->renderingintent;
+    rendering_params.cmm = gsCMM_DEFAULT;
+    if (gs_color_space_is_PSCIE(penum->pcs) && penum->pcs->icc_equivalent != NULL) {
+        pcs = penum->pcs->icc_equivalent;
     } else {
-        /* Set up the link now */
-        const gs_color_space *pcs;
-        gsicc_rendering_param_t rendering_params;
-        int k;
-        int src_num_comp = cs_num_components(penum->pcs);
-        int des_num_comp, bpc;
-        cmm_dev_profile_t *dev_profile;
-
-        code = dev_proc(penum->dev, get_profile)(penum->dev, &dev_profile);
-        if (code < 0)
-            return NULL;    /* This function does not return errors, best we can do is say 'we can't handle this' */
-
-        des_num_comp = gsicc_get_device_profile_comps(dev_profile);
-        bpc = penum->dev->color_info.depth / des_num_comp;	/* bits per component */
+        pcs = penum->pcs;
+    }
+    penum->icc_setup.is_lab = pcs->cmm_icc_profile_data->islab;
+    penum->icc_setup.must_halftone = gx_device_must_halftone(penum->dev);
+    penum->icc_setup.has_transfer = gx_has_transfer(penum->pgs, des_num_comp);
+    if (penum->icc_setup.is_lab)
         penum->icc_setup.need_decode = false;
-        /* Check if we need to do any decoding.  If yes, then that will slow us down */
-        for (k = 0; k < src_num_comp; k++) {
-            if ( penum->map[k].decoding != sd_none ) {
-                penum->icc_setup.need_decode = true;
+    if (penum->icc_link == NULL) {
+        penum->icc_link = gsicc_get_link(penum->pgs, penum->dev, pcs, NULL,
+            &rendering_params, penum->memory);
+    }
+    /* PS CIE color spaces may have addition decoding that needs to
+       be performed to ensure that the range of 0 to 1 is provided
+       to the CMM since ICC profiles are restricted to that range
+       but the PS color spaces are not. */
+    penum->use_cie_range = false;
+    if (gs_color_space_is_PSCIE(penum->pcs) &&
+        penum->pcs->icc_equivalent != NULL) {
+        /* We have a PS CIE space.  Check the range */
+        if ( !check_cie_range(penum->pcs) ) {
+            /* It is not 0 to 1.  We will be doing decode
+               plus an additional linear adjustment */
+            penum->use_cie_range = (get_cie_range(penum->pcs) != NULL);
+        }
+    }
+    if (!gx_device_uses_std_cmap_procs(penum->dev, penum->pgs)) {
+        *render_fn = &image_render_color_DeviceN;
+        return code;
+    }
+    if (gx_device_must_halftone(penum->dev) && use_fast_thresh &&
+        (penum->posture == image_portrait || penum->posture == image_landscape)
+        && penum->image_parent_type == gs_image_type1) {
+        bool transfer_is_monotonic = true;
+
+        for (k=0; k<des_num_comp; k++) {
+            if (!gx_transfer_is_monotonic(penum->pgs, k)) {
+                transfer_is_monotonic = false;
                 break;
             }
         }
-        /* Define the rendering intents */
-        rendering_params.black_point_comp = penum->pgs->blackptcomp;
-        rendering_params.graphics_type_tag = GS_IMAGE_TAG;
-        rendering_params.override_icc = false;
-        rendering_params.preserve_black = gsBKPRESNOTSPECIFIED;
-        rendering_params.rendering_intent = penum->pgs->renderingintent;
-        rendering_params.cmm = gsCMM_DEFAULT;
-        if (gs_color_space_is_PSCIE(penum->pcs) && penum->pcs->icc_equivalent != NULL) {
-            pcs = penum->pcs->icc_equivalent;
-        } else {
-            pcs = penum->pcs;
-        }
-        penum->icc_setup.is_lab = pcs->cmm_icc_profile_data->islab;
-        penum->icc_setup.must_halftone = gx_device_must_halftone(penum->dev);
-        penum->icc_setup.has_transfer = gx_has_transfer(penum->pgs, des_num_comp);
-        if (penum->icc_setup.is_lab)
-            penum->icc_setup.need_decode = false;
-        if (penum->icc_link == NULL) {
-            penum->icc_link = gsicc_get_link(penum->pgs, penum->dev, pcs, NULL,
-                &rendering_params, penum->memory);
-        }
-        /* PS CIE color spaces may have addition decoding that needs to
-           be performed to ensure that the range of 0 to 1 is provided
-           to the CMM since ICC profiles are restricted to that range
-           but the PS color spaces are not. */
-        penum->use_cie_range = false;
-        if (gs_color_space_is_PSCIE(penum->pcs) &&
-            penum->pcs->icc_equivalent != NULL) {
-            /* We have a PS CIE space.  Check the range */
-            if ( !check_cie_range(penum->pcs) ) {
-                /* It is not 0 to 1.  We will be doing decode
-                   plus an additional linear adjustment */
-                penum->use_cie_range = (get_cie_range(penum->pcs) != NULL);
+        /* If num components is 1 or if we are going to CMYK planar device
+           then we will may use the thresholding if it is a halftone
+           device IFF we have one bit per component */
+        if ((bpc == 1) && transfer_is_monotonic &&
+            (penum->dev->color_info.num_components == 1 || penum->dev->is_planar) &&
+            penum->bps == 8) {
+#ifdef WITH_CAL
+            penum->cal_ht = color_halftone_init(penum);
+            if (penum->cal_ht != NULL)
+            {
+                penum->skip_next_line = image_render_color_ht_cal_skip_line;
+                *render_fn = &image_render_color_ht_cal;
+                return code;
             }
-        }
-        if (gx_device_must_halftone(penum->dev) && use_fast_thresh &&
-            (penum->posture == image_portrait || penum->posture == image_landscape)
-            && penum->image_parent_type == gs_image_type1) {
-            bool transfer_is_monotonic = true;
-
-            for (k=0; k<des_num_comp; k++) {
-                if (!gx_transfer_is_monotonic(penum->pgs, k)) {
-                    transfer_is_monotonic = false;
-                    break;
-                }
-            }
-            /* If num components is 1 or if we are going to CMYK planar device
-               then we will may use the thresholding if it is a halftone
-               device IFF we have one bit per component */
-            if ((bpc == 1) && transfer_is_monotonic &&
-                (penum->dev->color_info.num_components == 1 || penum->dev->is_planar) &&
-                penum->bps == 8) {
-                code = gxht_thresh_image_init(penum);
-                if (code == 0) {
-                     /* NB: transfer function is pickled into the threshold arrray */
-                     penum->icc_setup.has_transfer = false;
-                     return &image_render_color_thresh;
-                }
+#else
+            code = gxht_thresh_image_init(penum);
+            if (code == 0) {
+                 /* NB: transfer function is pickled into the threshold arrray */
+                 penum->icc_setup.has_transfer = false;
+                 *render_fn = &image_render_color_thresh;
+                 return code;
             }
+#endif
         }
-        if (penum->posture == image_portrait)
-            return &image_render_color_icc_portrait;
-        if (penum->posture == image_landscape)
-            return &image_render_color_icc_landscape;
-        return &image_render_color_icc_skew;
     }
+    {
+        gs_int_rect rect;
+        transform_pixel_region_data data;
+        data.u.init.clip = &rect;
+        data.u.init.w = penum->rect.w;
+        data.u.init.h = penum->rect.h;
+        data.u.init.pixels = &penum->dda.pixel0;
+        data.u.init.rows = &penum->dda.row;
+        data.u.init.lop = penum->log_op;
+        rect.p.x = fixed2int(penum->clip_outer.p.x);
+        rect.p.y = fixed2int(penum->clip_outer.p.y);
+        rect.q.x = fixed2int_ceiling(penum->clip_outer.q.x);
+        rect.q.y = fixed2int_ceiling(penum->clip_outer.q.y);
+
+        if (penum->icc_link == NULL || (penum->icc_link->is_identity && !penum->icc_setup.need_decode))
+            data.u.init.spp = penum->spp;
+        else
+            data.u.init.spp = des_num_comp;
+
+        code = dev_proc(penum->dev, transform_pixel_region)(penum->dev, transform_pixel_region_begin, &data);
+        if (code >= 0) {
+            penum->tpr_state = data.state;
+            penum->skip_next_line = image_skip_color_icc_tpr;
+            *render_fn = &image_render_color_icc_tpr;
+            return code;
+        }
+    }
+    return code;
 }
 
 /* ------ Rendering procedures ------ */
@@ -310,14 +445,14 @@ decode_row(const gx_image_enum *penum, const byte *psrc, int spp, byte *pdes,
 static int
 image_color_icc_prep(gx_image_enum *penum_orig, const byte *psrc, uint w,
                      gx_device *dev, int *spp_cm_out, byte **psrc_cm,
-                     byte **psrc_cm_start, byte **bufend, bool planar_out)
+                     byte **psrc_cm_start, byte **bufend, int *pspan, bool planar_out)
 {
     const gx_image_enum *const penum = penum_orig; /* const within proc */
     const gs_gstate *pgs = penum->pgs;
     bool need_decode = penum->icc_setup.need_decode;
     gsicc_bufferdesc_t input_buff_desc;
     gsicc_bufferdesc_t output_buff_desc;
-    int num_pixels, spp_cm;
+    int spp_cm;
     int spp = penum->spp;
     bool force_planar = false;
     int num_des_comps;
@@ -327,7 +462,6 @@ image_color_icc_prep(gx_image_enum *penum_orig, const byte *psrc, uint w,
     const byte *planar_src;
     byte *planar_des;
     int j, k;
-    int width;
 
     code = dev_proc(dev, get_profile)(dev, &dev_profile);
     if (code < 0) return code;
@@ -335,6 +469,8 @@ image_color_icc_prep(gx_image_enum *penum_orig, const byte *psrc, uint w,
     if (penum->icc_link == NULL) {
         return gs_rethrow(-1, "ICC Link not created during image render color");
     }
+    if (pspan)
+        *pspan = w;
     /* If the link is the identity, then we don't need to do any color
        conversions except for potentially a decode.  Planar out is a special
        case. For now we let the CMM do the reorg into planar.  We will want
@@ -350,11 +486,18 @@ image_color_icc_prep(gx_image_enum *penum_orig, const byte *psrc, uint w,
         *bufend = *psrc_cm + w;
         *psrc_cm_start = NULL;
     } else {
+        int width = w/spp;
+        int span = (width+31)&~31;
         spp_cm = num_des_comps;
-        *psrc_cm = gs_alloc_bytes(pgs->memory,  w * spp_cm/spp,
-                                  "image_color_icc_prep");
-        *psrc_cm_start = *psrc_cm;
-        *bufend = *psrc_cm +  w * spp_cm/spp;
+
+        if (pspan)
+            *pspan = span;
+        /* Put the buffer on a 32 byte memory alignment for SSE/AVX for every
+         * line. Also extra space for 32 byte overrun. */
+        *psrc_cm_start = gs_alloc_bytes(pgs->memory,  span * spp_cm + 64,
+                                        "image_color_icc_prep");
+        *psrc_cm = *psrc_cm_start + ((32 - (intptr_t)(*psrc_cm_start)) & 31);
+        *bufend = *psrc_cm +  span * spp_cm;
         if (penum->icc_link->is_identity) {
             if (!force_planar) {
                 /* decode only. no CM.  This is slow but does not happen that often */
@@ -380,11 +523,10 @@ image_color_icc_prep(gx_image_enum *penum_orig, const byte *psrc, uint w,
                     planar_src = psrc;
                 }
                 /* Now to planar */
-                width = w/spp;
                 planar_des = *psrc_cm;
                 for (k = 0; k < width; k++) {
                     for (j = 0; j < spp; j++) {
-                        *(planar_des + j * width) = *planar_src++;
+                        *(planar_des + j * span) = *planar_src++;
                     }
                     planar_des++;
                 }
@@ -396,18 +538,17 @@ image_color_icc_prep(gx_image_enum *penum_orig, const byte *psrc, uint w,
             }
         } else {
             /* Set up the buffer descriptors. planar out always ends up here */
-            num_pixels = w/spp;
             gsicc_init_buffer(&input_buff_desc, spp, 1,
                           false, false, false, 0, w,
-                          1, num_pixels);
+                          1, width);
             if (!force_planar) {
                 gsicc_init_buffer(&output_buff_desc, spp_cm, 1,
-                              false, false, false, 0, num_pixels * spp_cm,
-                              1, num_pixels);
+                              false, false, false, 0, width * spp_cm,
+                              1, width);
             } else {
                 gsicc_init_buffer(&output_buff_desc, spp_cm, 1,
-                              false, false, true, w/spp, w/spp,
-                              1, num_pixels);
+                              false, false, true, span, span,
+                              1, width);
             }
             /* For now, just blast it all through the link. If we had a significant reduction
                we will want to repack the data first and then do this.  That will be
@@ -445,9 +586,46 @@ image_color_icc_prep(gx_image_enum *penum_orig, const byte *psrc, uint w,
     return 0;
 }
 
+#ifdef WITH_CAL
+static int
+image_render_color_ht_cal(gx_image_enum *penum, const byte *buffer, int data_x,
+                          uint w, int h, gx_device * dev)
+{
+    const byte *psrc = buffer + data_x;
+    int code = 0;
+    int spp_cm = 0;
+    byte *psrc_cm = NULL, *psrc_cm_start = NULL;
+    byte *bufend = NULL;
+    byte *input[GX_DEVICE_COLOR_MAX_COMPONENTS];    /* to ensure 128 bit boundary */
+    int i;
+    int planestride;
+
+    if (h == 0)
+        return 0;
+
+    /* Get the buffer into the device color space */
+    code = image_color_icc_prep(penum, psrc, w, dev, &spp_cm, &psrc_cm,
+                                &psrc_cm_start,  &bufend, &planestride, true);
+    if (code < 0)
+        return code;
+
+    for (i = 0; i < spp_cm; i++)
+        input[i] = psrc_cm + w*i;
+
+    code = cal_halftone_process_planar(penum->cal_ht, penum->memory->non_gc_memory,
+                                       (const byte * const *)input, color_halftone_callback, dev);
+
+    /* Free cm buffer, if it was used */
+    if (psrc_cm_start != NULL) {
+        gs_free_object(penum->pgs->memory, (byte *)psrc_cm_start,
+                       "image_render_color_thresh");
+    }
+    return code;
+}
+#else
 static int
 image_render_color_thresh(gx_image_enum *penum_orig, const byte *buffer, int data_x,
-                   uint w, int h, gx_device * dev)
+                          uint w, int h, gx_device * dev)
 {
     gx_image_enum *penum = penum_orig; /* const within proc */
     image_posture posture = penum->posture;
@@ -480,7 +658,7 @@ image_render_color_thresh(gx_image_enum *penum_orig, const byte *buffer, int dat
     if (h != 0 && penum->line_size != 0) {      /* line_size == 0, nothing to do */
         /* Get the buffer into the device color space */
         code = image_color_icc_prep(penum, psrc, w, dev, &spp_cm, &psrc_cm,
-                                    &psrc_cm_start,  &bufend, true);
+                                    &psrc_cm_start,  &bufend, &psrc_planestride, true);
         if (code < 0)
             return code;
     } else {
@@ -620,16 +798,16 @@ image_render_color_thresh(gx_image_enum *penum_orig, const byte *buffer, int dat
                                     *(devc_contone_gray+1) = *psrc_temp;
                             }
                         } else {
-                        /* Mono case, forward */
-                        psrc_temp = psrc_cm;
-                        for (k=0; k<src_size; k++) {
-                            dda_next(dda_ht);
-                            xn = fixed2int_var_rounded(dda_current(dda_ht));
-                            while (xr < xn) {
-                                *devc_contone_gray++ = *psrc_temp;
-                                xr++;
-                            }           /* at loop exit xn will be >= xr */
-                            psrc_temp++;
+                            /* Mono case, forward */
+                            psrc_temp = psrc_cm;
+                            for (k=0; k<src_size; k++) {
+                                dda_next(dda_ht);
+                                xn = fixed2int_var_rounded(dda_current(dda_ht));
+                                while (xr < xn) {
+                                    *devc_contone_gray++ = *psrc_temp;
+                                    xr++;
+                                }           /* at loop exit xn will be >= xr */
+                                psrc_temp++;
                             }
                         }
                     } else {
@@ -644,7 +822,7 @@ image_render_color_thresh(gx_image_enum *penum_orig, const byte *buffer, int dat
                                 xr--;
                             }           /* at loop exit xn will be >= xr */
                             psrc_temp++;
-                            }
+                        }
                     }
                     break;
                 /* Monochrome landscape */
@@ -787,10 +965,9 @@ image_render_color_thresh(gx_image_enum *penum_orig, const byte *buffer, int dat
                             dda_next(dda_ht);
                             xn = fixed2int_var_rounded(dda_current(dda_ht));
                             while (xr > xn) {
-                                for (j = 0; j < spp_out; j++) {
+                                for (j = 0; j < spp_out; j++)
                                     *(devc_contone[j] + position) = (psrc_plane[j])[i];
-                                    position -= LAND_BITS;
-                                }
+                                position -= LAND_BITS;
                                 xr--;
                             }           /* at loop exit xn will be <= xr */
                             i++;
@@ -801,9 +978,8 @@ image_render_color_thresh(gx_image_enum *penum_orig, const byte *buffer, int dat
                            and 2x scaling which we will run into in 300 and
                            600dpi devices and content */
                         /* Apply initial offset */
-                        for (k = 0; k < spp_out; k++) {
+                        for (k = 0; k < spp_out; k++)
                             devc_contone[k] = devc_contone[k] + position;
-                        }
                         if (src_size == dest_height) {
                             for (k = 0; k < data_length; k++) {
                                 /* Is it better to unwind this?  We know it is 4 */
@@ -829,10 +1005,9 @@ image_render_color_thresh(gx_image_enum *penum_orig, const byte *buffer, int dat
                                 dda_next(dda_ht);
                                 xn = fixed2int_var_rounded(dda_current(dda_ht));
                                 while (xr > xn) {
-                                    for (j = 0; j < spp_out; j++) {
+                                    for (j = 0; j < spp_out; j++)
                                         *(devc_contone[j] + position) = (psrc_plane[j])[i];
-                                        position -= LAND_BITS;
-                                    }
+                                    position -= LAND_BITS;
                                     xr--;
                                 }           /* at loop exit xn will be <= xr */
                                 i++;
@@ -860,7 +1035,7 @@ flush:
     thresh_align = penum->thresh_buffer + offset_threshold;
     code = gxht_thresh_planes(penum, xrun, dest_width, dest_height,
                               thresh_align, dev, offset_contone,
-                               contone_stride);
+                              contone_stride);
     /* Free cm buffer, if it was used */
     if (psrc_cm_start != NULL) {
         gs_free_object(penum->pgs->memory, (byte *)psrc_cm_start,
@@ -868,259 +1043,55 @@ flush:
     }
     return code;
 }
+#endif
 
 /* Render a color image with 8 or fewer bits per sample using ICC profile. */
 static int
-image_render_color_icc_portrait(gx_image_enum *penum_orig, const byte *buffer, int data_x,
-                                uint w, int h, gx_device * dev)
+image_skip_color_icc_tpr(gx_image_enum *penum, gx_device *dev)
 {
-    const gx_image_enum *const penum = penum_orig; /* const within proc */
-    const gs_gstate *pgs = penum->pgs;
-    gs_logical_operation_t lop = penum->log_op;
-    gx_dda_fixed_point pnext;
-    int vci, vdi;
-    int spp = penum->spp;
-    const byte *psrc = buffer + data_x * spp;
-    int irun;			/* int x/rrun */
-    byte *bufend = NULL;
-    int code = 0;
-    byte *psrc_cm = NULL, *psrc_cm_start = NULL;
-    byte *psrc_cm_initial;
-    byte *run;
-    int k;
-    int spp_cm = 0;
-    bool must_halftone = penum->icc_setup.must_halftone;
-    bool has_transfer = penum->icc_setup.has_transfer;
-    gx_cmapper_data data;
-    gx_cmapper_fn *mapper = gx_get_cmapper(&data, pgs, dev, has_transfer, must_halftone, gs_color_select_source);
-    gx_color_value *conc = &data.conc[0];
-
-    if (h == 0)
-        return 0;
-    code = image_color_icc_prep(penum_orig, psrc, w, dev, &spp_cm, &psrc_cm,
-                                &psrc_cm_start, &bufend, false);
-    if (code < 0) return code;
-    psrc_cm_initial = psrc_cm;
-    /* Needed for device N */
-    pnext = penum->dda.pixel0;
-    dda_translate(pnext.x,  (-fixed_epsilon));
-    irun = fixed2int_var_rounded(dda_current(pnext.x));
-    vci = penum->yci, vdi = penum->hci;
-    if_debug5m('b', penum->memory, "[b]y=%d data_x=%d w=%d xt=%f yt=%f\n",
-               penum->y, data_x, w, fixed2float(dda_current(pnext.x)), fixed2float(dda_current(pnext.y)));
-    while (psrc_cm < bufend) {
-        /* Find the length of the next run. It will either end when we hit
-         * the end of the source data, or when the pixel data differs. */
-        run = psrc_cm + spp_cm;
-        while (1)
-        {
-            dda_next(pnext.x);
-            if (run >= bufend)
-                break;
-            if (memcmp(run, psrc_cm, spp_cm))
-                break;
-            run += spp_cm;
-        }
-        /* So we have a run of pixels from psrc_cm to run that are all the same. */
-        /* This needs to be sped up */
-        for (k = 0; k < spp_cm; k++) {
-            conc[k] = gx_color_value_from_byte(psrc_cm[k]);
-        }
-        mapper(&data);
-        /* Fill the region between irun and fixed2int_var_rounded(pnext.x) */
-        {
-            int xi = irun;
-            int wi = (irun = fixed2int_var_rounded(dda_current(pnext.x))) - xi;
-
-            if (wi < 0)
-                xi += wi, wi = -wi;
-            if (wi > 0)
-                code = gx_fill_rectangle_device_rop(xi, vci, wi, vdi,
-                                                    &data.devc, dev, lop);
-        }
-        if (code < 0)
-            goto err;
-        psrc_cm = run;
-    }
-    /* Free cm buffer, if it was used */
-    if (psrc_cm_start != NULL) {
-        gs_free_object(pgs->memory, (byte *)psrc_cm_start, "image_render_color_icc");
-    }
-    return (code < 0 ? code : 1);
-    /* Save position if error, in case we resume. */
-err:
-    gs_free_object(pgs->memory, (byte *)psrc_cm_start, "image_render_color_icc");
-    penum_orig->used.x = (run - psrc_cm_initial) / spp_cm;
-    penum_orig->used.y = 0;
-    return code;
+    transform_pixel_region_data data;
+    data.state = penum->tpr_state;
+    return dev_proc(dev, transform_pixel_region)(dev, transform_pixel_region_data_needed, &data) == 0;
 }
 
 static int
-image_render_color_icc_landscape(gx_image_enum *penum_orig, const byte *buffer, int data_x,
-                                 uint w, int h, gx_device * dev)
+image_render_color_icc_tpr(gx_image_enum *penum_orig, const byte *buffer, int data_x,
+                          uint w, int h, gx_device * dev)
 {
     const gx_image_enum *const penum = penum_orig; /* const within proc */
     const gs_gstate *pgs = penum->pgs;
-    gs_logical_operation_t lop = penum->log_op;
-    gx_dda_fixed_point pnext;
-    int vci, vdi;
     int spp = penum->spp;
     const byte *psrc = buffer + data_x * spp;
-    int irun;			/* int x/rrun */
-    byte *bufend = NULL;
-    int code = 0;
+    int code;
     byte *psrc_cm = NULL, *psrc_cm_start = NULL;
     byte *psrc_cm_initial;
-    byte *run;
-    int k;
+    byte *bufend = NULL;
     int spp_cm = 0;
     bool must_halftone = penum->icc_setup.must_halftone;
     bool has_transfer = penum->icc_setup.has_transfer;
-    gx_cmapper_data data;
-    gx_cmapper_fn *mapper = gx_get_cmapper(&data, pgs, dev, has_transfer, must_halftone, gs_color_select_source);
-    gx_color_value *conc = &data.conc[0];
+    gx_cmapper_t cmapper;
+    transform_pixel_region_data data;
 
     if (h == 0)
         return 0;
     code = image_color_icc_prep(penum_orig, psrc, w, dev, &spp_cm, &psrc_cm,
-                                &psrc_cm_start, &bufend, false);
+                                &psrc_cm_start, &bufend, NULL, false);
     if (code < 0) return code;
     psrc_cm_initial = psrc_cm;
-    /* Needed for device N */
-    pnext = penum->dda.pixel0;
-    dda_translate(pnext.x,  (-fixed_epsilon));
-    irun = fixed2int_var_rounded(dda_current(pnext.y));
-    vci = penum->xci, vdi = penum->wci;
-    if_debug5m('b', penum->memory, "[b]y=%d data_x=%d w=%d xt=%f yt=%f\n",
-               penum->y, data_x, w, fixed2float(dda_current(pnext.x)), fixed2float(dda_current(pnext.y)));
-    while (psrc_cm < bufend) {
-        /* Find the length of the next run. It will either end when we hit
-         * the end of the source data, or when the pixel data differs. */
-        run = psrc_cm + spp_cm;
-        while (1)
-        {
-            dda_next(pnext.y);
-            if (run >= bufend)
-                break;
-            if (memcmp(run, psrc_cm, spp_cm))
-                break;
-            run += spp_cm;
-        }
-        /* So we have a run of pixels from psrc_cm to run that are all the same. */
-        /* This needs to be sped up */
-        for (k = 0; k < spp_cm; k++) {
-            conc[k] = gx_color_value_from_byte(psrc_cm[k]);
-        }
-        mapper(&data);
-        /* Fill the region between irun and fixed2int_var_rounded(pnext.y) */
-        {		/* 90 degree rotated rectangle */
-            int yi = irun;
-            int hi = (irun = fixed2int_var_rounded(dda_current(pnext.y))) - yi;
+    gx_get_cmapper(&cmapper, pgs, dev, has_transfer, must_halftone, gs_color_select_source);
 
-            if (hi < 0)
-                yi += hi, hi = -hi;
-            if (hi > 0)
-                code = gx_fill_rectangle_device_rop(vci, yi, vdi, hi,
-                                                    &data.devc, dev, lop);
-        }
-        if (code < 0)
-            goto err;
-        psrc_cm = run;
-    }
-    /* Free cm buffer, if it was used */
-    if (psrc_cm_start != NULL) {
-        gs_free_object(pgs->memory, (byte *)psrc_cm_start, "image_render_color_icc");
-    }
-    return (code < 0 ? code : 1);
-    /* Save position if error, in case we resume. */
-err:
+    data.state = penum->tpr_state;
+    data.u.process_data.buffer[0] = psrc_cm;
+    data.u.process_data.data_x = 0;
+    data.u.process_data.cmapper = &cmapper;
+    code = dev_proc(dev, transform_pixel_region)(dev, transform_pixel_region_process_data, &data);
     gs_free_object(pgs->memory, (byte *)psrc_cm_start, "image_render_color_icc");
-    penum_orig->used.x = (run - psrc_cm_initial) / spp_cm;
-    penum_orig->used.y = 0;
-    return code;
-}
 
-static int
-image_render_color_icc_skew(gx_image_enum *penum_orig, const byte *buffer, int data_x,
-                   uint w, int h, gx_device * dev)
-{
-    const gx_image_enum *const penum = penum_orig; /* const within proc */
-    const gs_gstate *pgs = penum->pgs;
-    gs_logical_operation_t lop = penum->log_op;
-    gx_dda_fixed_point pnext;
-    fixed xprev, yprev;
-    fixed pdyx, pdyy;		/* edge of parallelogram */
-    int spp = penum->spp;
-    const byte *psrc = buffer + data_x * spp;
-    fixed xpos;			/* x ditto */
-    fixed ypos;			/* y ditto */
-    byte *bufend = NULL;
-    int code = 0;
-    byte *psrc_cm = NULL, *psrc_cm_start = NULL;
-    byte *psrc_cm_initial;
-    int k;
-    int spp_cm = 0;
-    bool must_halftone = penum->icc_setup.must_halftone;
-    bool has_transfer = penum->icc_setup.has_transfer;
-    byte initial_run[GX_DEVICE_COLOR_MAX_COMPONENTS] = { 0 };
-    byte *prev_cm = &initial_run[0];
-    gx_cmapper_data data;
-    gx_cmapper_fn *mapper = gx_get_cmapper(&data, pgs, dev, has_transfer, must_halftone, gs_color_select_source);
-    gx_color_value *conc = &data.conc[0];
-
-    if (h == 0)
-        return 0;
-    code = image_color_icc_prep(penum_orig, psrc, w, dev, &spp_cm, &psrc_cm,
-                                &psrc_cm_start, &bufend, false);
-    if (code < 0) return code;
-    psrc_cm_initial = psrc_cm;
-    /* Needed for device N */
-    pnext = penum->dda.pixel0;
-    dda_translate(pnext.x,  (-fixed_epsilon));
-    xprev = dda_current(pnext.x);
-    yprev = dda_current(pnext.y);
-    pdyx = dda_current(penum->dda.row.x) - penum->cur.x;
-    pdyy = dda_current(penum->dda.row.y) - penum->cur.y;
-    if_debug5m('b', penum->memory, "[b]y=%d data_x=%d w=%d xt=%f yt=%f\n",
-               penum->y, data_x, w, fixed2float(xprev), fixed2float(yprev));
-    prev_cm[0] = ~psrc_cm[0];	/* Force intial setting */
-    while (psrc_cm < bufend) {
-        dda_next(pnext.x);
-        dda_next(pnext.y);
-        xpos = dda_current(pnext.x);
-        ypos = dda_current(pnext.y);
-
-        if (memcmp(prev_cm, psrc_cm, spp_cm) != 0)
-        {
-            /* This needs to be sped up */
-            for (k = 0; k < spp_cm; k++) {
-                conc[k] = gx_color_value_from_byte(psrc_cm[k]);
-            }
-            mapper(&data);
-        }
-        /* Fill the region between */
-        /* xprev/yprev and xpos/ypos */
-        /* Parallelogram */
-        code = (*dev_proc(dev, fill_parallelogram))
-                    (dev, xprev, yprev, xpos - xprev, ypos - yprev, pdyx, pdyy,
-                     &data.devc, lop);
-        xprev = xpos;
-        yprev = ypos;
-        if (code < 0)
-            goto err;
-        prev_cm = psrc_cm;
-        psrc_cm += spp_cm;
+    if (code < 0) {
+        /* Save position if error, in case we resume. */
+        penum_orig->used.x = (data.u.process_data.buffer[0] - psrc_cm_initial) / spp_cm;
+        penum_orig->used.y = 0;
     }
-    /* Free cm buffer, if it was used */
-    if (psrc_cm_start != NULL) {
-        gs_free_object(pgs->memory, (byte *)psrc_cm_start, "image_render_color_icc");
-    }
-    return (code < 0 ? code : 1);
-    /* Save position if error, in case we resume. */
-err:
-    gs_free_object(pgs->memory, (byte *)psrc_cm_start, "image_render_color_icc");
-    penum_orig->used.x = (psrc_cm - psrc_cm_initial) / spp_cm;
-    penum_orig->used.y = 0;
     return code;
 }
 
@@ -1241,8 +1212,12 @@ image_render_color_DeviceN(gx_image_enum *penum_orig, const byte *buffer, int da
             dmputs(dev->memory, "\n");
         }
 #endif
-        mcode = remap_color(&cc, pcs, pdevc_next, pgs, dev,
-                           gs_color_select_source);
+        if (lab_case || penum->icc_link == NULL || pcs->cmm_icc_profile_data == NULL)
+            mcode = remap_color(&cc, pcs, pdevc_next, pgs, dev, gs_color_select_source);
+        else
+            mcode = gx_remap_ICC_with_link(&cc, pcs, pdevc_next, pgs, dev,
+                                           gs_color_select_source, penum->icc_link);
+
 mapped:	if (mcode < 0)
             goto fill;
         if (sizeof(pdevc_next->colors.binary.color[0]) <= sizeof(ulong))
