@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -29,6 +29,7 @@
 #include "gximag3x.h"
 #include "gxgstate.h"
 #include "gdevbbox.h"
+#include <limits.h> /* For INT_MAX etc */
 
 extern_st(st_color_space);
 
@@ -224,6 +225,17 @@ gx_begin_image3x_generic(gx_device * dev,
             (code = gs_bbox_transform(&mrect, &mat, &mrect)) < 0
             )
             return code;
+
+        /* Bug 700438: If the rectangle is out of range, bail */
+        if (mrect.p.x >= (double)INT_MAX || mrect.q.x <= (double)INT_MIN ||
+            mrect.p.y >= (double)INT_MAX || mrect.q.y <= (double)INT_MIN) {
+                code = gs_note_error(gs_error_rangecheck);
+            goto out1;
+        }
+
+        /* This code was changed for bug 686843/687411, but in a way that
+         * a) looked wrong, and b) doesn't appear to make a difference. Revert
+         * it to the sane version until we have evidence why not. */
         origin[i].x = (int)floor(mrect.p.x);
         origin[i].y = (int)floor(mrect.p.y);
         code = make_mid(&mdev, dev,
@@ -611,6 +623,7 @@ gx_image3x_plane_data(gx_image_enum_common_t * info,
         int mh = mask_height[i] = penum->mask[i].height;
 
         mask_plane[i].data = 0;
+        mask_plane[i].raster = 0;
         mask_used[i] = 0;
         if (!penum->mask[i].depth)
             continue;
