@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -24,17 +24,12 @@
 #include "gslparam.h"
 #include "gspenum.h"
 #include "gsrect.h"
+#include "gsgstate.h" /* For stroke flattening */
 
 /* The routines and types in this interface use */
 /* device, rather than user, coordinates, and fixed-point, */
 /* rather than floating, representation. */
 
-/* Opaque type for a path */
-#ifndef gx_path_DEFINED
-#  define gx_path_DEFINED
-typedef struct gx_path_s gx_path;
-#endif
-
 /* Define the two insideness rules */
 #define gx_rule_winding_number (-1)
 #define gx_rule_even_odd 1
@@ -260,10 +255,6 @@ gx_path_is_rectangular(const gx_path *, gs_fixed_rect *);
 /* Path transformers */
 
 /* The gs_gstate is only needed when flattening for stroke. */
-#ifndef gs_gstate_DEFINED
-#  define gs_gstate_DEFINED
-typedef struct gs_gstate_s gs_gstate;
-#endif
 int gx_path_copy_reducing(const gx_path * ppath_old, gx_path * ppath_new,
                           fixed fixed_flatness, const gs_gstate *pgs,
                           gx_path_copy_options options);
@@ -287,6 +278,8 @@ int gx_path_add_dash_expansion(const gx_path * /*old*/, gx_path * /*new*/,
 void gx_point_scale_exp2(gs_fixed_point *, int, int),
       gx_rect_scale_exp2(gs_fixed_rect *, int, int);
 
+int gx_path_elide_1d(gx_path *ppath);
+
 /* Path enumerator */
 
 /* This interface does not make a copy of the path. */
@@ -309,30 +302,26 @@ int gx_path_merge_contacting_contours(gx_path *ppath);
 /* ------ Clipping paths ------ */
 
 /* Opaque type for a clipping path */
-#ifndef gx_clip_path_DEFINED
-#  define gx_clip_path_DEFINED
 typedef struct gx_clip_path_s gx_clip_path;
-#endif
 
 /* Graphics state clipping */
 int gx_clip_to_rectangle(gs_gstate *, gs_fixed_rect *);
 int gx_clip_to_path(gs_gstate *);
 int gx_default_clip_box(const gs_gstate *, gs_fixed_rect *);
 int gx_effective_clip_path(gs_gstate *, gx_clip_path **);
+int gx_curr_fixed_bbox(gs_gstate * pgs, gs_fixed_rect *bbox, gs_bbox_comp_t comp_type);
 int gx_curr_bbox(gs_gstate * pgs, gs_rect *bbox, gs_bbox_comp_t comp_type);
  
 
 /* Opaque type for a clip list. */
-#ifndef gx_clip_list_DEFINED
-#  define gx_clip_list_DEFINED
 typedef struct gx_clip_list_s gx_clip_list;
-#endif
 
-/* Opaque type for fill parameters. */
-#ifndef gx_fill_params_DEFINED
-#  define gx_fill_params_DEFINED
+
+/* We need abstract types for paths and fill/stroke parameters, */
+/* for the path-oriented device procedures. */
 typedef struct gx_fill_params_s gx_fill_params;
-#endif
+typedef struct gx_stroke_params_s gx_stroke_params;
+typedef struct patch_fill_state_s  patch_fill_state_t;
 
 /* Opaque type for a clipping path enumerator. */
 typedef struct gs_cpath_enum_s gs_cpath_enum;
@@ -388,6 +377,7 @@ int
                                bool segments_shared),
     gx_cpath_to_path(gx_clip_path *, gx_path *),
     gx_cpath_to_path_synthesize(const gx_clip_path * pcpath, gx_path * ppath);
+int gx_cpath_ensure_path_list(gx_clip_path *pcpath);
 bool
     gx_cpath_inner_box(const gx_clip_path *, gs_fixed_rect *),
     gx_cpath_outer_box(const gx_clip_path *, gs_fixed_rect *),
