@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -177,7 +177,11 @@ static const gx_device_pattern_accum gs_pattern_accum_device =
      NULL,                              /* get_profile */
      NULL,                              /* set_graphics_type_tag */
      gx_default_strip_copy_rop2,
-     gx_default_strip_tile_rect_devn
+     gx_default_strip_tile_rect_devn,
+     NULL,                              /* alpha_hl_color */
+     NULL,                              /* process_page */
+     gx_default_transform_pixel_region, /* NOT the default forwarding one */
+     gx_default_fill_stroke_path,
 },
  0,                             /* target */
  0, 0, 0, 0                     /* bitmap_memory, bits, mask, instance */
@@ -307,7 +311,7 @@ gx_pattern_accum_alloc(gs_memory_t * mem, gs_memory_t * storage_memory,
         gx_device_clist_writer *cwdev;
         const int data_size = 1024*128;
         gx_band_params_t band_params = { 0 };
-        byte *data  = gs_alloc_bytes(storage_memory->non_gc_memory, data_size, cname);
+        byte *data  = gs_alloc_bytes(tdev->memory->non_gc_memory, data_size, cname);
 
         if (data == NULL)
             return 0;
@@ -321,7 +325,7 @@ gx_pattern_accum_alloc(gs_memory_t * mem, gs_memory_t * storage_memory,
                                        &buf_procs, &band_params, true, /* use_memory_clist */
                                        pinst->templat.uses_transparency, pinst);
         if (cdev == 0) {
-            gs_free_object(storage_memory->non_gc_memory, data, cname);
+            gs_free_object(tdev->memory->non_gc_memory, data, cname);
             return 0;
         }
         cwdev = (gx_device_clist_writer *)cdev;
@@ -338,6 +342,23 @@ gx_pattern_accum_alloc(gs_memory_t * mem, gs_memory_t * storage_memory,
     return fdev;
 }
 
+gx_pattern_trans_t*
+new_pattern_trans_buff(gs_memory_t *mem)
+{
+    gx_pattern_trans_t *result;
+
+    /* Allocate structure that we will use for the trans pattern */
+    result = gs_alloc_struct(mem, gx_pattern_trans_t, &st_pattern_trans, "new_pattern_trans_buff");
+    result->transbytes = NULL;
+    result->pdev14 = NULL;
+    result->mem = NULL;
+    result->fill_trans_buffer = NULL;
+    result->buf = NULL;
+    result->n_chan = 0;
+
+    return(result);
+}
+
 /*
  * Initialize a pattern accumulator.
  * Client must already have set instance and bitmap_memory.
@@ -389,13 +410,9 @@ pattern_accum_open(gx_device * dev)
        do this since the transparency code all */
     if (pinst->templat.uses_transparency) {
         /* Allocate structure that we will use for the trans pattern */
-        padev->transbuff = gs_alloc_struct(mem,gx_pattern_trans_t,&st_pattern_trans,"pattern_accum_open(trans)");
-        padev->transbuff->transbytes = NULL;
-        padev->transbuff->mem = NULL;
-        padev->transbuff->pdev14 = NULL;
-        padev->transbuff->fill_trans_buffer = NULL;
-        /* n_chan = 0 => padev->transbuff isn't inited. */
-        padev->transbuff->n_chan = 0;
+        padev->transbuff = new_pattern_trans_buff(mem);
+        if (padev->transbuff == NULL)
+            return_error(gs_error_VMerror);
     } else {
         padev->transbuff = NULL;
     }
@@ -492,21 +509,6 @@ pattern_accum_open(gx_device * dev)
     return code;
 }
 
-gx_pattern_trans_t*
-new_pattern_trans_buff(gs_memory_t *mem)
-{
-    gx_pattern_trans_t *result;
-
-    /* Allocate structure that we will use for the trans pattern */
-    result = gs_alloc_struct(mem, gx_pattern_trans_t, &st_pattern_trans, "new_pattern_trans_buff");
-    result->transbytes = NULL;
-    result->pdev14 = NULL;
-    result->mem = NULL;
-    result->fill_trans_buffer = NULL;
-
-    return(result);
-}
-
 /* Close an accumulator and free the bits. */
 static int
 pattern_accum_close(gx_device * dev)
@@ -882,6 +884,8 @@ ensure_pattern_cache(gs_gstate * pgs)
 void
 gx_pattern_cache_free(gx_pattern_cache *pcache)
 {
+    if (pcache == NULL)
+        return;
     pattern_cache_free_all(pcache);
     gs_free_object(pcache->memory, pcache->tiles, "gx_pattern_cache_free");
     pcache->tiles = NULL;
@@ -901,12 +905,14 @@ gstate_set_pattern_cache(gs_gstate * pgs, gx_pattern_cache * pcache)
 }
 
 /* Free a Pattern cache entry. */
+/* This will not free a pattern if it is 'locked' which should only be for */
+/* a stroke pattern during fill_stroke_path.                               */
 static void
 gx_pattern_cache_free_entry(gx_pattern_cache * pcache, gx_color_tile * ctile)
 {
     gx_device *temp_device;
 
-    if ((ctile->id != gx_no_bitmap_id) && !ctile->is_dummy) {
+    if ((ctile->id != gx_no_bitmap_id) && !ctile->is_dummy && !ctile->is_locked) {
         gs_memory_t *mem = pcache->memory;
 
         /*
@@ -985,18 +991,25 @@ gx_pattern_cache_ensure_space(gs_gstate * pgs, int needed)
 {
     int code = ensure_pattern_cache(pgs);
     gx_pattern_cache *pcache;
+    int start_free_id;
 
     if (code < 0)
         return;                 /* no cache -- just exit */
 
     pcache = pgs->pattern_cache;
-
+    start_free_id = pcache->next;	/* for scan wrap check */
     /* If too large then start freeing entries */
-    /* By starting at 'next', we attempt to first free the oldest entries */
+    /* By starting just after 'next', we attempt to first free the oldest entries */
     while (pcache->bits_used + needed > pcache->max_bits &&
            pcache->bits_used != 0) {
         pcache->next = (pcache->next + 1) % pcache->num_tiles;
         gx_pattern_cache_free_entry(pcache, &pcache->tiles[pcache->next]);
+        /* since a pattern may be temporarily locked (stroke pattern for fill_stroke_path) */
+        /* we may not have freed all entries even though we've scanned the entire cache.   */
+        /* The following check for wrapping prevents infinite loop if stroke pattern was   */
+        /* larger than pcache->max_bits,                                                   */
+        if (pcache->next == start_free_id)
+            break;		/* we wrapped -- cache may not be empty */
     }
 }
 
@@ -1016,10 +1029,10 @@ gx_pattern_cache_update_used(gs_gstate *pgs, ulong used)
  * device, but it may zero out the bitmap_memory pointers to prevent
  * the accumulated bitmaps from being freed when the device is closed.
  */
-static void make_bitmap(gx_strip_bitmap *, const gx_device_memory *, gx_bitmap_id);
+static void make_bitmap(gx_strip_bitmap *, const gx_device_memory *, gx_bitmap_id, const gs_memory_t *);
 int
 gx_pattern_cache_add_entry(gs_gstate * pgs,
-                   gx_device_forward * fdev, gx_color_tile ** pctile)
+                   gx_device_forward * fdev, gx_color_tile ** pctile, gs_gstate* saved)
 {
     gx_pattern_cache *pcache;
     const gs_pattern1_instance_t *pinst;
@@ -1050,16 +1063,35 @@ gx_pattern_cache_add_entry(gs_gstate * pgs,
          * If so, we can avoid the expensive masking operations
          * when using the pattern.
          */
-        if (mmask != 0) {
+        /* Bug 700624: In cases where the mask is completely full,
+         * but the pattern cells are separated from one another,
+         * we need to leave gaps between the cells when rendering
+         * them. Sadly, the graphics library can't cope with this
+         * in the no-mask case. Therefore, only do the optimisation
+         * of not sending the mask if the step matrix is suitable.
+         *
+         * To do this, we compare the step matrix to the size. My
+         * belief is that the mask will only ever be full if it's
+         * orthogonal, cos otherwise the edges will be clipped,
+         * hence we lose no generality by checking for .xy and .yx
+         * being 0.
+         */
+        if (mmask != 0 &&
+            fabsf(pinst->step_matrix.xx) <= pinst->size.x &&
+            fabsf(pinst->step_matrix.yy) <= pinst->size.y &&
+            pinst->step_matrix.xy == 0 &&
+            pinst->step_matrix.yx == 0) {
             int y;
+            int w_less_8 = mmask->width-8;
 
             for (y = 0; y < mmask->height; y++) {
                 const byte *row = scan_line_base(mmask, y);
                 int w;
 
-                for (w = mmask->width; w > 8; w -= 8)
+                for (w = w_less_8; w > 0; w -= 8)
                     if (*row++ != 0xff)
                         goto keep;
+                w += 8;
                 if ((*row | (0xff >> w)) != 0xff)
                     goto keep;
             }
@@ -1108,18 +1140,22 @@ gx_pattern_cache_add_entry(gs_gstate * pgs,
     ctile->is_simple = pinst->is_simple;
     ctile->has_overlap = pinst->has_overlap;
     ctile->is_dummy = false;
-    if (pinst->templat.uses_transparency)
-        ctile->blending_mode = ((pdf14_device *)(pgs->device))->blend_mode;
+    ctile->is_locked = false;
+    if (pinst->templat.uses_transparency) {
+        /* to work with pdfi get the blend mode out of the saved pgs device */
+        ctile->blending_mode = ((pdf14_device*)(saved->device))->blend_mode;
+    }
     else
         ctile->blending_mode = 0;
+    ctile->trans_group_popped = false;
     if (dev_proc(fdev, open_device) != pattern_clist_open_device) {
         if (mbits != 0) {
-            make_bitmap(&ctile->tbits, mbits, gs_next_ids(pgs->memory, 1));
+            make_bitmap(&ctile->tbits, mbits, gs_next_ids(pgs->memory, 1), pgs->memory);
             mbits->bitmap_memory = 0;   /* don't free the bits */
         } else
             ctile->tbits.data = 0;
         if (mmask != 0) {
-            make_bitmap(&ctile->tmask, mmask, id);
+            make_bitmap(&ctile->tmask, mmask, id, pgs->memory);
             mmask->bitmap_memory = 0;   /* don't free the bits */
         } else
             ctile->tmask.data = 0;
@@ -1156,6 +1192,26 @@ gx_pattern_cache_add_entry(gs_gstate * pgs,
     return 0;
 }
 
+/* set or clear the 'is_locked' flag for a tile in the cache. Used by	*/
+/* fill_stroke_path to make sure a large stroke pattern stays in the	*/
+/* cache even if the fill is also a pattern.				*/
+int
+gx_pattern_cache_entry_set_lock(gs_gstate *pgs, gs_id id, bool new_lock_value)
+{
+    gx_pattern_cache *pcache;
+    gx_color_tile *ctile;
+    int code = ensure_pattern_cache(pgs);
+
+    if (code < 0)
+        return code;
+    pcache = pgs->pattern_cache;
+    ctile = &pcache->tiles[id % pcache->num_tiles];
+    if (ctile->id != id)
+        return_error(gs_error_undefined);
+    ctile->is_locked = new_lock_value;
+    return 0;
+}
+
 /* Get entry for reading a pattern from clist. */
 int
 gx_pattern_cache_get_entry(gs_gstate * pgs, gs_id id, gx_color_tile ** pctile)
@@ -1205,6 +1261,7 @@ gx_pattern_cache_add_dummy_entry(gs_gstate *pgs,
     ctile->is_simple = pinst->is_simple;
     ctile->has_overlap = pinst->has_overlap;
     ctile->is_dummy = true;
+    ctile->is_locked = false;
     memset(&ctile->tbits, 0 , sizeof(ctile->tbits));
     ctile->tbits.size = pinst->size;
     ctile->tbits.id = gs_no_bitmap_id;
@@ -1222,10 +1279,11 @@ gx_pattern_cache_add_dummy_entry(gs_gstate *pgs,
    file name */
 static void
 dump_raw_pattern(int height, int width, int n_chan, int depth,
-                byte *Buffer, int raster, const gx_device_memory * mdev)
+                byte *Buffer, int raster, const gx_device_memory * mdev,
+                const gs_memory_t *memory)
 {
     char full_file_name[50];
-    FILE *fid;
+    gp_file *fid;
     int max_bands;
     int j, k, m;
     int byte_number, bit_position;
@@ -1244,17 +1302,17 @@ dump_raw_pattern(int height, int width, int n_chan, int depth,
         gs_sprintf(full_file_name, "%d)PATTERN_CHUNK_%dx%dx%d.raw", global_pat_index,
                 width, height, max_bands);
     }
-    fid = gp_fopen(full_file_name,"wb");
+    fid = gp_fopen(memory,full_file_name,"wb");
     if (depth >= 8) {
         /* Contone data. */
         if (is_planar) {
             for (m = 0; m < max_bands; m++) {
                 curr_ptr = mdev->line_ptrs[m*mdev->height];
-                fwrite(curr_ptr, 1, mdev->height * mdev->raster, fid);
+                gp_fwrite(curr_ptr, 1, mdev->height * mdev->raster, fid);
             }
         } else {
             /* Just dump it like it is */
-            fwrite(Buffer, 1, max_bands * height * width, fid);
+            gp_fwrite(Buffer, 1, max_bands * height * width, fid);
         }
     } else {
         /* Binary Data. Lets get to 8 bit for debugging.  We have to
@@ -1270,7 +1328,7 @@ dump_raw_pattern(int height, int width, int n_chan, int depth,
                         current_byte = curr_ptr[j*(mdev->raster) + byte_number];
                         bit_position = 7 - (k -  byte_number*8);
                         output_val = ((current_byte >> bit_position) & 0x1) * 255;
-                        fwrite(&output_val,1,1,fid);
+                        gp_fwrite(&output_val,1,1,fid);
                     }
                 }
             }
@@ -1291,19 +1349,19 @@ dump_raw_pattern(int height, int width, int n_chan, int depth,
                         /* extract and create byte */
                         output_val =
                                 ((current_byte >> bit_position) & 0x1) * 255;
-                        fwrite(&output_val,1,1,fid);
+                        gp_fwrite(&output_val,1,1,fid);
                     }
                 }
             }
         }
     }
-    fclose(fid);
+    gp_fclose(fid);
 }
 #endif
 
 static void
 make_bitmap(register gx_strip_bitmap * pbm, const gx_device_memory * mdev,
-            gx_bitmap_id id)
+            gx_bitmap_id id, const gs_memory_t *memory)
 {
     pbm->data = mdev->base;
     pbm->raster = mdev->raster;
@@ -1320,7 +1378,7 @@ make_bitmap(register gx_strip_bitmap * pbm, const gx_device_memory * mdev,
                         mdev->color_info.num_components,
                         mdev->color_info.depth,
                         (unsigned char*) mdev->base,
-                        pbm->raster, mdev);
+                        pbm->raster, mdev, memory);
 
         global_pat_index++;
 
@@ -1340,6 +1398,7 @@ gx_pattern_cache_winnow(gx_pattern_cache * pcache,
     for (i = 0; i < pcache->num_tiles; ++i) {
         gx_color_tile *ctile = &pcache->tiles[i];
 
+        ctile->is_locked = false;		/* force freeing */
         if (ctile->id != gx_no_bitmap_id && (*proc) (ctile, proc_data))
             gx_pattern_cache_free_entry(pcache, ctile);
     }
@@ -1437,7 +1496,7 @@ gx_pattern_load(gx_device_color * pdc, const gs_gstate * pgs,
         goto fail;
     if (pinst->templat.uses_transparency) {
         if_debug0m('v', mem, "gx_pattern_load: pushing the pdf14 compositor device into this graphics state\n");
-        if ((code = gs_push_pdf14trans_device(saved, true)) < 0)
+        if ((code = gs_push_pdf14trans_device(saved, true, false)) < 0)
             return code;
         saved->device->is_open = true;
     } else {
@@ -1475,6 +1534,7 @@ gx_pattern_load(gx_device_color * pdc, const gs_gstate * pgs,
                 gs_free_object(((gx_device_pattern_accum *)adev)->bitmap_memory,
                                ((gx_device_pattern_accum *)adev)->transbuff,
                                "gx_pattern_load");
+                ((gx_device_pattern_accum *)adev)->transbuff = NULL;
             }
             dev_proc(adev, close_device)((gx_device *)adev);
             /* adev was the target of the pdf14 device, so also is no longer retained */
@@ -1483,6 +1543,8 @@ gx_pattern_load(gx_device_color * pdc, const gs_gstate * pgs,
         dev_proc(saved->device, close_device)((gx_device *)saved->device);
         /* Freeing the state should now free the device which may be the pdf14 compositor. */
         gs_gstate_free_chain(saved);
+        if (code == gs_error_handled)
+            code = 0;
         return code;
     }
     if (pinst->templat.uses_transparency) {
@@ -1509,7 +1571,7 @@ gx_pattern_load(gx_device_color * pdc, const gs_gstate * pgs,
     }
     /* We REALLY don't like the following cast.... */
     code = gx_pattern_cache_add_entry((gs_gstate *)pgs,
-                adev, &ctile);
+                adev, &ctile, saved);
     if (code >= 0) {
         if (!gx_pattern_cache_lookup(pdc, pgs, dev, select)) {
             mlprintf(mem, "Pattern cache lookup failed after insertion!\n");
