@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -32,18 +32,8 @@
 
 #define RAW_PATTERN_DUMP 0
 
-#ifndef gx_device_clist_DEFINED
-#define gx_device_clist_DEFINED
 typedef union gx_device_clist_s gx_device_clist;
-#endif
 
-/*
- * Define the type of a Pattern, also used with Pattern instances.
- */
-#ifndef gs_pattern_type_DEFINED
-#  define gs_pattern_type_DEFINED
-typedef struct gs_pattern_type_s gs_pattern_type_t;
-#endif
 struct gs_pattern_type_s {
     int PatternType;
     struct pp_ {
@@ -168,22 +158,25 @@ struct gx_pattern_trans_s {
     int planestride;
     int n_chan; /* number of pixel planes including alpha */
     bool has_shape;  /* extra plane inserted */
+    bool has_tags;	/* and yet another plane for the tag */
     int width; /* Complete plane width/height; rect may be a subset of this */
     int height;
     const pdf14_nonseparable_blending_procs_t *blending_procs;
     bool is_additive;
-    gs_int_rect *dirty;
+    bool deep;
+    void *buf;
     void (* pat_trans_fill)(int xmin, int ymin, int xmax, int ymax, int px,
                             int py, const gx_color_tile *ptile,
-                            gx_pattern_trans_t *fill_trans_buffer);
+                            gx_pattern_trans_t *fill_trans_buffer,
+                            int native16);
     int (* image_render)(gx_image_enum * penum, const byte * buffer,
                             int data_x, uint w, int h, gx_device * dev);
 };
 
 #define private_st_pattern_trans() /* in gxpcmap.c */\
-gs_private_st_ptrs3(st_pattern_trans, gx_pattern_trans_t, "gx_pattern_trans",\
+gs_private_st_ptrs4(st_pattern_trans, gx_pattern_trans_t, "gx_pattern_trans",\
                     pattern_trans_enum_ptrs, pattern_trans_reloc_ptrs,\
-                    pdev14, transbytes, fill_trans_buffer)
+                    pdev14, transbytes, fill_trans_buffer, buf)
 /*
  * Define a color tile, an entry in the rendered Pattern cache (and
  * eventually in the colored halftone cache).  Note that the depth is
@@ -221,17 +214,17 @@ struct gx_color_tile_s {
     byte has_overlap;           /* true if step size is smaller than bounding box */
     byte is_dummy;		/* if true, the device manages the pattern,
                                    and the content of the tile is empty. */
-    byte pad[2];		/* structure members alignment. */
-    /* The following is neither key nor value. */
-    uint index;			/* the index of the tile within */
-    bool trans_group_popped;    /* Used to avoid multiple group pops in image mask fills */
-    bool is_planar;             /* Has to be stored here due to the device
+    byte trans_group_popped;    /* Used to avoid multiple group pops in image mask fills */
+    byte is_planar;             /* Has to be stored here due to the device
                                    change that can occur when the tile is
                                    created and when it is written in the clist
                                    when we are writing to a transparency
                                    device which, is not planar but the target
                                    is */
-    /* the cache (for GC) */
+    byte is_locked;		/* stroke patterns cannot be freed during fill_stroke_path */
+    byte pad[2];		/* structure members alignment. */
+    /* The following is neither key nor value. */
+    uint index;			/* the index of the tile within the cache (for GC) */
 };
 
 #define private_st_color_tile()	/* in gxpcmap.c */\
@@ -313,12 +306,15 @@ void gx_pattern_cache_update_space(gs_gstate * pgs, int64_t used);
 /* device, but it may zero out the bitmap_memory pointers to prevent */
 /* the accumulated bitmaps from being freed when the device is closed. */
 int gx_pattern_cache_add_entry(gs_gstate *, gx_device_forward *,
-                               gx_color_tile **);
+                               gx_color_tile **, gs_gstate*);
 /* Add a dummy Pattern cache entry.  Stubs a pattern tile for interpreter when
    device handles high level patterns. */
 int gx_pattern_cache_add_dummy_entry(gs_gstate *pgs, gs_pattern1_instance_t *pinst,
                                 int depth);
 
+/* set or clear the lock for a tile in the cache. Returns error if tile not in cache */
+int gx_pattern_cache_entry_set_lock(gs_gstate * pgs, gs_id id, bool new_lock_value);
+
 /* Get entry for reading a pattern from clist. */
 int gx_pattern_cache_get_entry(gs_gstate * pgs, gs_id id, gx_color_tile ** pctile);
 
@@ -344,20 +340,23 @@ int gx_trans_pattern_fill_rect(int xmin, int ymin, int xmax, int ymax,
                                gx_color_tile *ptile,
                                gx_pattern_trans_t *fill_trans_buffer,
                                gs_int_point phase, gx_device *dev,
-                               const gx_device_color * pdevc);
+                               const gx_device_color * pdevc,
+                               int native16);
 
 gx_pattern_trans_t* new_pattern_trans_buff(gs_memory_t *mem);
 
 void tile_rect_trans_simple(int xmin, int ymin, int xmax, int ymax, int px,
                             int py, const gx_color_tile *ptile,
-                            gx_pattern_trans_t *fill_trans_buffer);
+                            gx_pattern_trans_t *fill_trans_buffer,
+                            int native16);
 
 /* This is used for the case when we may have overlapping tiles.
    We need to get better detection for this as
    it would be best to avoid doing it if not needed. */
 void tile_rect_trans_blend(int xmin, int ymin, int xmax, int ymax, int px,
                            int py, const gx_color_tile *ptile,
-                           gx_pattern_trans_t *fill_trans_buffer);
+                           gx_pattern_trans_t *fill_trans_buffer,
+                           int native16);
 
 /* File a colored pattern with white */
 int gx_erase_colored_pattern(gs_gstate *pgs);
