@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 
@@ -389,13 +389,15 @@ gs_type1_piece_codes(/*const*/ gs_font_type1 *pfont,
     for (;;) {
         uint c0;
 
-        if (cip > end)
+        if (cip >= end) {
              /* We used to treat buffer overrun as a simple invalid font, now we assume that
-             * there is an implicit endcharr.
+             * there is an implicit endchar/return.
              * Part of bug #693170 where the fonts are invalid (no endchar on some glyphs).
              */
+            if (call_depth > 0)
+                goto c_return;
             goto out;
-
+        }
         c0 = *cip++;
 
         charstring_next(c0, state, c, encrypted);
@@ -490,6 +492,7 @@ gs_type1_piece_codes(/*const*/ gs_font_type1 *pfont,
             end = ipsp->cs_data.bits.data + ipsp->cs_data.bits.size;
             goto call;
         case c_return:
+c_return:
             if (call_depth == 0)
                 return (gs_note_error(gs_error_invalidfont));
             else
@@ -515,9 +518,14 @@ gs_type1_piece_codes(/*const*/ gs_font_type1 *pfont,
                 goto out;	/* not seac */
         do_seac:
             /* This is the payoff for all this code! */
-            chars[0] = fixed2int(csp[-1]);
-            chars[1] = fixed2int(csp[0]);
-            return 1;
+            if (CS_CHECK_CSTACK_BOUNDS(&csp[-1], cstack)) {
+                chars[0] = fixed2int(csp[-1]);
+                chars[1] = fixed2int(csp[0]);
+                return 1;
+            }
+            else {
+                return_error(gs_error_invalidfont);
+            }
         case cx_escape:
             charstring_next(*cip, state, c, encrypted);
             ++cip;
