@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2012 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -9,8 +9,8 @@
    of the license contained in the file LICENSE in this distribution.
 
    Refer to licensing information at http://www.artifex.com or contact
-   Artifex Software, Inc.,  7 Mt. Lassen Drive - Suite A-134, San Rafael,
-   CA  94903, U.S.A., +1(415)492-9861, for further information.
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
 */
 
 #ifndef __RAMFS_H__
@@ -52,7 +52,6 @@ typedef struct _ramfs_enum ramfs_enum;
 #define RAMFS_APPEND s_mode_append  /* 8 */
 #define RAMFS_CREATE 16
 #define RAMFS_TRUNC  32
-#define RAMFS_APPEND 64
 
 #define RAMFS_SEEK_SET 0
 #define RAMFS_SEEK_CUR 1
