@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -883,7 +883,7 @@ calculate_dst_contrib(stream_IScale_state * ss, int y)
                  i >= first_index_mod ?
                  ss->dst_items[i - first_index_mod].weight :
                  0);
-            if_debug1m('W', ss->memory, " %f", shuffle[i].weight);
+            if_debug1m('W', ss->memory, " %d", shuffle[i].weight);
         }
         memcpy(ss->dst_items, shuffle, ss->max_support * sizeof(CONTRIB));
         ss->dst_next_list.n = ss->max_support;
@@ -1193,7 +1193,7 @@ s_IScale_release(stream_state * st)
     ss->dst = 0;
     gs_free_object(mem, ss->items, "image_scale contrib[*]");
     ss->items = 0;
-    gs_free_object(mem, ss->items, "image_scale contrib_dst[*]");
+    gs_free_object(mem, ss->dst_items, "image_scale contrib_dst[*]");
     ss->dst_items = 0;
     gs_free_object(mem, ss->contrib, "image_scale contrib");
     ss->contrib = 0;
