@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -38,20 +38,16 @@ int sjpxd_create(gs_memory_t *mem)
 #if !defined(SHARE_JPX) || (SHARE_JPX == 0)
     gs_lib_ctx_t *ctx = mem->gs_lib_ctx;
 
-#ifdef MEMENTO_SQUEEZE_BUILD
-    ctx->sjpxd_private = NULL;
-#else
     ctx->sjpxd_private = gx_monitor_label(gx_monitor_alloc(mem), "sjpxd_monitor");
     if (ctx->sjpxd_private == NULL)
         return gs_error_VMerror;
-#endif
 #endif
     return 0;
 }
 
 void sjpxd_destroy(gs_memory_t *mem)
 {
-#if (!defined(SHARE_JPX) || (SHARE_JPX == 0)) && !defined(MEMENTO_SQUEEZE_BUILD)
+#if !defined(SHARE_JPX) || (SHARE_JPX == 0)
     gs_lib_ctx_t *ctx = mem->gs_lib_ctx;
 
     gx_monitor_free((gx_monitor_t *)ctx->sjpxd_private);
@@ -64,13 +60,9 @@ static int opj_lock(gs_memory_t *mem)
 #if !defined(SHARE_JPX) || (SHARE_JPX == 0)
     int ret;
 
-#ifdef MEMENTO_SQUEEZE_BUILD
-    ret = 0;
-#else
     gs_lib_ctx_t *ctx = mem->gs_lib_ctx;
 
     ret = gx_monitor_enter((gx_monitor_t *)ctx->sjpxd_private);
-#endif
     assert(opj_memory == NULL);
     opj_memory = mem->non_gc_memory;
     return ret;
@@ -86,12 +78,7 @@ static int opj_unlock(gs_memory_t *mem)
 
     assert(opj_memory != NULL);
     opj_memory = NULL;
-#ifdef MEMENTO_SQUEEZE_BUILD
-    (void)ctx;
-    return 0;
-#else
     return gx_monitor_leave((gx_monitor_t *)ctx->sjpxd_private);
-#endif
 #else
     return 0;
 #endif
@@ -106,6 +93,9 @@ void *opj_malloc(size_t size)
 
     assert(opj_memory != NULL);
 
+    if (size > (size_t) ARCH_MAX_UINT)
+	    return NULL;
+
     return (void *)gs_alloc_bytes(opj_memory, size, "opj_malloc");
 }
 
@@ -199,8 +189,8 @@ static OPJ_SIZE_T sjpx_stream_read(void * p_buffer, OPJ_SIZE_T p_nb_bytes, void
 	stream_block *sb = (stream_block *)p_user_data;
 	OPJ_SIZE_T len;
 
-	len = sb->size - sb->pos;
-        if (sb->size < sb->pos)
+	len = sb->fill - sb->pos;
+        if (sb->fill < sb->pos)
 		len = 0;
 	if (len == 0)
 		return (OPJ_SIZE_T)-1;  /* End of file! */
@@ -215,8 +205,8 @@ static OPJ_OFF_T sjpx_stream_skip(OPJ_OFF_T skip, void * p_user_data)
 {
 	stream_block *sb = (stream_block *)p_user_data;
 
-	if (skip > sb->size - sb->pos)
-		skip = sb->size - sb->pos;
+	if (skip > sb->fill - sb->pos)
+		skip = sb->fill - sb->pos;
 	sb->pos += skip;
 	return sb->pos;
 }
@@ -225,7 +215,7 @@ static OPJ_BOOL sjpx_stream_seek(OPJ_OFF_T seek_pos, void * p_user_data)
 {
 	stream_block *sb = (stream_block *)p_user_data;
 
-	if (seek_pos > sb->size)
+	if (seek_pos > sb->fill)
 		return OPJ_FALSE;
 	sb->pos = seek_pos;
 	return OPJ_TRUE;
@@ -436,7 +426,7 @@ static int decode_image(stream_jpxd_state * const state)
     {
         if (state->image->comps[compno].alpha == 0x00)
             numprimcomp++;
-        else if (state->image->comps[compno].alpha == 0x01)
+        else if (state->image->comps[compno].alpha == 0x01 || state->image->comps[compno].alpha == 0x02)
             alpha_comp = compno;
     }
 
