@@ -1,4 +1,4 @@
-/* Copyright (C) 2001-2018 Artifex Software, Inc.
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
    All Rights Reserved.
 
    This software is provided AS-IS with no warranty, either express or
@@ -49,6 +49,8 @@
 #include "gdevpccm.h"		/* 4-bit PC color */
 #include "gxdevmem.h"
 #include "gdevdevn.h"
+#include "gxpcolor.h"		/* for gx_dc_devn_masked */
+#include "gxdevsop.h"
 #include "gsequivc.h"
 #include "gdevdsp.h"
 #include "gdevdsp2.h"
@@ -94,6 +96,8 @@ static dev_proc_encode_color(display_separation_encode_color);
 static dev_proc_decode_color(display_separation_decode_color);
 static dev_proc_update_spot_equivalent_colors(display_update_spot_equivalent_colors);
 static dev_proc_ret_devn_params(display_ret_devn_params);
+static dev_proc_dev_spec_op(display_spec_op);
+static dev_proc_fill_rectangle_hl_color(display_fill_rectangle_hl_color);
 
 static const gx_device_procs display_procs =
 {
@@ -152,13 +156,18 @@ static const gx_device_procs display_procs =
     NULL,           			/* encode_color */
     NULL,           			/* decode_color */
     NULL,                          	/* pattern_manage */
-    NULL,				/* fill_rectangle_hl_color */\
+    display_fill_rectangle_hl_color,	/* fill_rectangle_hl_color */\
     NULL,				/* include_color_space */\
     NULL,				/* fill_linear_color_scanline */\
     NULL,				/* fill_linear_color_trapezoid */\
     NULL,				/* fill_linear_color_triangle */\
     display_update_spot_equivalent_colors, /* update_spot_equivalent_colors */
-    display_ret_devn_params		/* ret_devn_params */\
+    display_ret_devn_params,		/* ret_devn_params */\
+    NULL,                        /* fillpage */\
+    NULL,                        /* push_transparency_state */\
+    NULL,                        /* pop_transparency_state */\
+    NULL,                        /* put_image */\
+    display_spec_op              /* dev_spec_op */\
 };
 
 /* GC descriptor */
@@ -248,6 +257,9 @@ display_open(gx_device * dev)
         return ccode;
     dev = (gx_device *)ddev;
 
+    while(dev->parent)
+        dev = dev->parent;
+
     /* Make sure we have been passed a valid callback structure. */
     if ((ccode = display_check_structure(ddev)) < 0)
         return_error(ccode);
@@ -309,6 +321,9 @@ display_sync_output(gx_device * dev)
         return 0;	/* ignore the call */
     display_set_separations(ddev);
 
+    while(dev->parent)
+        dev = dev->parent;
+
     (*(ddev->callback->display_sync))(ddev->pHandle, dev);
     return (0);
 }
@@ -324,6 +339,9 @@ display_output_page(gx_device * dev, int copies, int flush)
         return gs_error_Fatal;
     display_set_separations(ddev);
 
+    while(dev->parent)
+        dev = dev->parent;
+
     code = (*(ddev->callback->display_page))
                         (ddev->pHandle, dev, copies, flush);
 
@@ -346,6 +364,9 @@ display_close(gx_device * dev)
     /* Release memory. */
     display_free_bitmap(ddev);
 
+    while(dev->parent)
+        dev = dev->parent;
+
     /* Tell caller that device is closed. */
     /* This is always the last callback */
     (*(ddev->callback->display_close))(ddev->pHandle, dev);
@@ -579,7 +600,7 @@ display_map_rgb_color_rgb(gx_device * dev, const gx_color_value cv[])
                 return gx_default_rgb_map_rgb_color(dev, rgb); /* RGB */
             }
             else
-                return (blue<<16) + (green<<8) + red;		/* BGR */
+                return ((gx_color_index)blue<<16) + (green<<8) + red;		/* BGR */
         case DISPLAY_ALPHA_FIRST:
         case DISPLAY_UNUSED_FIRST:
             if ((ddev->nFormat & DISPLAY_ENDIAN_MASK) == DISPLAY_BIGENDIAN)
@@ -589,9 +610,9 @@ display_map_rgb_color_rgb(gx_device * dev, const gx_color_value cv[])
         case DISPLAY_ALPHA_LAST:
         case DISPLAY_UNUSED_LAST:
             if ((ddev->nFormat & DISPLAY_ENDIAN_MASK) == DISPLAY_BIGENDIAN)
-                return ((gx_color_index)red<<24) + (green<<16) + (blue<<8);	/* RGBx */
+                return ((gx_color_index)red<<24) + ((gx_color_index)green<<16) + (blue<<8);	/* RGBx */
             else
-                return ((gx_color_index)blue<<24) + (green<<16) + (red<<8);	/* BGRx */
+                return ((gx_color_index)blue<<24) + ((gx_color_index)green<<16) + (red<<8);	/* BGRx */
     }
     return 0;
 }
@@ -712,6 +733,10 @@ display_fill_rectangle(gx_device * dev, int x, int y, int w, int h,
         return 0;		/* ignore -- needed for fillpage when device wasn't really opened */
     dev_proc(ddev->mdev, fill_rectangle)((gx_device *)ddev->mdev,
         x, y, w, h, color);
+
+    while(dev->parent)
+        dev = dev->parent;
+
     if (ddev->callback->display_update)
         (*(ddev->callback->display_update))(ddev->pHandle, dev, x, y, w, h);
     return 0;
@@ -729,6 +754,10 @@ display_copy_mono(gx_device * dev,
         return gs_error_Fatal;
     dev_proc(ddev->mdev, copy_mono)((gx_device *)ddev->mdev,
         base, sourcex, raster, id, x, y, w, h, zero, one);
+
+    while(dev->parent)
+        dev = dev->parent;
+
     if (ddev->callback->display_update)
         (*(ddev->callback->display_update))(ddev->pHandle, dev, x, y, w, h);
     return 0;
@@ -745,6 +774,10 @@ display_copy_color(gx_device * dev,
         return gs_error_Fatal;
     dev_proc(ddev->mdev, copy_color)((gx_device *)ddev->mdev,
         base, sourcex, raster, id, x, y, w, h);
+
+    while(dev->parent)
+        dev = dev->parent;
+
     if (ddev->callback->display_update)
         (*(ddev->callback->display_update))(ddev->pHandle, dev, x, y, w, h);
     return 0;
@@ -822,13 +855,11 @@ display_put_params(gx_device * dev, gs_param_list * plist)
     int old_height = dev->height;
     int old_format = ddev->nFormat;
     void *old_handle = ddev->pHandle;
-
     gs_devn_params *pdevn_params = &ddev->devn_params;
     equivalent_cmyk_color_params *pequiv_colors = &ddev->equiv_cmyk_colors;
     /* Save current data in case we have a problem */
     gs_devn_params saved_devn_params = *pdevn_params;
     equivalent_cmyk_color_params saved_equiv_colors = *pequiv_colors;
-
     int format;
     void *handle;
     int found_string_handle = 0;
@@ -982,11 +1013,11 @@ display_put_params(gx_device * dev, gs_param_list * plist)
         /* Use utility routine to handle devn parameters */
         ecode = devn_put_params(dev, plist, pdevn_params, pequiv_colors);
         /*
-         * Setting MaxSeparations changes color_info.depth in
-         * devn_put_params, but we always use 64bpp,
-         * so reset it to the the correct value.
+         * If we support_devn, setting MaxSeparations or PageSpotColors changed the
+         * color_info.depth in devn_put_params, but we always use 64bpp, so reset it
+         * to the correct value.
          */
-        dev->color_info.depth = ARCH_SIZEOF_COLOR_INDEX * 8;
+        ddev->color_info.depth = ARCH_SIZEOF_COLOR_INDEX * 8;
     }
 
     if (ecode >= 0) {
@@ -1012,9 +1043,15 @@ display_put_params(gx_device * dev, gs_param_list * plist)
         /* We can resize this device while it is open, but we cannot
          * change the color format or handle.
          */
+
+        while(dev->parent) {
+            dev = dev->parent;
+            gx_update_from_subclass(dev);
+        }
+
         /* Tell caller we are about to change the device parameters */
         if ((*ddev->callback->display_presize)(ddev->pHandle, dev,
-            dev->width, dev->height, display_raster(ddev),
+            ddev->width, ddev->height, display_raster(ddev),
             ddev->nFormat) < 0) {
             /* caller won't let us change the size */
             /* restore parameters then return an error */
@@ -1023,11 +1060,17 @@ display_put_params(gx_device * dev, gs_param_list * plist)
             display_set_color_format(ddev, old_format);
             ddev->nFormat = old_format;
             ddev->pHandle = old_handle;
-            dev->width = old_width;
-            dev->height = old_height;
+            ddev->width = old_width;
+            ddev->height = old_height;
             return_error(gs_error_rangecheck);
         }
 
+        dev = (gx_device *)ddev;
+        while(dev->parent) {
+            dev = dev->parent;
+            gx_update_from_subclass(dev);
+        }
+
         display_free_bitmap(ddev);
 
         code = display_alloc_bitmap(ddev, dev);
@@ -1055,7 +1098,15 @@ display_put_params(gx_device * dev, gs_param_list * plist)
             ddev->nFormat, ddev->mdev->base) < 0)
             return_error(gs_error_rangecheck);
     }
-
+        /*
+         * Make the color_info.depth correct for the bpc and num_components since
+         * devn mode always has the display bitmap set up for 64-bits, but others,
+         * such as pdf14 compositor expect it to match (for "deep" detection).
+         */
+        if (ddev->icc_struct && ddev->icc_struct->supports_devn) {
+            ddev->color_info.depth = ddev->devn_params.bitspercomponent *
+                                         ddev->color_info.num_components;
+        }
     return 0;
 }
 
@@ -1128,7 +1179,6 @@ static gx_color_index
 display_separation_encode_color(gx_device *dev, const gx_color_value colors[])
 {
     int bpc = ((gx_device_display *)dev)->devn_params.bitspercomponent;
-    int drop = sizeof(gx_color_value) * 8 - bpc;
     gx_color_index color = 0;
     int i = 0;
     int ncomp = dev->color_info.num_components;
@@ -1152,7 +1202,6 @@ display_separation_decode_color(gx_device * dev, gx_color_index color,
     gx_color_value * out)
 {
     int bpc = ((gx_device_display *)dev)->devn_params.bitspercomponent;
-    int drop = sizeof(gx_color_value) * 8 - bpc;
     int mask = (1 << bpc) - 1;
     int i = 0;
     int ncomp = dev->color_info.num_components;
@@ -1193,6 +1242,37 @@ display_ret_devn_params(gx_device * dev)
     return &pdev->devn_params;
 }
 
+static int
+display_spec_op(gx_device *dev, int op, void *data, int datasize)
+{
+
+    if (op == gxdso_supports_devn) {
+        return (dev_proc(dev, fill_rectangle_hl_color) == display_fill_rectangle_hl_color);
+    }
+    return gx_default_dev_spec_op(dev, op, data, datasize);
+}
+
+/* Fill a rectangle with a high level color.  This is used in separation mode */
+static int
+display_fill_rectangle_hl_color(gx_device *dev, const gs_fixed_rect *rect,
+    const gs_gstate *pgs, const gx_drawing_color *pdcolor,
+    const gx_clip_path *pcpath)
+{
+    int x = fixed2int(rect->p.x);
+    int y = fixed2int(rect->p.y);
+    int w = fixed2int(rect->q.x) - x;
+    int h = fixed2int(rect->q.y) - y;
+    gx_color_index pure_color;
+
+    /* We can only handle devn cases, so use the default if not */
+    /* We can get called here from gx_dc_devn_masked_fill_rectangle */
+    if (pdcolor->type != gx_dc_type_devn && pdcolor->type != &gx_dc_devn_masked) {
+        return gx_fill_rectangle_device_rop( x, y, w, h, pdcolor, dev, lop_default);
+    }
+    pure_color = display_separation_encode_color(dev, pdcolor->colors.devn.values);
+    return display_fill_rectangle(dev, x, y, w, h, pure_color);
+}
+
 /*
  * This routine will check to see if the color component name  match those
  * that are available amoung the current device's color components.
@@ -1328,6 +1408,7 @@ display_alloc_bitmap(gx_device_display * ddev, gx_device * param_dev)
 {
     int ccode;
     const gx_device_memory *mdproto;
+
     if (ddev->callback == NULL)
         return gs_error_Fatal;
 
@@ -1393,6 +1474,10 @@ display_alloc_bitmap(gx_device_display * ddev, gx_device * param_dev)
         display_free_bitmap(ddev);
 
     /* erase bitmap - before display gets redrawn */
+    /*
+     * Note that this will fill all 64 bits even if we've reset depth in the
+     * devn case, since the underlying mdev is 64-bit (see above).
+     */
     if (ccode == 0) {
         int i;
         gx_color_value cv[GX_DEVICE_COLOR_MAX_COMPONENTS];
@@ -1471,6 +1556,9 @@ display_set_separations(gx_device_display *dev)
                            * 65535 / frac_1;
                 }
             }
+            while(dev->parent)
+                dev = (gx_device_display *)dev->parent;
+
             (*dev->callback->display_separation)(dev->pHandle, dev,
                 comp_num, name,
                 (unsigned short)c, (unsigned short)m,
@@ -1544,16 +1632,14 @@ set_color_procs(gx_device * pdev,
         dev_t_proc_encode_color((*encode_color), gx_device),
         dev_t_proc_decode_color((*decode_color), gx_device),
         dev_t_proc_get_color_mapping_procs((*get_color_mapping_procs), gx_device),
-        dev_t_proc_get_color_comp_index((*get_color_comp_index), gx_device))
+        dev_t_proc_get_color_comp_index((*get_color_comp_index), gx_device),
+        dev_t_proc_fill_rectangle_hl_color((*fill_hl_color), gx_device))
 {
-#if 0				/* These procs are no longer used */
-    pdev->procs.map_rgb_color = encode_color;
-    pdev->procs.map_color_rgb = decode_color;
-#endif
     pdev->procs.get_color_mapping_procs = get_color_mapping_procs;
     pdev->procs.get_color_comp_index = get_color_comp_index;
     pdev->procs.encode_color = encode_color;
     pdev->procs.decode_color = decode_color;
+    pdev->procs.fill_rectangle_hl_color = fill_hl_color;
 }
 
 /*
@@ -1567,7 +1653,8 @@ set_gray_color_procs(gx_device * pdev,
 {
     set_color_procs(pdev, encode_color, decode_color,
         gx_default_DevGray_get_color_mapping_procs,
-        gx_default_DevGray_get_color_comp_index);
+        gx_default_DevGray_get_color_comp_index,
+        gx_default_fill_rectangle_hl_color);
 }
 
 /*
@@ -1581,7 +1668,8 @@ set_rgb_color_procs(gx_device * pdev,
 {
     set_color_procs(pdev, encode_color, decode_color,
         gx_default_DevRGB_get_color_mapping_procs,
-        gx_default_DevRGB_get_color_comp_index);
+        gx_default_DevRGB_get_color_comp_index,
+        gx_default_fill_rectangle_hl_color);
 }
 
 /*
@@ -1595,7 +1683,8 @@ set_rgbk_color_procs(gx_device * pdev,
 {
     set_color_procs(pdev, encode_color, decode_color,
         gx_default_DevRGBK_get_color_mapping_procs,
-        gx_default_DevRGBK_get_color_comp_index);
+        gx_default_DevRGBK_get_color_comp_index,
+        gx_default_fill_rectangle_hl_color);
 }
 
 /*
@@ -1609,7 +1698,8 @@ set_cmyk_color_procs(gx_device * pdev,
 {
     set_color_procs(pdev, encode_color, decode_color,
         gx_default_DevCMYK_get_color_mapping_procs,
-        gx_default_DevCMYK_get_color_comp_index);
+        gx_default_DevCMYK_get_color_comp_index,
+        gx_default_fill_rectangle_hl_color);
 }
 
 /* Set the color_info and mapping functions for this instance of the device */
@@ -1777,11 +1867,13 @@ display_set_color_format(gx_device_display *ddev, int nFormat)
                 maxvalue, maxvalue);
             if ((nFormat & DISPLAY_DEPTH_MASK) == DISPLAY_DEPTH_8) {
                 ddev->devn_params.bitspercomponent = bpc;
+                ddev->icc_struct->supports_devn = true;
                 set_color_procs(pdev,
                     display_separation_encode_color,
                     display_separation_decode_color,
                     display_separation_get_color_mapping_procs,
-                    display_separation_get_color_comp_index);
+                    display_separation_get_color_comp_index,
+                    display_fill_rectangle_hl_color);
             }
             else
                 return_error(gs_error_rangecheck);
