@@ -1,3 +1,18 @@
+/* Copyright (C) 2001-2019 Artifex Software, Inc.
+   All Rights Reserved.
+
+   This software is provided AS-IS with no warranty, either express or
+   implied.
+
+   This software is distributed under license and may not be copied,
+   modified or distributed except as expressly authorized under the terms
+   of the license contained in the file LICENSE in this distribution.
+
+   Refer to licensing information at http://www.artifex.com or contact
+   Artifex Software, Inc.,  1305 Grant Avenue - Suite 200, Novato,
+   CA 94945, U.S.A., +1(415)492-9861, for further information.
+*/
+
 #include "stdint_.h"
 #include "gdevprn.h"
 #include "gxdownscale.h"
@@ -10,7 +25,6 @@
 #include "strimpl.h"
 #include "slzwx.h"
 #include "szlibx.h"
-#include "jpeglib.h"
 #include "sdct.h"
 #include "srlx.h"
 #include "gsicc_cache.h"
@@ -58,7 +72,7 @@ typedef struct pdfimage_page_s {
 
 typedef struct PCLm_temp_file_s {
     char file_name[gp_file_name_sizeof];
-    FILE *file;
+    gp_file *file;
     stream *strm;
     stream *save;
     byte *strm_buf;
@@ -172,8 +186,53 @@ const gx_device_pdf_image gs_pdfimage32_device = {
     0    /* JPEGQ */
 };
 
+static int
+pdfimage_write_args_comment(gx_device_pdf_image *pdev, stream *s)
+{
+    const char * const *argv = NULL;
+    const char *arg;
+    int towrite, length, i, j, argc;
+
+    argc = gs_lib_ctx_get_args(pdev->memory->gs_lib_ctx, &argv);
+
+    stream_write(s, (byte *)"%%Invocation:", 13);
+    length = 12;
+    for (i=0;i < argc; i++) {
+        arg = argv[i];
+
+        if ((strlen(arg) + length) > 255) {
+            stream_write(s, (byte *)"\n%%+ ", 5);
+            length = 5;
+        } else {
+            stream_write(s, (byte *)" ", 1);
+            length++;
+        }
+
+        if (strlen(arg) > 250)
+            towrite = 250;
+        else
+            towrite = strlen(arg);
+
+        length += towrite;
+
+        for (j=0;j < towrite;j++) {
+            if (arg[j] == 0x0A) {
+                stream_write(s, (byte *)"<0A>", 4);
+            } else {
+                if (arg[j] == 0x0D) {
+                    stream_write(s, (byte *)"<0D>", 4);
+                } else {
+                    stream_write(s, (byte *)&arg[j], 1);
+                }
+            }
+        }
+    }
+    stream_write(s, (byte *)"\n", 1);
+    return 0;
+}
+
 static int gdev_pdf_image_begin_page(gx_device_pdf_image *pdf_dev,
-                         FILE *file)
+                         gp_file *file)
 {
     gx_device_printer *const pdev = (gx_device_printer *)pdf_dev;
     cmm_dev_profile_t *profile_struct;
@@ -212,6 +271,10 @@ static int gdev_pdf_image_begin_page(gx_device_pdf_image *pdf_dev,
                     profile_struct->device_profile[0], profile_struct->postren_profile,
                     &rendering_params);
             }
+            if (pdf_dev->icclink == NULL) {
+                gs_free_object(pdf_dev->memory->non_gc_memory, page, "pdfimage create new page");
+                return_error(gs_error_VMerror);
+            }
             /* If it is identity, release it now and set link to NULL */
             if (pdf_dev->icclink->is_identity) {
                 pdf_dev->icclink->procs.free_link(pdf_dev->icclink);
@@ -222,8 +285,10 @@ static int gdev_pdf_image_begin_page(gx_device_pdf_image *pdf_dev,
 
         /* Set up the stream and insert the file header */
         pdf_dev->strm = s_alloc(pdf_dev->memory->non_gc_memory, "pdfimage_open_temp_stream(strm)");
-        if (pdf_dev->strm == 0)
+        if (pdf_dev->strm == 0) {
+            gs_free_object(pdf_dev->memory->non_gc_memory, page, "pdfimage create new page");
             return_error(gs_error_VMerror);
+        }
         pdf_dev->strm_buf = gs_alloc_bytes(pdf_dev->memory->non_gc_memory, pdf_dev->width * (pdf_dev->color_info.depth / 8),
                                        "pdfimage_open_temp_stream(strm_buf)");
         if (pdf_dev->strm_buf == 0) {
@@ -231,12 +296,14 @@ static int gdev_pdf_image_begin_page(gx_device_pdf_image *pdf_dev,
             gs_free_object(pdf_dev->memory->non_gc_memory, pdf_dev->strm,
                            "pdfimage_open_temp_stream(strm)");
             pdf_dev->strm = 0;
+            gs_free_object(pdf_dev->memory->non_gc_memory, page, "pdfimage create new page");
             return_error(gs_error_VMerror);
         }
         swrite_file(pdf_dev->strm, pdf_dev->file, pdf_dev->strm_buf, pdf_dev->width * (pdf_dev->color_info.depth / 8));
 
         stream_puts(pdf_dev->strm, "%PDF-1.3\n");
         stream_puts(pdf_dev->strm, "%\307\354\217\242\n");
+        pdfimage_write_args_comment(pdf_dev, pdf_dev->strm);
         pdf_dev->Pages = page;
     } else {
         pdfimage_page *current = pdf_dev->Pages;
@@ -587,9 +654,21 @@ pdf_compute_fileID(gx_device_pdf_image * pdev, byte fileID[16], char *CreationDa
     if (s == NULL)
         return_error(gs_error_VMerror);
 
-    gp_get_usertime(secs_ns);
+#ifdef CLUSTER
+    secs_ns[0] = 0;
+    secs_ns[1] = 0;
+#else
+    gp_get_realtime(secs_ns);
+#endif
     sputs(s, (byte *)secs_ns, sizeof(secs_ns), &ignore);
+#ifdef CLUSTER
+    /* Don't have the ID's vary by filename output in the cluster testing.
+     * This prevents us comparing gs to gpdl results, and makes it harder
+     * to manually reproduce results. */
+    sputs(s, (const byte *)"ClusterTest.pdf", strlen("ClusterTest.pdf"), &ignore);
+#else
     sputs(s, (const byte *)pdev->fname, strlen(pdev->fname), &ignore);
+#endif
 
     stream_puts(s, "/ModDate ");
     stream_puts(s, CreationDate);
@@ -644,13 +723,8 @@ static int pdf_image_finish_file(gx_device_pdf_image *pdf_dev, int PCLm)
 
         pdf_store_default_Producer(Producer);
 
-        if (PCLm) {
-            pdf_dev->RootOffset = stell(pdf_dev->strm);
-            stream_puts(pdf_dev->strm, "1 0 obj\n<<\n/Pages 2 0 R\n/Type /Catalog\n>>\nendobj\n");
-        } else {
-            pdf_dev->RootOffset = stell(pdf_dev->strm);
-            stream_puts(pdf_dev->strm, "1 0 obj\n<<\n/Pages 2 0 R\n/Type /Catalog\n>>\nendobj\n");
-        }
+        pdf_dev->RootOffset = stell(pdf_dev->strm);
+        stream_puts(pdf_dev->strm, "1 0 obj\n<<\n/Pages 2 0 R\n/Type /Catalog\n>>\nendobj\n");
 
         pdf_dev->PagesOffset = stell(pdf_dev->strm);
         pprintd1(pdf_dev->strm, "2 0 obj\n<<\n/Count %d\n", pdf_dev->NumPages);
@@ -663,6 +737,12 @@ static int pdf_image_finish_file(gx_device_pdf_image *pdf_dev, int PCLm)
 
         stream_puts(pdf_dev->strm, "]\n/Type /Pages\n>>\nendobj\n");
 
+#ifdef CLUSTER
+        memset(&t, 0, sizeof(t));
+        memset(&tms, 0, sizeof(tms));
+        timesign = 'Z';
+        timeoffset = 0;
+#else
         time(&t);
         tms = *gmtime(&t);
         tms.tm_isdst = -1;
@@ -670,6 +750,7 @@ static int pdf_image_finish_file(gx_device_pdf_image *pdf_dev, int PCLm)
         timesign = (timeoffset == 0 ? 'Z' : timeoffset < 0 ? '-' : '+');
         timeoffset = any_abs(timeoffset) / 60;
         tms = *localtime(&t);
+#endif
 
         gs_sprintf(CreationDate, "(D:%04d%02d%02d%02d%02d%02d%c%02d\'%02d\')",
             tms.tm_year + 1900, tms.tm_mon + 1, tms.tm_mday,
@@ -703,8 +784,8 @@ static int pdf_image_finish_file(gx_device_pdf_image *pdf_dev, int PCLm)
             char Buffer[1024];
 
             sflush(pdf_dev->xref_stream.strm);
-            streamsize = gp_ftell_64(pdf_dev->xref_stream.file);
-            if (gp_fseek_64(pdf_dev->xref_stream.file, 0, SEEK_SET) != 0)
+            streamsize = gp_ftell(pdf_dev->xref_stream.file);
+            if (gp_fseek(pdf_dev->xref_stream.file, 0, SEEK_SET) != 0)
                 return_error(gs_error_ioerror);
 
             while(streamsize > 0) {
@@ -719,7 +800,7 @@ static int pdf_image_finish_file(gx_device_pdf_image *pdf_dev, int PCLm)
                     streamsize = 0;
                 }
             }
-            if (gp_fseek_64(pdf_dev->xref_stream.file, 0, SEEK_SET) != 0)
+            if (gp_fseek(pdf_dev->xref_stream.file, 0, SEEK_SET) != 0)
                 return_error(gs_error_ioerror);
 
             pprintd1(pdf_dev->strm, "trailer\n<<\n/Size %d\n/Root 1 0 R\n/ID [", pdf_dev->NextObject);
@@ -764,7 +845,7 @@ static int pdf_image_finish_file(gx_device_pdf_image *pdf_dev, int PCLm)
 }
 
 static int
-pdf_image_print_page(gx_device_printer * pdev, FILE * file)
+pdf_image_print_page(gx_device_printer * pdev, gp_file * file)
 {
     gx_device_pdf_image *const pdf_dev = (gx_device_pdf_image *)pdev;
     const char *fmt;
@@ -1047,7 +1128,7 @@ PCLm_open_temp_file(gx_device_pdf_image *pdev, PCLm_temp_file_t *ptf)
 
     strcpy(fmode, "w+");
     strcat(fmode, gp_fmode_binary_suffix);
-    ptf->file =	gp_open_scratch_file_64(pdev->memory,
+    ptf->file =	gp_open_scratch_file(pdev->memory,
                                      gp_scratch_file_name_prefix,
                                      ptf->file_name,
                                      fmode);
@@ -1060,7 +1141,7 @@ static int
 PCLm_close_temp_file(gx_device_pdf_image *pdev, PCLm_temp_file_t *ptf, int code)
 {
     int err = 0;
-    FILE *file = ptf->file;
+    gp_file *file = ptf->file;
 
     /*
      * ptf->strm == 0 or ptf->file == 0 is only possible if this procedure
@@ -1082,7 +1163,7 @@ PCLm_close_temp_file(gx_device_pdf_image *pdev, PCLm_temp_file_t *ptf, int code)
         ptf->strm = 0;
     }
     if (file) {
-        err = ferror(file) | fclose(file);
+        err = gp_ferror(file) | gp_fclose(file);
         unlink(ptf->file_name);
         ptf->file = 0;
     }
@@ -1163,7 +1244,7 @@ PCLm_open(gx_device *pdev)
 }
 
 static int gdev_PCLm_begin_page(gx_device_pdf_image *pdf_dev,
-                         FILE *file)
+                         gp_file *file)
 {
     gx_device_printer *const pdev = (gx_device_printer *)pdf_dev;
     cmm_dev_profile_t *profile_struct;
@@ -1202,6 +1283,10 @@ static int gdev_PCLm_begin_page(gx_device_pdf_image *pdf_dev,
                     profile_struct->device_profile[0], profile_struct->postren_profile,
                     &rendering_params);
             }
+            if (pdf_dev->icclink == NULL) {
+                gs_free_object(pdf_dev->memory->non_gc_memory, page, "pdfimage create new page");
+                return_error(gs_error_VMerror);
+            }
             /* If it is identity, release it now and set link to NULL */
             if (pdf_dev->icclink->is_identity) {
                 pdf_dev->icclink->procs.free_link(pdf_dev->icclink);
@@ -1212,8 +1297,10 @@ static int gdev_PCLm_begin_page(gx_device_pdf_image *pdf_dev,
 
         /* Set up the stream and insert the file header */
         pdf_dev->strm = s_alloc(pdf_dev->memory->non_gc_memory, "pdfimage_open_temp_stream(strm)");
-        if (pdf_dev->strm == 0)
+        if (pdf_dev->strm == 0) {
+            gs_free_object(pdf_dev->memory->non_gc_memory, page, "pdfimage create new page");
             return_error(gs_error_VMerror);
+        }
         pdf_dev->strm_buf = gs_alloc_bytes(pdf_dev->memory->non_gc_memory, 512,
                                        "pdfimage_open_temp_stream(strm_buf)");
         if (pdf_dev->strm_buf == 0) {
@@ -1221,6 +1308,7 @@ static int gdev_PCLm_begin_page(gx_device_pdf_image *pdf_dev,
             gs_free_object(pdf_dev->memory->non_gc_memory, pdf_dev->strm,
                            "pdfimage_open_temp_stream(strm)");
             pdf_dev->strm = 0;
+            gs_free_object(pdf_dev->memory->non_gc_memory, page, "pdfimage create new page");
             return_error(gs_error_VMerror);
         }
         swrite_file(pdf_dev->strm, pdf_dev->file, pdf_dev->strm_buf, 512);
@@ -1302,7 +1390,7 @@ PCLm_downscale_and_print_page(gx_device_printer *dev, int factor,
     stream_puts(pdf_dev->strm, Buffer);
     stream_puts(pdf_dev->strm, "/Resources <<\n/XObject <<\n");
 
-    if (gp_fseek_64(pdf_dev->temp_stream.file, 0, SEEK_SET) != 0) {
+    if (gp_fseek(pdf_dev->temp_stream.file, 0, SEEK_SET) != 0) {
         gs_free_object(dev->memory, data, "pdf_image_print_page(data)");
         gx_downscaler_fin(&ds);
         return_error(gs_error_ioerror);
@@ -1324,7 +1412,7 @@ PCLm_downscale_and_print_page(gx_device_printer *dev, int factor,
         pprintd2(pdf_dev->strm, "/Im%d %d 0 R\n", row, page->ImageObjectNumber + (row * 2));
     }
     sflush(pdf_dev->temp_stream.strm);
-    stream_pos = gp_ftell_64(pdf_dev->temp_stream.file);
+    stream_pos = gp_ftell(pdf_dev->temp_stream.file);
     stream_puts(pdf_dev->strm, ">>\n>>\n>>\nendobj\n");
 
     page->PageStreamOffset = stell(pdf_dev->strm);
@@ -1332,7 +1420,7 @@ PCLm_downscale_and_print_page(gx_device_printer *dev, int factor,
     pprintd1(pdf_dev->strm, "%d 0 obj\n", page->PageStreamObjectNumber);
     pprintd1(pdf_dev->strm, "<<\n/Length %d\n>>\nstream\n", stream_pos);
 
-    if (gp_fseek_64(pdf_dev->temp_stream.file, 0, SEEK_SET) != 0) {
+    if (gp_fseek(pdf_dev->temp_stream.file, 0, SEEK_SET) != 0) {
         gs_free_object(dev->memory, data, "pdf_image_print_page(data)");
         gx_downscaler_fin(&ds);
         return_error(gs_error_ioerror);
@@ -1354,7 +1442,7 @@ PCLm_downscale_and_print_page(gx_device_printer *dev, int factor,
 
     stream_puts(pdf_dev->strm, "endstream\nendobj\n");
 
-    if (gp_fseek_64(pdf_dev->temp_stream.file, 0, SEEK_SET) != 0) {
+    if (gp_fseek(pdf_dev->temp_stream.file, 0, SEEK_SET) != 0) {
         gs_free_object(dev->memory, data, "pdf_image_print_page(data)");
         gx_downscaler_fin(&ds);
         return_error(gs_error_ioerror);
@@ -1388,7 +1476,7 @@ PCLm_downscale_and_print_page(gx_device_printer *dev, int factor,
             if (pdf_dev->temp_stream.save != pdf_dev->temp_stream.strm)
                 s_close_filters(&pdf_dev->temp_stream.strm, pdf_dev->temp_stream.save);
             sflush(pdf_dev->temp_stream.strm);
-            stream_pos = gp_ftell_64(pdf_dev->temp_stream.file);
+            stream_pos = gp_ftell(pdf_dev->temp_stream.file);
 
             page->ImageOffset = stell(pdf_dev->strm);
             write_xref_entry(pdf_dev->xref_stream.strm, page->ImageOffset);
@@ -1418,7 +1506,7 @@ PCLm_downscale_and_print_page(gx_device_printer *dev, int factor,
                     stream_puts(pdf_dev->strm, ">>\nstream\n");
                     break;
             }
-            if (gp_fseek_64(pdf_dev->temp_stream.file, 0, SEEK_SET) != 0) {
+            if (gp_fseek(pdf_dev->temp_stream.file, 0, SEEK_SET) != 0) {
                 gs_free_object(dev->memory, data, "pdf_image_print_page(data)");
                 gx_downscaler_fin(&ds);
                 return_error(gs_error_ioerror);
@@ -1440,7 +1528,7 @@ PCLm_downscale_and_print_page(gx_device_printer *dev, int factor,
 
             stream_puts(pdf_dev->strm, "\nendstream\nendobj\n");
 
-            if (gp_fseek_64(pdf_dev->temp_stream.file, 0, SEEK_SET) != 0) {
+            if (gp_fseek(pdf_dev->temp_stream.file, 0, SEEK_SET) != 0) {
                 gs_free_object(dev->memory, data, "pdf_image_print_page(data)");
                 gx_downscaler_fin(&ds);
                 return_error(gs_error_ioerror);
@@ -1488,7 +1576,7 @@ PCLm_downscale_and_print_page(gx_device_printer *dev, int factor,
         if (pdf_dev->temp_stream.save != pdf_dev->temp_stream.strm)
             s_close_filters(&pdf_dev->temp_stream.strm, pdf_dev->temp_stream.save);
         sflush(pdf_dev->temp_stream.strm);
-        stream_pos = gp_ftell_64(pdf_dev->temp_stream.file);
+        stream_pos = gp_ftell(pdf_dev->temp_stream.file);
 
         page->ImageOffset = stell(pdf_dev->strm);
         write_xref_entry(pdf_dev->xref_stream.strm, page->ImageOffset);
@@ -1518,7 +1606,7 @@ PCLm_downscale_and_print_page(gx_device_printer *dev, int factor,
                 stream_puts(pdf_dev->strm, ">>\nstream\n");
                 break;
         }
-        if (gp_fseek_64(pdf_dev->temp_stream.file, 0, SEEK_SET) != 0) {
+        if (gp_fseek(pdf_dev->temp_stream.file, 0, SEEK_SET) != 0) {
             gs_free_object(dev->memory, data, "pdf_image_print_page(data)");
             gx_downscaler_fin(&ds);
             return_error(gs_error_ioerror);
@@ -1539,7 +1627,7 @@ PCLm_downscale_and_print_page(gx_device_printer *dev, int factor,
 
         stream_puts(pdf_dev->strm, "\nendstream\nendobj\n");
 
-        if (gp_fseek_64(pdf_dev->temp_stream.file, 0, SEEK_SET) != 0) {
+        if (gp_fseek(pdf_dev->temp_stream.file, 0, SEEK_SET) != 0) {
             gs_free_object(dev->memory, data, "pdf_image_print_page(data)");
             gx_downscaler_fin(&ds);
             return_error(gs_error_ioerror);
@@ -1560,7 +1648,7 @@ PCLm_downscale_and_print_page(gx_device_printer *dev, int factor,
 }
 
 static int
-PCLm_print_page(gx_device_printer * pdev, FILE * file)
+PCLm_print_page(gx_device_printer * pdev, gp_file * file)
 {
     gx_device_pdf_image *const pdf_dev = (gx_device_pdf_image *)pdev;
     const char *fmt;
